@@ -41,142 +41,79 @@
 
 #include "ikec.h"
 
-IKEC ikec;
-
 int main( int argc, char ** argv )
 {
+	IKEC ikec;
+
 	signal( SIGPIPE, SIG_IGN );
 
-	// init the app
-
-	QApplication a( argc, argv );
-
-	// create our root window
-
-	root w;
-
-	// init our ikec object
-
-	ikec.init( &w );
+	ikec.log( 0,
+		"## : VPN Connect, ver %d.%d.%d\n"
+		"## : Copyright %i Shrew Soft Inc.\n"
+		"## : press the <h> key for help\n",
+		CLIENT_VER_MAJ,
+		CLIENT_VER_MIN,
+		CLIENT_VER_BLD,
+		CLIENT_YEAR );
 
 	// read our command line args
 
-	bool syntax_error = false;
-	bool auto_connect = false;
-
-	for( int argi = 0; argi < argc; argi++ )
+	if( ikec.read_opts( argc, argv ) != OPT_RESULT_SUCCESS )
 	{
-		// remote site name
+		ikec.show_help();
+		return -1;
+	}
 
-		if( !strcmp( argv[ argi ], "-r" ) )
+	// load our site configuration
+
+	if( ikec.config_load() )
+	{
+		// autoconnect if requested
+
+		if( ikec.auto_connect() )
+			ikec.vpn_connect( true );
+	}
+
+	// process user input
+
+	bool exit = false;
+
+	while( !exit )
+	{
+		char next;
+		if( !ikec.read_key( next ) )
+			next = 'q';
+
+		switch( next )
 		{
-			if( ++argi >= argc )
-			{
-				syntax_error = true;
+			case 'c': // <c> connect
+				ikec.vpn_connect( true );
 				break;
-			}
 
-			ikec.file_spec( argv[ argi ] );
-			continue;
-		}
-
-		// remote site username
-
-		if( !strcmp( argv[ argi ], "-u" ) )
-		{
-			if( ++argi >= argc )
-			{
-				syntax_error = true;
+			case 'd': // <d> disconnect
+				ikec.vpn_disconnect();
 				break;
-			}
 
-			w.lineEditUsername->setText( argv[ argi ] );
-			continue;
-		}
-
-		// remote site password
-
-		if( !strcmp( argv[ argi ], "-p" ) )
-		{
-			if( ++argi >= argc )
-			{
-				syntax_error = true;
+			case 'h': // <h> help
+			case '?': // <?> help
+				ikec.log( 0, "%s",
+					"Use the following keys to control client connectivity\n"
+					" - : <c> connect\n"
+					" - : <d> disconnect\n"
+					" - : <h> help\n"
+					" - : <s> status\n"
+					" - : <q> quit\n" );
 				break;
-			}
 
-			w.lineEditPassword->setText( argv[ argi ] );
-			syntax_error = false;
-			
-			continue;
-		}
+			case 'q': // <q> quit
+				exit = true;
+				break;
 
-		// auto connect
-
-		if( !strcmp( argv[ argi ], "-a" ) )
-		{
-			auto_connect = true;
-			continue;
+			case 's': // <s> status
+				ikec.show_stats();
+				break;
 		}
 	}
 
-	if( syntax_error )
-	{
-		ikec.log( STATUS_FAIL,
-			"invalid parameters specified ...\n" );
-
-		ikec.log( STATUS_INFO,
-			"ikec -r \"name\" [ -u <user> ][ -p <pass> ][ -a ]\n"
-			"  -r\tsite configuration path\n"
-			"  -u\tconnection user name\n"
-			"  -p\tconnection user password\n"
-			"  -a\tauto connect\n" );
-
-		w.pushButtonConnect->setHidden( true );
-		w.groupBoxCredentials->setHidden( true );
-	}
-	else
-	{
-		// load site config
-
-		if( ikec.config.file_read( ikec.file_path() ) )
-		{
-			// config loaded
-
-			ikec.log( STATUS_INFO, "config loaded for site \'%s\'\n",
-				ikec.file_spec() );
-		}
-		else
-		{
-			// config load failed
-
-			ikec.log( STATUS_INFO, "failed to load \'%s\'\n",
-				ikec.file_spec() );
-
-			w.pushButtonConnect->setHidden( true );
-			w.groupBoxCredentials->setHidden( true );
-		}
-
-		// hide the credentials group
-		// if the autentication method
-		// does not require xauth
-
-		char auth_method[ 64 ] = { 0 };
-		ikec.config.get_string( "auth-method", auth_method, 63, 0 );
-
-		if( strstr( auth_method, "xauth" ) == NULL )
-			w.groupBoxCredentials->setHidden( true );
-	}
-
-	// show the root window
-
-	w.show();
-
-	// auto connect if requested
-
-	if( auto_connect )
-		w.SiteConnect();
-
-	a.connect( &a, SIGNAL( lastWindowClosed() ), &a, SLOT( quit() ) );
-
-	return a.exec();
+	return 0;
 }
