@@ -120,7 +120,7 @@ long _IKED::process_config_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 
 	if( cfg->status() >= XCH_STATUS_MATURE )
 	{
-		log.txt( LLOG_ERROR, "!! : config packet ignored, ( config already mature )\n" );
+		log.txt( LLOG_ERROR, "!! : config packet ignored ( config already mature )\n" );
 		cfg->dec( true );
 		return LIBIKE_OK;
 	}
@@ -427,8 +427,11 @@ long _IKED::process_config_send( IDB_PH1 * ph1, IDB_CFG * cfg )
 	{
 		if( cfg->tunnel->peer->xconf_mode != CONFIG_MODE_DHCP )
 		{
-			cfg->tunnel->tstate |= TSTATE_VNET_CONFIG;
-			cfg->tunnel->ikei->wakeup();
+			if( !( cfg->tunnel->tstate & TSTATE_VNET_CONFIG ) )
+			{
+				cfg->tunnel->tstate |= TSTATE_VNET_CONFIG;
+				cfg->tunnel->ikei->wakeup();
+			}
 		}
 
 		cfg->status( XCH_STATUS_MATURE, XCH_NORMAL, 0 );
@@ -518,7 +521,7 @@ bool _IKED::config_client_xauth_recv( IDB_CFG * cfg, IDB_PH1 * ph1 )
 					break;
 
 				default:
-					log.txt( LLOG_INFO, "!! : warning, unhandled xauth attribute %i\n", attr->atype );
+					log.txt( LLOG_INFO, "ww : unhandled xauth attribute %i\n", attr->atype );
 					break;
 			}
 		}
@@ -855,7 +858,7 @@ bool _IKED::config_client_xauth_send( IDB_CFG * cfg, IDB_PH1 * ph1 )
 
 		cfg->mtype = ISAKMP_CFG_ACK;
 
- 		//
+		//
 		// add the status attribute
 		//
 
@@ -896,12 +899,18 @@ bool _IKED::config_client_xconf_pull_recv( IDB_CFG * cfg, IDB_PH1 * ph1 )
 
 		log.txt( LLOG_INFO, "ii : received config pull response\n" );
 
-		long getmask = 0;
+		long getbits = 0;
 		config_xconf_get( cfg,
-			getmask,
+			getbits,
 			cfg->tunnel->xconf.rqst,
 			ph1->vendopts_r );
 
+		//
+		// add negotiated options
+		//
+
+		cfg->tunnel->xconf.opts |= cfg->tunnel->xconf.rqst & getbits;
+
 		cfg->xstate |= CSTATE_RECV_XCONF;
 
 		return false;
@@ -987,12 +996,18 @@ bool _IKED::config_client_xconf_push_recv( IDB_CFG * cfg, IDB_PH1 * ph1 )
 
 		log.txt( LLOG_INFO, "ii : received config push request\n" );
 
-		long getmask = 0;
+		long getbits = 0;
 		config_xconf_get( cfg,
-			getmask,
+			getbits,
 			cfg->tunnel->xconf.rqst,
 			ph1->vendopts_r );
 
+		//
+		// add negotiated options
+		//
+
+		cfg->tunnel->xconf.opts |= cfg->tunnel->xconf.rqst & getbits;
+
 		//
 		// config is now mature
 		//
@@ -1408,7 +1423,7 @@ bool _IKED::config_server_xconf_push_send( IDB_CFG * cfg, IDB_PH1 * ph1 )
 	return false;
 }
 
-long _IKED::config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask, VENDOPTS vendopts )
+long _IKED::config_xconf_set( IDB_CFG * cfg, long setbits, long setmask, VENDOPTS vendopts )
 {
 	//
 	// the modecfg draft defines valid lengths
@@ -1433,9 +1448,9 @@ long _IKED::config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask, VEND
 	// standard attributes
 	//
 
-	if( setmask & IPSEC_OPTS_ADDR )
+	if( setbits & IPSEC_OPTS_ADDR )
 	{
-		if( nullmask & IPSEC_OPTS_ADDR )
+		if( setmask & IPSEC_OPTS_ADDR )
 		{
 			cfg->attr_add_v( INTERNAL_IP4_ADDRESS,
 				null_ptr, null_len );
@@ -1470,14 +1485,14 @@ long _IKED::config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask, VEND
 		}
 	}
 
-	if( setmask & IPSEC_OPTS_MASK )
+	if( setbits & IPSEC_OPTS_MASK )
 	{
-		if( nullmask & IPSEC_OPTS_MASK )
+		if( setmask & IPSEC_OPTS_MASK )
 		{
 			cfg->attr_add_v( INTERNAL_IP4_NETMASK,
 				null_ptr, null_len );
 
-			log.txt( LLOG_DEBUG,	"ii : - IP4 Netamask\n" );
+			log.txt( LLOG_DEBUG,	"ii : - IP4 Netmask\n" );
 		}
 		else
 		{
@@ -1489,14 +1504,14 @@ long _IKED::config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask, VEND
 			text_addr( txtaddr, cfg->tunnel->xconf.mask );
 
 			log.txt( LLOG_DEBUG,
-				"ii : - IP4 Netamask = %s\n",
+				"ii : - IP4 Netmask = %s\n",
 				txtaddr );
 		}
 	}
 
-	if( setmask & IPSEC_OPTS_DNSS )
+	if( setbits & IPSEC_OPTS_DNSS )
 	{
-		if( nullmask & IPSEC_OPTS_DNSS )
+		if( setmask & IPSEC_OPTS_DNSS )
 		{
 			cfg->attr_add_v( INTERNAL_IP4_DNS,
 				null_ptr, null_len );
@@ -1523,9 +1538,9 @@ long _IKED::config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask, VEND
 		}
 	}
 
-	if( setmask & IPSEC_OPTS_NBNS )
+	if( setbits & IPSEC_OPTS_NBNS )
 	{
-		if( nullmask & IPSEC_OPTS_NBNS )
+		if( setmask & IPSEC_OPTS_NBNS )
 		{
 			cfg->attr_add_v( INTERNAL_IP4_NBNS,
 				null_ptr, null_len );
@@ -1558,9 +1573,9 @@ long _IKED::config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask, VEND
 
 	if( !vendopts.flag.unity )
 	{
-		if( setmask & IPSEC_OPTS_SPLITNET )
+		if( setbits & IPSEC_OPTS_SPLITNET )
 		{
-			if( nullmask & IPSEC_OPTS_SPLITNET )
+			if( setmask & IPSEC_OPTS_SPLITNET )
 			{
 				cfg->attr_add_v( INTERNAL_IP4_SUBNET, NULL, 0 );
 
@@ -1613,9 +1628,9 @@ long _IKED::config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask, VEND
 
 	if( vendopts.flag.unity )
 	{
-		if( setmask & IPSEC_OPTS_DOMAIN )
+		if( setbits & IPSEC_OPTS_DOMAIN )
 		{
-			if( nullmask & IPSEC_OPTS_DOMAIN )
+			if( setmask & IPSEC_OPTS_DOMAIN )
 			{
 				cfg->attr_add_v( UNITY_DEF_DOMAIN, NULL, 0 );
 				log.txt( LLOG_DEBUG,
@@ -1633,9 +1648,9 @@ long _IKED::config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask, VEND
 			}
 		}
 
-		if( setmask & IPSEC_OPTS_SPLITDNS )
+		if( setbits & IPSEC_OPTS_SPLITDNS )
 		{
-			if( nullmask & IPSEC_OPTS_SPLITDNS )
+			if( setmask & IPSEC_OPTS_SPLITDNS )
 			{
 				cfg->attr_add_v( UNITY_SPLIT_DOMAIN, NULL, 0 );
 				log.txt( LLOG_DEBUG, "ii : - Split DNS Domain\n" );
@@ -1662,9 +1677,9 @@ long _IKED::config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask, VEND
 			}
 		}
 
-		if( setmask & IPSEC_OPTS_SPLITNET )
+		if( setbits & IPSEC_OPTS_SPLITNET )
 		{
-			if( nullmask & IPSEC_OPTS_SPLITNET )
+			if( setmask & IPSEC_OPTS_SPLITNET )
 			{
 				cfg->attr_add_v( UNITY_SPLIT_INCLUDE, NULL, 0 );
 				cfg->attr_add_v( UNITY_SPLIT_EXCLUDE, NULL, 0 );
@@ -1747,9 +1762,9 @@ long _IKED::config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask, VEND
 			}
 		}
 
-		if( setmask & IPSEC_OPTS_BANNER )
+		if( setbits & IPSEC_OPTS_BANNER )
 		{
-			if( nullmask & IPSEC_OPTS_BANNER )
+			if( setmask & IPSEC_OPTS_BANNER )
 			{
 				cfg->attr_add_v( UNITY_BANNER, NULL, 0 );
 				log.txt( LLOG_DEBUG,
@@ -1769,9 +1784,9 @@ long _IKED::config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask, VEND
 			}
 		}
 
-		if( setmask & IPSEC_OPTS_PFS )
+		if( setbits & IPSEC_OPTS_PFS )
 		{
-			if( nullmask & IPSEC_OPTS_PFS )
+			if( setmask & IPSEC_OPTS_PFS )
 			{
 				cfg->attr_add_v( UNITY_PFS, NULL, 0 );
 				log.txt( LLOG_DEBUG,
@@ -1788,9 +1803,9 @@ long _IKED::config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask, VEND
 			}
 		}
 
-		if( setmask & IPSEC_OPTS_SAVEPW )
+		if( setbits & IPSEC_OPTS_SAVEPW )
 		{
-			if( nullmask & IPSEC_OPTS_SAVEPW )
+			if( setmask & IPSEC_OPTS_SAVEPW )
 			{
 				cfg->attr_add_v( UNITY_SAVE_PASSWD, NULL, 0 );
 				log.txt( LLOG_DEBUG,
@@ -1807,6 +1822,25 @@ long _IKED::config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask, VEND
 			}
 		}
 
+		if( setbits & IPSEC_OPTS_CISCO_UDP )
+		{
+			if( setmask & IPSEC_OPTS_CISCO_UDP )
+			{
+				cfg->attr_add_v( UNITY_NATT_PORT, NULL, 0 );
+				log.txt( LLOG_DEBUG,
+					"ii : - CISCO UDP Port\n" );
+			}
+			else
+			{
+				cfg->attr_add_b( UNITY_NATT_PORT,
+					cfg->tunnel->peer->natt_port );
+
+				log.txt( LLOG_DEBUG,
+					"ii : - CISCO UDP Port = %i\n",
+					ntohs( cfg->tunnel->peer->natt_port ) );
+			}
+		}
+
 		cfg->attr_add_v( APPLICATION_VERSION,
 			UNITY_APP_VERSION_STRING,
 			strlen( UNITY_APP_VERSION_STRING ) );
@@ -1848,9 +1882,9 @@ long _IKED::config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask, VEND
 			macaddr[ 4 ],
 			macaddr[ 5 ] );
 
-		if( setmask & IPSEC_OPTS_DOMAIN )
+		if( setbits & IPSEC_OPTS_DOMAIN )
 		{
-			if( nullmask & IPSEC_OPTS_DOMAIN )
+			if( setmask & IPSEC_OPTS_DOMAIN )
 			{
 				cfg->attr_add_v( CHKPT_DEF_DOMAIN,
 					null_ptr, null_len );
@@ -1874,7 +1908,7 @@ long _IKED::config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask, VEND
 	return LIBIKE_OK;
 }
 
-long _IKED::config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask, VENDOPTS vendopts )
+long _IKED::config_xconf_get( IDB_CFG * cfg, long & getbits, long getmask, VENDOPTS vendopts )
 {
 	long count = cfg->attr_count();
 	long index = 0;
@@ -1893,9 +1927,9 @@ long _IKED::config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask, VEND
 		{
 			case INTERNAL_IP4_ADDRESS:
 			{
-				getmask |= IPSEC_OPTS_ADDR;
+				getbits |= IPSEC_OPTS_ADDR;
 
-				if( ( readmask & IPSEC_OPTS_ADDR ) && attr->vdata.size() )
+				if( ( getmask & IPSEC_OPTS_ADDR ) && attr->vdata.size() )
 				{
 					if( attr->vdata.size() != 4 )
 					{
@@ -1918,16 +1952,16 @@ long _IKED::config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask, VEND
 						txtaddr );
 				}
 				else
-					log.txt( LLOG_DEBUG,	"ii : - IP4 Address\n" );
+					log.txt( LLOG_DEBUG, "ii : - IP4 Address\n" );
 
 				break;
 			}
 
 			case INTERNAL_IP4_NETMASK:
 			{
-				getmask |= IPSEC_OPTS_MASK;
+				getbits |= IPSEC_OPTS_MASK;
 
-				if( ( readmask & IPSEC_OPTS_MASK ) && attr->vdata.size() )
+				if( ( getmask & IPSEC_OPTS_MASK ) && attr->vdata.size() )
 				{
 					if( attr->vdata.size() != 4 )
 					{
@@ -1950,16 +1984,16 @@ long _IKED::config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask, VEND
 						txtaddr );
 				}
 				else
-					log.txt( LLOG_DEBUG,	"ii : - IP4 Netmask\n" );
+					log.txt( LLOG_DEBUG, "ii : - IP4 Netmask\n" );
 
 				break;
 			}
 
 			case INTERNAL_IP4_DNS:
 			{
-				getmask |= IPSEC_OPTS_DNSS;
+				getbits |= IPSEC_OPTS_DNSS;
 
-				if( ( readmask & IPSEC_OPTS_DNSS ) && attr->vdata.size() )
+				if( ( getmask & IPSEC_OPTS_DNSS ) && attr->vdata.size() )
 				{
 					if( cfg->tunnel->xconf.nscfg.dnss_count < IPSEC_DNSS_MAX )
 					{
@@ -1994,9 +2028,9 @@ long _IKED::config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask, VEND
 
 			case INTERNAL_IP4_NBNS:
 			{
-				getmask |= IPSEC_OPTS_NBNS;
+				getbits |= IPSEC_OPTS_NBNS;
 
-				if( ( readmask & IPSEC_OPTS_NBNS ) && attr->vdata.size() )
+				if( ( getmask & IPSEC_OPTS_NBNS ) && attr->vdata.size() )
 				{
 					if( cfg->tunnel->xconf.nscfg.nbns_count < IPSEC_NBNS_MAX )
 					{
@@ -2031,9 +2065,9 @@ long _IKED::config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask, VEND
 
 			case INTERNAL_ADDRESS_EXPIRY:
 			{
-				getmask |= IPSEC_OPTS_ADDR;
+				getbits |= IPSEC_OPTS_ADDR;
 
-				if( ( readmask & IPSEC_OPTS_ADDR ) && attr->vdata.size() )
+				if( ( getmask & IPSEC_OPTS_ADDR ) && attr->vdata.size() )
 				{
 					if( attr->vdata.size() != 4 )
 					{
@@ -2092,9 +2126,9 @@ long _IKED::config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask, VEND
 			{
 				case INTERNAL_IP4_SUBNET:
 				{
-					getmask |= IPSEC_OPTS_SPLITNET;
+					getbits |= IPSEC_OPTS_SPLITNET;
 
-					if( ( readmask & IPSEC_OPTS_SPLITNET ) && attr->vdata.size() )
+					if( ( getmask & IPSEC_OPTS_SPLITNET ) && attr->vdata.size() )
 					{
 						int net_count = int( attr->vdata.size() / sizeof( IKE_SUBNET ) );
 						int net_index = 0;
@@ -2152,25 +2186,11 @@ long _IKED::config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask, VEND
 
 			switch( attr->atype )
 			{
-				case UNITY_FW_TYPE:
-				{
-					if( attr->vdata.size() )
-					{
-						log.txt( LLOG_DEBUG,
-							"ii : - Firewall Type = %i bytes\n",
-							attr->vdata.size() );
-					}
-					else
-						log.txt( LLOG_DEBUG, "ii : - Firewall Type\n" );
-
-					break;
-				}
-
 				case UNITY_DEF_DOMAIN:
 				{
-					getmask |= IPSEC_OPTS_DOMAIN;
+					getbits |= IPSEC_OPTS_DOMAIN;
 
-					if( ( readmask & IPSEC_OPTS_DOMAIN ) && attr->vdata.size() )
+					if( ( getmask & IPSEC_OPTS_DOMAIN ) && attr->vdata.size() )
 					{
 						size_t nlen = attr->vdata.size();
 						if( nlen > ( CONF_STRLEN - 1 ) )
@@ -2194,9 +2214,9 @@ long _IKED::config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask, VEND
 
 				case UNITY_SPLIT_DOMAIN:
 				{
-					getmask |= IPSEC_OPTS_SPLITDNS;
+					getbits |= IPSEC_OPTS_SPLITDNS;
 
-					if( ( readmask & IPSEC_OPTS_SPLITDNS ) && attr->vdata.size() )
+					if( ( getmask & IPSEC_OPTS_SPLITDNS ) && attr->vdata.size() )
 					{
 						attr->vdata.add( 0, 1 );
 
@@ -2223,7 +2243,7 @@ long _IKED::config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask, VEND
 							dnslen += tmplen;
 							dnsstr += tmplen;
 
-							if( readmask & IPSEC_OPTS_SPLITDNS )
+							if( getmask & IPSEC_OPTS_SPLITDNS )
 								cfg->tunnel->domains.add( domain );
 						}
 					}
@@ -2236,9 +2256,9 @@ long _IKED::config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask, VEND
 				case UNITY_SPLIT_INCLUDE:
 				case UNITY_SPLIT_EXCLUDE:
 				{
-					getmask |= IPSEC_OPTS_SPLITNET;
+					getbits |= IPSEC_OPTS_SPLITNET;
 
-					if( ( readmask & IPSEC_OPTS_SPLITNET ) && attr->vdata.size() )
+					if( ( getmask & IPSEC_OPTS_SPLITNET ) && attr->vdata.size() )
 					{
 						int net_count = int( attr->vdata.size() / sizeof( IKE_UNITY_NET ) );
 						int net_index = 0;
@@ -2325,9 +2345,9 @@ long _IKED::config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask, VEND
 
 				case UNITY_BANNER:
 				{
-					getmask |= IPSEC_OPTS_BANNER;
+					getbits |= IPSEC_OPTS_BANNER;
 
-					if( ( readmask & IPSEC_OPTS_BANNER ) && attr->vdata.size() )
+					if( ( getmask & IPSEC_OPTS_BANNER ) && attr->vdata.size() )
 					{
 						cfg->tunnel->banner.add( 0, 1 );
 
@@ -2346,16 +2366,16 @@ long _IKED::config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask, VEND
 						cfg->tunnel->banner.add( 0, 1 );
 					}
 					else
-						log.txt( LLOG_DEBUG,	"ii : - Login Banner\n" );
+						log.txt( LLOG_DEBUG, "ii : - Login Banner\n" );
 
 					break;
 				}
 
 				case UNITY_PFS:
 				{
-					getmask |= IPSEC_OPTS_PFS;
+					getbits |= IPSEC_OPTS_PFS;
 
-					if( ( readmask & IPSEC_OPTS_PFS ) && attr->basic )
+					if( ( getmask & IPSEC_OPTS_PFS ) && attr->basic )
 					{
 						log.txt( LLOG_DEBUG,
 							"ii : - PFS Group = %d\n",
@@ -2364,16 +2384,16 @@ long _IKED::config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask, VEND
 						cfg->tunnel->xconf.dhgr = attr->bdata;
 					}
 					else
-						log.txt( LLOG_DEBUG,	"ii : - PFS Group\n" );
+						log.txt( LLOG_DEBUG, "ii : - PFS Group\n" );
 
 					break;
 				}
 
 				case UNITY_SAVE_PASSWD:
 				{
-					getmask |= IPSEC_OPTS_SAVEPW;
+					getbits |= IPSEC_OPTS_SAVEPW;
 
-					if( ( readmask & IPSEC_OPTS_SAVEPW ) && attr->basic )
+					if( ( getmask & IPSEC_OPTS_SAVEPW ) && attr->basic )
 					{
 						log.txt( LLOG_DEBUG,
 							"ii : - Save Password = %d\n",
@@ -2382,7 +2402,49 @@ long _IKED::config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask, VEND
 						cfg->tunnel->xconf.svpw = attr->bdata;
 					}
 					else
-						log.txt( LLOG_DEBUG,	"ii : - Save Password\n" );
+						log.txt( LLOG_DEBUG, "ii : - Save Password\n" );
+
+					break;
+				}
+
+				case UNITY_NATT_PORT:
+				{
+					getbits |= IPSEC_OPTS_CISCO_UDP;
+
+					if( ( getmask & IPSEC_OPTS_CISCO_UDP ) && attr->basic )
+					{
+						log.txt( LLOG_DEBUG,
+							"ii : - Cisco UDP Port = %d\n",
+							attr->bdata );
+
+						if( cfg->tunnel->natt_version == IPSEC_NATT_NONE )
+						{
+							cfg->tunnel->natt_version = IPSEC_NATT_CISCO;
+							cfg->tunnel->peer->natt_port = htons( attr->bdata );
+
+							cfg->tunnel->inc( true );
+							cfg->tunnel->event_natt.delay = cfg->tunnel->peer->natt_rate * 1000;
+							ith_timer.add( &cfg->tunnel->event_natt );
+
+							log.txt( LLOG_INFO, "ii : switching nat-t to cisco-udp\n" );
+						}
+					}
+					else
+						log.txt( LLOG_DEBUG, "ii : - Cisco UDP Port\n" );
+
+					break;
+				}
+
+				case UNITY_FW_TYPE:
+				{
+					if( attr->vdata.size() )
+					{
+						log.txt( LLOG_DEBUG,
+							"ii : - Firewall Type = %i bytes\n",
+							attr->vdata.size() );
+					}
+					else
+						log.txt( LLOG_DEBUG, "ii : - Firewall Type\n" );
 
 					break;
 				}
@@ -2420,11 +2482,14 @@ long _IKED::config_chk_hash( IDB_PH1 * ph1, IDB_CFG * cfg, unsigned long msgid )
 	hash_c.size( ph1->hash_size );
 
 	HMAC_CTX ctx_prf;
-	HMAC_Init( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash );
+	HMAC_CTX_init( &ctx_prf );
+
+	HMAC_Init_ex( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash, NULL );
 	HMAC_Update( &ctx_prf, ( unsigned char * ) &msgid, 4 );
 	HMAC_Update( &ctx_prf, cfg->hda.buff(), cfg->hda.size() );
 	HMAC_Final( &ctx_prf, hash_c.buff(), NULL );
-	HMAC_cleanup( &ctx_prf );
+
+	HMAC_CTX_cleanup( &ctx_prf );
 
 	log.bin(
 		LLOG_DEBUG,
@@ -2440,7 +2505,7 @@ long _IKED::config_chk_hash( IDB_PH1 * ph1, IDB_CFG * cfg, unsigned long msgid )
 		hash_c.size(),
 		"== : configure hash_c ( computed )" );
 
-	if( memcmp( cfg->hash_r.buff(), hash_c.buff(), hash_c.size() ) )
+	if( cfg->hash_r != hash_c )
 	{
 		log.txt( LLOG_ERROR,	"!! : configure hash verification failed\n" );
 		return LIBIKE_FAILED;
@@ -2479,11 +2544,14 @@ long _IKED::config_message_send( IDB_PH1 * ph1, IDB_CFG * cfg )
 	//
 
 	HMAC_CTX ctx_prf;
-	HMAC_Init( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash );
+	HMAC_CTX_init( &ctx_prf );
+
+	HMAC_Init_ex( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash, NULL );
 	HMAC_Update( &ctx_prf, ( unsigned char * ) &cfg->msgid, sizeof( cfg->msgid ) );
 	HMAC_Update( &ctx_prf, packet.buff() + beg, end - beg );
 	HMAC_Final( &ctx_prf, hash.buff(), 0 );
-	HMAC_cleanup( &ctx_prf );
+
+	HMAC_CTX_cleanup( &ctx_prf );
 
 	memcpy( packet.buff() + off + 4, hash.buff(), hash.size() );
 
