@@ -400,11 +400,14 @@ long _IKED::inform_chk_hash( IDB_PH1 * ph1, IDB_XCH * inform )
 	hash_c.size( ph1->hash_size );
 
 	HMAC_CTX ctx_prf;
-	HMAC_Init( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash );
+	HMAC_CTX_init( &ctx_prf );
+
+	HMAC_Init_ex( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash, NULL );
 	HMAC_Update( &ctx_prf, ( unsigned char * ) &inform->msgid, 4 );
 	HMAC_Update( &ctx_prf, inform->hda.buff(), inform->hda.size() );
 	HMAC_Final( &ctx_prf, hash_c.buff(), NULL );
-	HMAC_cleanup( &ctx_prf );
+
+	HMAC_CTX_cleanup( &ctx_prf );
 
 	log.bin(
 		LLOG_DEBUG,
@@ -420,7 +423,7 @@ long _IKED::inform_chk_hash( IDB_PH1 * ph1, IDB_XCH * inform )
 		inform->hash_r.size(),
 		"== : informational hash_c ( received )" );
 
-	if( memcmp( inform->hash_r.buff(), hash_c.buff(), hash_c.size() ) )
+	if( inform->hash_r != hash_c )
 	{
 		log.txt( LLOG_ERROR,	"!! : informational hash verification failed\n" );
 
@@ -437,11 +440,14 @@ long _IKED::inform_gen_hash( IDB_PH1 * ph1, IDB_XCH * inform )
 	inform->hash_l.size( ph1->hash_size );
 
 	HMAC_CTX ctx_prf;
-	HMAC_Init( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash );
+	HMAC_CTX_init( &ctx_prf );
+
+	HMAC_Init_ex( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash, NULL );
 	HMAC_Update( &ctx_prf, ( unsigned char * ) &inform->msgid, sizeof( inform->msgid ) );
 	HMAC_Update( &ctx_prf, inform->hda.buff(), inform->hda.size() );
 	HMAC_Final( &ctx_prf, inform->hash_l.buff(), 0 );
-	HMAC_cleanup( &ctx_prf );
+
+	HMAC_CTX_cleanup( &ctx_prf );
 
 	log.bin(
 		LLOG_DEBUG,
@@ -540,10 +546,25 @@ long _IKED::inform_chk_notify( IDB_PH1 * ph1, IKE_NOTIFY * notify, bool secure )
 		// phase1 notify
 		//
 
+		case 0:
+
+			//
+			// NOTE: the Netgear DPD implementation in
+			// badly broken. it sends a protocol type
+			// if ISAKMP and an SPI length of zero.
+			//
+
 		case ( ISAKMP_COOKIE_SIZE * 2 ):
 		{
 			switch( notify->code )
 			{
+				//
+				// NOTE : a cisco device working in a high
+				// availability group will send this notify
+				// message when it would like us to migrate
+				// to a different server.
+				//
+
 				case ISAKMP_N_UNITY_LOAD_BALANCE:
 				{
 					if( notify->data.size() != sizeof( in_addr ) )
@@ -612,7 +633,7 @@ long _IKED::inform_chk_notify( IDB_PH1 * ph1, IKE_NOTIFY * notify, bool secure )
 
 					ph1_ulb->tunnel->stats.dpd = false;
 					ph1_ulb->tunnel->stats.frag = false;
-					ph1_ulb->tunnel->stats.natt = false;
+					ph1_ulb->tunnel->stats.natt = IPSEC_NATT_NONE;
 
 					//
 					// update the peer and tunnel objects to use
@@ -640,14 +661,43 @@ long _IKED::inform_chk_notify( IDB_PH1 * ph1, IKE_NOTIFY * notify, bool secure )
 					break;
 				}
 
+				//
+				// NOTE : the Netgear DPD implementation is
+				// badly broken. they send 16 bytes of zero
+				// padding before the 4 byte sequence value.
+				// if we respond with the full 20 bytes sent
+				// to us, the gateway rejects the value. if
+				// we send only the relevent 4 bytes, the
+				// gateway accepts the value. this is likely
+				// related to the absent ISAKMP cookie value
+				// as the null padding is the same length as
+				// an ISAKMP cookie pair.
+				//
+
 				case ISAKMP_N_DPD_R_U_THERE:
 				{
+					size_t seq_size = sizeof( ph1->tunnel->event_dpd.sequence );
+					size_t seq_padd = notify->data.size() - seq_size;
+
+					if( notify->data.size() < seq_size )
+					{
+						log.txt( LLOG_ERROR, "!! : DPD ARE-YOU-THERE sequence data is invalid ( %i bytes )\n", notify->data.size() );
+						break;
+					}
+
 					uint32_t sequence;
+					notify->data.oset( seq_padd );
 					notify->data.get( &sequence, sizeof( sequence ) );
-					sequence = ntohl( sequence );
+
+					if( seq_padd )
+					{
+						notify->data.del();
+						notify->data.add( &sequence, sizeof( sequence ) );
+					}
 
 					inform_new_notify( ph1, NULL, ISAKMP_N_DPD_R_U_THERE_ACK, &notify->data );
 
+					sequence = ntohl( sequence );
 					log.txt( LLOG_DEBUG, "ii : DPD ARE-YOU-THERE sequence %08x returned\n", sequence );
 
 					break;
@@ -655,36 +705,43 @@ long _IKED::inform_chk_notify( IDB_PH1 * ph1, IKE_NOTIFY * notify, bool secure )
 
 				case ISAKMP_N_DPD_R_U_THERE_ACK:
 				{
-					if( notify->data.size() == sizeof( ph1->tunnel->event_dpd.sequence ) )
+					size_t seq_size = sizeof( ph1->tunnel->event_dpd.sequence );
+					size_t seq_padd = notify->data.size() - seq_size;
+
+					if( notify->data.size() < seq_size )
 					{
-						//
-						// obtain sequence number and
-						// convert to host byte order
-						//
-
-						uint32_t sequence;
-						notify->data.get( &sequence, sizeof( sequence ) );
-						sequence = ntohl( sequence );
-
-						//
-						// check dpd sequence number
-						//
-
-						if( sequence != ph1->tunnel->event_dpd.sequence )
-						{
-							log.txt( LLOG_ERROR, "!! : DPD ARE-YOU-THERE-ACK sequence %08x rejected\n", sequence );
-							break;
-						}
-
-						//
-						// setup the next dpd cycle
-						//
-
-						log.txt( LLOG_DEBUG, "ii : DPD ARE-YOU-THERE-ACK sequence %08x accepted\n", sequence );
-
-						ph1->tunnel->event_dpd.next();
+						log.txt( LLOG_ERROR, "!! : DPD ARE-YOU-THERE-ACK sequence data is invalid ( %i bytes )\n", notify->data.size() );
+						break;
 					}
 
+					//
+					// obtain sequence number and
+					// convert to host byte order
+					//
+
+					uint32_t sequence;
+					notify->data.oset( seq_padd );
+					notify->data.get( &sequence, sizeof( sequence ) );
+
+					//
+					// check dpd sequence number
+					//
+
+					sequence = ntohl( sequence );
+					if( sequence != ph1->tunnel->event_dpd.sequence )
+					{
+						log.txt( LLOG_ERROR, "!! : DPD ARE-YOU-THERE-ACK sequence %08x rejected\n", sequence );
+						break;
+					}
+
+					//
+					// setup the next dpd cycle
+					//
+
+					log.txt( LLOG_DEBUG, "ii : DPD ARE-YOU-THERE-ACK sequence %08x accepted\n", sequence );
+
+					ph1->tunnel->event_dpd.next();
+
 					break;
 				}
 			}
