@@ -560,7 +560,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 				// optionally add nat discovery hash payloads
 				//
 
-				if( ph1->natt_version != IPSEC_NATT_NONE )
+				if( ph1->natt_pldtype != ISAKMP_PAYLOAD_NONE )
 				{
 					phase1_gen_natd( ph1 );
 					phase1_add_natd( ph1, packet, ISAKMP_PAYLOAD_NONE );
@@ -596,7 +596,8 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 				// calculate key material
 				//
 
-				phase1_gen_keys( ph1 );
+				if( phase1_gen_keys( ph1 ) != LIBIKE_OK )
+					return LIBIKE_FAILED;
 
 				//
 				// write packet header
@@ -640,8 +641,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 
 						BDATA sign;
 						phase1_gen_hash_i( ph1, ph1->hash_l );
-						prvkey_rsa_encrypt( ph1->tunnel->peer->key, ph1->hash_l, sign );
-
+						prvkey_rsa_encrypt( ph1->tunnel->peer->cert_k, ph1->hash_l, sign );
 						payload_add_sign( packet, sign, ISAKMP_PAYLOAD_NONE );
 
 						ph1->xstate |= XSTATE_SENT_CT;
@@ -747,7 +747,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 				// optionally add nat discovery hash payloads
 				//
 
-				if( ph1->natt_version != IPSEC_NATT_NONE )
+				if( ph1->natt_pldtype != ISAKMP_PAYLOAD_NONE )
 				{
 					phase1_gen_natd( ph1 );
 					phase1_add_natd( ph1, packet, ISAKMP_PAYLOAD_NONE );
@@ -768,7 +768,8 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 				// calculate key material
 				//
 
-				phase1_gen_keys( ph1 );
+				if( phase1_gen_keys( ph1 ) != LIBIKE_OK )
+					return LIBIKE_FAILED;
 			}
 
 			//
@@ -872,7 +873,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 
 						BDATA sign;
 						phase1_gen_hash_r( ph1, ph1->hash_l );
-						prvkey_rsa_encrypt( ph1->tunnel->peer->key, ph1->hash_l, sign );
+						prvkey_rsa_encrypt( ph1->tunnel->peer->cert_k, ph1->hash_l, sign );
 						payload_add_sign( packet, sign, ISAKMP_PAYLOAD_NONE );
 
 						//
@@ -1003,7 +1004,8 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 							// calculate key material
 							//
 
-							phase1_gen_keys( ph1 );
+							if( phase1_gen_keys( ph1 ) != LIBIKE_OK )
+								break;
 
 							//
 							// write packet header
@@ -1043,10 +1045,13 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 								psk_hash.size( ph1->hash_size );
 
 								HMAC_CTX ctx_prf;
-								HMAC_Init( &ctx_prf, ph1->skeyid.buff(), ( int ) ph1->skeyid.size(), ph1->evp_hash );
+								HMAC_CTX_init( &ctx_prf );
+
+								HMAC_Init_ex( &ctx_prf, ph1->skeyid.buff(), ( int ) ph1->skeyid.size(), ph1->evp_hash, NULL );
 								HMAC_Update( &ctx_prf, ph1->tunnel->peer->psk.buff(), ph1->tunnel->peer->psk.size() );
 								HMAC_Final( &ctx_prf, psk_hash.buff(), NULL );
-								HMAC_cleanup( &ctx_prf );
+
+								HMAC_CTX_cleanup( &ctx_prf );
 
 								//
 								// add the notification payload
@@ -1067,7 +1072,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 							// optionally add nat discovery hash payloads
 							//
 
-							if( ph1->natt_version != IPSEC_NATT_NONE )
+							if( ph1->natt_pldtype != ISAKMP_PAYLOAD_NONE )
 							{
 								phase1_gen_natd( ph1 );
 								phase1_add_natd( ph1, packet, ISAKMP_PAYLOAD_NONE );
@@ -1093,7 +1098,12 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 						if(  ( ph1->xstate & XSTATE_RECV_SI ) &&
 							!( ph1->xstate & XSTATE_SENT_SI ) )
 						{
-							phase1_gen_keys( ph1 );
+							//
+							// calculate key material
+							//
+
+							if( phase1_gen_keys( ph1 ) != LIBIKE_OK )
+								break;
 
 							//
 							// write packet header
@@ -1110,14 +1120,14 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 
 							BDATA sign;
 							phase1_gen_hash_i( ph1, ph1->hash_l );
-							prvkey_rsa_encrypt( ph1->tunnel->peer->key, ph1->hash_l, sign );
+							prvkey_rsa_encrypt( ph1->tunnel->peer->cert_k, ph1->hash_l, sign );
 							payload_add_sign( packet, sign, ph1->natt_pldtype );
 
 							//
 							// optionally add nat discovery hash payloads
 							//
 
-							if( ph1->natt_version != IPSEC_NATT_NONE )
+							if( ph1->natt_pldtype != ISAKMP_PAYLOAD_NONE )
 							{
 								phase1_gen_natd( ph1 );
 								phase1_add_natd( ph1, packet, ISAKMP_PAYLOAD_NONE );
@@ -1158,10 +1168,11 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 				!( ph1->xstate & XSTATE_SENT_SA ) )
 			{
 				//
-				// generate our keys
+				// calculate key material
 				//
 
-				phase1_gen_keys( ph1 );
+				if( phase1_gen_keys( ph1 ) != LIBIKE_OK )
+					return LIBIKE_FAILED;
 
 				//
 				// write packet header
@@ -1214,7 +1225,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 
 						BDATA sign;
 						phase1_gen_hash_r( ph1, ph1->hash_l );
-						prvkey_rsa_encrypt( ph1->tunnel->peer->key, ph1->hash_l, sign );
+						prvkey_rsa_encrypt( ph1->tunnel->peer->cert_k, ph1->hash_l, sign );
 						payload_add_sign( packet, sign, ISAKMP_PAYLOAD_VEND );
 
 						ph1->xstate |= XSTATE_SENT_CT;
@@ -1231,7 +1242,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 				// optionally add nat discovery hash payloads
 				//
 
-				if( ph1->natt_version != IPSEC_NATT_NONE )
+				if( ph1->natt_pldtype != ISAKMP_PAYLOAD_NONE )
 				{
 					phase1_gen_natd( ph1 );
 					phase1_add_natd( ph1, packet, ISAKMP_PAYLOAD_NONE );
@@ -1341,8 +1352,6 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 
 			if( ph1->tunnel->natt_version != IPSEC_NATT_NONE )
 			{
-				ph1->tunnel->stats.natt = true;
-
 				ph1->tunnel->inc( true );
 				ph1->tunnel->event_natt.delay = ph1->tunnel->peer->natt_rate * 1000;
 
@@ -1371,7 +1380,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 			ph1->tunnel->tstate |= TSTATE_INITIALIZED;
 		}
 
- 		//
+		//
 		// determine the policy level if auto
 		//
 
@@ -1648,11 +1657,9 @@ long _IKED::phase1_gen_keys( IDB_PH1 * ph1 )
 		case XAUTH_AUTH_RESP_PSK:
 		{
 			HMAC_CTX ctx_prf;
-			HMAC_Init(
-				&ctx_prf,
-				ph1->tunnel->peer->psk.buff(),
-				( int ) ph1->tunnel->peer->psk.size(),
-				ph1->evp_hash );
+			HMAC_CTX_init( &ctx_prf );
+
+			HMAC_Init_ex( &ctx_prf, ph1->tunnel->peer->psk.buff(), ( int ) ph1->tunnel->peer->psk.size(), ph1->evp_hash, NULL );
 
 			if( ph1->initiator )
 			{
@@ -1666,7 +1673,8 @@ long _IKED::phase1_gen_keys( IDB_PH1 * ph1 )
 			}
 
 			HMAC_Final( &ctx_prf, skeyid_data, NULL );
-			HMAC_cleanup( &ctx_prf );
+
+			HMAC_CTX_cleanup( &ctx_prf );
 
 			break;
 		}
@@ -1697,10 +1705,13 @@ long _IKED::phase1_gen_keys( IDB_PH1 * ph1 )
 			}
 
 			HMAC_CTX ctx_prf;
-			HMAC_Init( &ctx_prf, nonce.buff(), ( int ) nonce.size(), ph1->evp_hash );
+			HMAC_CTX_init( &ctx_prf );
+
+			HMAC_Init_ex( &ctx_prf, nonce.buff(), ( int ) nonce.size(), ph1->evp_hash, NULL );
 			HMAC_Update( &ctx_prf, shared.buff(), shared.size() );
 			HMAC_Final( &ctx_prf, skeyid_data, NULL );
-			HMAC_cleanup( &ctx_prf );
+
+			HMAC_CTX_cleanup( &ctx_prf );
 
 			break;
 		}	
@@ -1720,13 +1731,14 @@ long _IKED::phase1_gen_keys( IDB_PH1 * ph1 )
 	//
 
 	HMAC_CTX ctx_prf;
-	HMAC_Init( &ctx_prf, ph1->skeyid.buff(), ( int ) ph1->skeyid.size(), ph1->evp_hash );
+	HMAC_CTX_init( &ctx_prf );
+
+	HMAC_Init_ex( &ctx_prf, ph1->skeyid.buff(), ( int ) ph1->skeyid.size(), ph1->evp_hash, NULL );
 	HMAC_Update( &ctx_prf, shared.buff(), shared.size() );
 	HMAC_Update( &ctx_prf, ph1->cookies.i, ISAKMP_COOKIE_SIZE );
 	HMAC_Update( &ctx_prf, ph1->cookies.r, ISAKMP_COOKIE_SIZE );
 	HMAC_Update( &ctx_prf, ( unsigned char * ) "\0", 1 );
 	HMAC_Final( &ctx_prf, skeyid_data, NULL );
-	HMAC_cleanup( &ctx_prf );
 
 	ph1->skeyid_d.set( skeyid_data, skeyid_size );
 
@@ -1741,14 +1753,13 @@ long _IKED::phase1_gen_keys( IDB_PH1 * ph1 )
 	// compute SKEYID_a
 	//
 
-	HMAC_Init( &ctx_prf, ph1->skeyid.buff(), ( int ) ph1->skeyid.size(), ph1->evp_hash );
+	HMAC_Init_ex( &ctx_prf, ph1->skeyid.buff(), ( int ) ph1->skeyid.size(), ph1->evp_hash, NULL );
 	HMAC_Update( &ctx_prf, skeyid_data, skeyid_size );
 	HMAC_Update( &ctx_prf, shared.buff(), shared.size() );
 	HMAC_Update( &ctx_prf, ph1->cookies.i, ISAKMP_COOKIE_SIZE );
 	HMAC_Update( &ctx_prf, ph1->cookies.r, ISAKMP_COOKIE_SIZE );
 	HMAC_Update( &ctx_prf, ( unsigned char * ) "\1", 1 );
 	HMAC_Final( &ctx_prf, skeyid_data, NULL );
-	HMAC_cleanup( &ctx_prf );
 
 	ph1->skeyid_a.set( skeyid_data, skeyid_size );
 
@@ -1763,14 +1774,13 @@ long _IKED::phase1_gen_keys( IDB_PH1 * ph1 )
 	// compute SKEYID_e
 	//
 
-	HMAC_Init( &ctx_prf, ph1->skeyid.buff(), ( int ) ph1->skeyid.size(), ph1->evp_hash );
+	HMAC_Init_ex( &ctx_prf, ph1->skeyid.buff(), ( int ) ph1->skeyid.size(), ph1->evp_hash, NULL );
 	HMAC_Update( &ctx_prf, skeyid_data, skeyid_size );
 	HMAC_Update( &ctx_prf, shared.buff(), shared.size() );
 	HMAC_Update( &ctx_prf, ph1->cookies.i, ISAKMP_COOKIE_SIZE );
 	HMAC_Update( &ctx_prf, ph1->cookies.r, ISAKMP_COOKIE_SIZE );
 	HMAC_Update( &ctx_prf, ( unsigned char * ) "\2", 1 );
 	HMAC_Final( &ctx_prf, skeyid_data, NULL );
-	HMAC_cleanup( &ctx_prf );
 
 	ph1->skeyid_e.set( skeyid_data, skeyid_size );
 
@@ -1811,18 +1821,16 @@ long _IKED::phase1_gen_keys( IDB_PH1 * ph1 )
 
 		// create extended key data
 
-		HMAC_Init( &ctx_prf, skeyid_data, skeyid_size, ph1->evp_hash );
+		HMAC_Init_ex( &ctx_prf, skeyid_data, skeyid_size, ph1->evp_hash, NULL );
 		HMAC_Update( &ctx_prf, ( unsigned char * ) "\0", 1 );
 		HMAC_Final( &ctx_prf, key_data, NULL );
 
 		for( long size = skeyid_size; size < key_size; size += skeyid_size )
 		{
-			HMAC_Init( &ctx_prf, skeyid_data, skeyid_size, ph1->evp_hash );
+			HMAC_Init_ex( &ctx_prf, skeyid_data, skeyid_size, ph1->evp_hash, NULL );
 			HMAC_Update( &ctx_prf, key_data + size - skeyid_size, skeyid_size );
 			HMAC_Final( &ctx_prf, key_data + size, NULL );
 		}
-
-		HMAC_cleanup( &ctx_prf );
 	}
 	else
 	{
@@ -1831,6 +1839,8 @@ long _IKED::phase1_gen_keys( IDB_PH1 * ph1 )
 		memcpy( key_data, skeyid_data, key_size );
 	}
 
+	HMAC_CTX_cleanup( &ctx_prf );
+
 	if( proposal->ciph_kl )
 		key_size = ( proposal->ciph_kl + 7 ) / 8;
 
@@ -1894,7 +1904,9 @@ long _IKED::phase1_gen_hash_i( IDB_PH1 * sa, BDATA & hash )
 	hash.size( sa->hash_size );
 
 	HMAC_CTX ctx_prf;
-	HMAC_Init( &ctx_prf, sa->skeyid.buff(), ( int ) sa->skeyid.size(), sa->evp_hash );
+	HMAC_CTX_init( &ctx_prf );
+
+	HMAC_Init_ex( &ctx_prf, sa->skeyid.buff(), ( int ) sa->skeyid.size(), sa->evp_hash, NULL );
 
 	if( sa->initiator )
 	{
@@ -1912,7 +1924,8 @@ long _IKED::phase1_gen_hash_i( IDB_PH1 * sa, BDATA & hash )
 	HMAC_Update( &ctx_prf, sa->hda.buff(), sa->hda.size() );
 	HMAC_Update( &ctx_prf, sa->idi.buff(), sa->idi.size() );
 	HMAC_Final( &ctx_prf, hash.buff(), NULL );
-	HMAC_cleanup( &ctx_prf );
+
+	HMAC_CTX_cleanup( &ctx_prf );
 
 	log.bin(
 		LLOG_DEBUG,
@@ -1933,7 +1946,9 @@ long _IKED::phase1_gen_hash_r( IDB_PH1 * sa, BDATA & hash )
 	hash.size( sa->hash_size );
 
 	HMAC_CTX ctx_prf;
-	HMAC_Init( &ctx_prf, sa->skeyid.buff(), ( int ) sa->skeyid.size(), sa->evp_hash );
+	HMAC_CTX_init( &ctx_prf );
+
+	HMAC_Init_ex( &ctx_prf, sa->skeyid.buff(), ( int ) sa->skeyid.size(), sa->evp_hash, NULL );
 
 	if( sa->initiator )
 	{
@@ -1951,7 +1966,8 @@ long _IKED::phase1_gen_hash_r( IDB_PH1 * sa, BDATA & hash )
 	HMAC_Update( &ctx_prf, sa->hda.buff(), sa->hda.size() );
 	HMAC_Update( &ctx_prf, sa->idr.buff(), sa->idr.size() );
 	HMAC_Final( &ctx_prf, hash.buff(), NULL );
-	HMAC_cleanup( &ctx_prf );
+
+	HMAC_CTX_cleanup( &ctx_prf );
 
 	log.bin(
 		LLOG_DEBUG,
@@ -1998,7 +2014,7 @@ long _IKED::phase1_add_vend( IDB_PH1 * ph1, PACKET_IKE & packet, uint8_t next )
 		vid_count++;
 
 	if( ph1->vendopts_l.flag.dpdv1 )
-		vid_count++;
+		vid_count += 2;
 
 	if( ph1->vendopts_l.flag.ssoft )
 		vid_count++;
@@ -2077,6 +2093,7 @@ long _IKED::phase1_add_vend( IDB_PH1 * ph1, PACKET_IKE & packet, uint8_t next )
 	if( ph1->vendopts_l.flag.dpdv1 )
 	{
 		payload_add_vend( packet, vend_dpd1, vendpld( vid_count, next ) );
+		payload_add_vend( packet, vend_dpd1_ng, vendpld( vid_count, next ) );
 		log.txt( LLOG_INFO, "ii : local supports DPDv1\n" );
 	}
 
@@ -2126,8 +2143,7 @@ long _IKED::phase1_add_vend( IDB_PH1 * ph1, PACKET_IKE & packet, uint8_t next )
 	}
 
 	//
-	// prepair and optionally add checkpoint vendor
-	// id payload ( must be last )
+	// optionally add checkpoint vendor payload ( must be last )
 	//
 
 	if( ph1->vendopts_l.flag.chkpt )
@@ -2206,7 +2222,7 @@ long _IKED::phase1_chk_vend( IDB_PH1 * ph1, BDATA & vend )
 	// check for dead peer detection vendor id
 	//
 
-	if( vendcmp( vend, vend_dpd1 ) )
+	if( vendcmp( vend, vend_dpd1 ) || vendcmp( vend, vend_dpd1_ng ) )
 	{
 		ph1->vendopts_r.flag.dpdv1 = true;
 		log.txt( LLOG_INFO, "ii : peer supports DPDv1\n" );
@@ -2430,28 +2446,28 @@ long _IKED::phase1_chk_hash( IDB_PH1 * ph1 )
 	// generate hash data for comparison
 	//
 
-	BDATA hash;
+	BDATA hash_c;
 
 	if( ph1->initiator )
 	{
-		phase1_gen_hash_r( ph1, hash );
+		phase1_gen_hash_r( ph1, hash_c );
 
 		log.bin(
 			LLOG_DEBUG,
 			LLOG_DECODE,
 			ph1->hash_r.buff(),
-			hash.size(),
+			hash_c.size(),
 			"== : phase1 hash_r ( received )" );
 	}
 	else
 	{
-		phase1_gen_hash_i( ph1, hash );
+		phase1_gen_hash_i( ph1, hash_c );
 
 		log.bin(
 			LLOG_DEBUG,
 			LLOG_DECODE,
 			ph1->hash_r.buff(),
-			hash.size(),
+			hash_c.size(),
 			"== : phase1 hash_i ( received )" );
 	}
 
@@ -2461,8 +2477,7 @@ long _IKED::phase1_chk_hash( IDB_PH1 * ph1 )
 	text_addr( txtaddr_l, &ph1->tunnel->saddr_l, true );
 	text_addr( txtaddr_r, &ph1->tunnel->saddr_r, true );
 
-	long result = memcmp( hash.buff(), ph1->hash_r.buff(), ph1->hash_size );
-	if( result )
+	if( hash_c != ph1->hash_r )
 	{
 		log.txt( LLOG_INFO,
 			"!! : phase1 sa rejected, invalid auth data\n"
@@ -2513,9 +2528,9 @@ long _IKED::phase1_chk_sign( IDB_PH1 * ph1 )
 	// peer provided certificate
 	//
 
-	EVP_PKEY * evp_pkey = NULL;
+	BDATA pubkey;
 
-	if( !pubkey_rsa_read( cert, &evp_pkey ) )
+	if( !pubkey_rsa_read( cert, pubkey ) )
 	{
 		log.txt( LLOG_ERROR, "!! : unable to extract public key from remote peer certificate\n" );
 		return LIBIKE_FAILED;
@@ -2527,7 +2542,7 @@ long _IKED::phase1_chk_sign( IDB_PH1 * ph1 )
 	// by the remote peer
 	//
 
-	if( !pubkey_rsa_decrypt( evp_pkey, ph1->sign_r, ph1->hash_r ) )
+	if( !pubkey_rsa_decrypt( pubkey, ph1->sign_r, ph1->hash_r ) )
 	{
 		log.txt( LLOG_ERROR, "!! : unable to compute remote peer signed hash\n" );
 		return LIBIKE_FAILED;
@@ -2621,10 +2636,8 @@ bool _IKED::phase1_chk_natd( IDB_PH1 * ph1 )
 	switch( ph1->tunnel->peer->natt_mode )
 	{
 		case IPSEC_NATT_DISABLE:
-		{
 			log.txt( LLOG_INFO, "ii : nat-t is disabled locally\n" );
 			break;
-		}
 
 		case IPSEC_NATT_ENABLE:
 		{
@@ -2747,6 +2760,18 @@ bool _IKED::phase1_chk_natd( IDB_PH1 * ph1 )
 				ph1->tunnel->natt_version = IPSEC_NATT_RFC;
 
 			break;
+
+		case IPSEC_NATT_FORCE_CISCO:
+
+			log.txt( LLOG_INFO, "ii : forcing nat-t to cisco-udp\n" );
+
+			//
+			// set natt to cisco-udp
+			//
+
+			ph1->tunnel->natt_version = IPSEC_NATT_CISCO;
+
+			break;
 	}
 	
 	//
