@@ -1009,10 +1009,13 @@ long _IKED::phase2_gen_hash_i( IDB_PH1 * ph1, IDB_PH2 * ph2, BDATA & hash )
 	hash.size( ph1->hash_size );
 
 	HMAC_CTX ctx_prf;
-	HMAC_Init( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash );
+	HMAC_CTX_init( &ctx_prf );
+
+	HMAC_Init_ex( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash, NULL );
 	HMAC_Update( &ctx_prf, input.buff(), input.size() );
 	HMAC_Final( &ctx_prf, hash.buff(), NULL );
-	HMAC_cleanup( &ctx_prf );
+
+	HMAC_CTX_cleanup( &ctx_prf );
 
 	log.bin(
 		LLOG_DEBUG,
@@ -1046,10 +1049,13 @@ long _IKED::phase2_gen_hash_r( IDB_PH1 * ph1, IDB_PH2 * ph2, BDATA & hash )
 	hash.size( ph1->hash_size );
 
 	HMAC_CTX ctx_prf;
-	HMAC_Init( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash );
+	HMAC_CTX_init( &ctx_prf );
+
+	HMAC_Init_ex( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash, NULL );
 	HMAC_Update( &ctx_prf, input.buff(), input.size() );
 	HMAC_Final( &ctx_prf, hash.buff(), NULL );
-	HMAC_cleanup( &ctx_prf );
+
+	HMAC_CTX_cleanup( &ctx_prf );
 
 	log.bin(
 		LLOG_DEBUG,
@@ -1088,10 +1094,13 @@ long _IKED::phase2_gen_hash_p( IDB_PH1 * ph1, IDB_PH2 * ph2, BDATA & hash )
 	hash.size( ph1->hash_size );
 
 	HMAC_CTX ctx_prf;
-	HMAC_Init( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash );
+	HMAC_CTX_init( &ctx_prf );
+
+	HMAC_Init_ex( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash, NULL );
 	HMAC_Update( &ctx_prf, input.buff(), input.size() );
 	HMAC_Final( &ctx_prf, hash.buff(), 0 );
-	HMAC_cleanup( &ctx_prf );
+
+	HMAC_CTX_cleanup( &ctx_prf );
 
 	log.bin(
 		LLOG_DEBUG,
@@ -1109,8 +1118,8 @@ long _IKED::phase2_chk_hash_i( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 	// generate hash data for comparison
 	//
 
-	BDATA hash;
-	phase2_gen_hash_i( ph1, ph2, hash );
+	BDATA hash_c;
+	phase2_gen_hash_i( ph1, ph2, hash_c );
 
 	log.bin(
 		LLOG_DEBUG,
@@ -1133,7 +1142,7 @@ long _IKED::phase2_chk_hash_i( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 	// compare hash data
 	//
 
-	if( memcmp( hash.buff(), ph2->hash_r.buff(), ph1->hash_size ) )
+	if( ph2->hash_r != hash_c )
 	{
 		log.txt( LLOG_ERROR,
 			"!! : phase2 sa rejected, initiator quick mode hash invalid\n"
@@ -1153,8 +1162,8 @@ long _IKED::phase2_chk_hash_r( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 	// generate hash data for comparison
 	//
 
-	BDATA hash;
-	phase2_gen_hash_r( ph1, ph2, hash );
+	BDATA hash_c;
+	phase2_gen_hash_r( ph1, ph2, hash_c );
 
 	log.bin(
 		LLOG_DEBUG,
@@ -1177,7 +1186,7 @@ long _IKED::phase2_chk_hash_r( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 	// compare hash data
 	//
 
-	if( memcmp( hash.buff(), ph2->hash_r.buff(), ph1->hash_size ) )
+	if( ph2->hash_r != hash_c )
 	{
 		log.txt( LLOG_ERROR,
 			"!! : phase2 sa rejected, responder quick mode hash invalid\n"
@@ -1197,8 +1206,8 @@ long _IKED::phase2_chk_hash_p( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 	// generate hash data for comparison
 	//
 
-	BDATA hash;
-	phase2_gen_hash_p( ph1, ph2, hash );
+	BDATA hash_c;
+	phase2_gen_hash_p( ph1, ph2, hash_c );
 
 	log.bin(
 		LLOG_DEBUG,
@@ -1221,7 +1230,7 @@ long _IKED::phase2_chk_hash_p( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 	// compare hash data
 	//
 
-	if( memcmp( hash.buff(), ph2->hash_r.buff(), ph1->hash_size ) )
+	if( ph2->hash_r != hash_c )
 	{
 		log.txt( LLOG_ERROR,
 			"!! : phase2 sa rejected, initiator liveliness proof hash invalid\n"
@@ -1531,8 +1540,6 @@ long _IKED::phase2_chk_params( IDB_PH1 * ph1, IDB_PH2 * ph2, PACKET_IKE & packet
 	return LIBIKE_OK;
 }
 
-#define MAX_KEYSIZE ( EVP_MAX_KEY_LENGTH + HMAC_MAX_MD_CBLOCK * 2 )
-
 long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 {
 	//
@@ -1677,6 +1684,8 @@ long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 	return LIBIKE_OK;
 }
 
+#define PH2_MAX_KEYLEN ( ( EVP_MAX_KEY_LENGTH + HMAC_MAX_MD_CBLOCK ) * 2 )
+
 long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2, long dir, IKE_PROPOSAL * proposal, BDATA & shared )
 {
 	//
@@ -1753,10 +1762,28 @@ long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2, long dir, IKE_PROPOSA
 
 				break;
 
-			case ISAKMP_AUTH_HMAC_SHA:
+			case ISAKMP_AUTH_HMAC_SHA1:
 
 				key_size_h = EVP_MD_size( EVP_sha1() );
 
+				break;
+
+			case ISAKMP_AUTH_HMAC_SHA2_256:
+
+				key_size_h = EVP_MD_size( EVP_sha256() );
+
+				break;
+
+			case ISAKMP_AUTH_HMAC_SHA2_384:
+
+				key_size_h = EVP_MD_size( EVP_sha384() );
+
+				break;
+
+			case ISAKMP_AUTH_HMAC_SHA2_512:
+
+				key_size_h = EVP_MD_size( EVP_sha512() );
+
 				break;
 		}
 	}
@@ -1766,12 +1793,11 @@ long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2, long dir, IKE_PROPOSA
 	// sa and once for the outbound sa
 	//
 
-	unsigned char key_data[ MAX_KEYSIZE ];
+	unsigned char key_data[ PH2_MAX_KEYLEN ];
 
 	long key_size = key_size_c + key_size_h;
 	long skeyid_size = ph1->hash_size;
 
-
 	//
 	// grow our key to be a multiple
 	// of SKEYID_d ( HMAC block size )
@@ -1779,7 +1805,7 @@ long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2, long dir, IKE_PROPOSA
 	if( key_size % skeyid_size )
 		key_size += skeyid_size - ( key_size % skeyid_size );
 
-	assert( key_size < MAX_KEYSIZE );
+	assert( key_size < PH2_MAX_KEYLEN );
 
 	//
 	// create our extended key material
@@ -1792,8 +1818,9 @@ long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2, long dir, IKE_PROPOSA
 	//
 	
 	HMAC_CTX ctx_prf;
+	HMAC_CTX_init( &ctx_prf );
 
-	HMAC_Init( &ctx_prf, ph1->skeyid_d.buff(), ( int ) ph1->skeyid_d.size(), ph1->evp_hash );
+	HMAC_Init_ex( &ctx_prf, ph1->skeyid_d.buff(), ( int ) ph1->skeyid_d.size(), ph1->evp_hash, NULL );
 
 	if( ph2->dhgr_id )
 		HMAC_Update( &ctx_prf, shared.buff(), shared.size() );
@@ -1816,7 +1843,7 @@ long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2, long dir, IKE_PROPOSA
 
 	for( long size = skeyid_size; size < key_size; size += skeyid_size )
 	{
-		HMAC_Init( &ctx_prf, ph1->skeyid_d.buff(), ( int ) ph1->skeyid_d.size(), ph1->evp_hash );
+		HMAC_Init_ex( &ctx_prf, ph1->skeyid_d.buff(), ( int ) ph1->skeyid_d.size(), ph1->evp_hash, NULL );
 		HMAC_Update( &ctx_prf, key_data + size - skeyid_size, skeyid_size );
 
 		if( ph2->dhgr_id )
@@ -1839,7 +1866,7 @@ long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2, long dir, IKE_PROPOSA
 		HMAC_Final( &ctx_prf, key_data + size, 0 );
 	}
 
-	HMAC_cleanup( &ctx_prf );
+	HMAC_CTX_cleanup( &ctx_prf );
 
 	//
 	// separate encrypt and auth key data
