@@ -43,6 +43,7 @@
 #define _IKE_H_
 
 #include "libidb.h"
+#include "inttypes.h"
 
 #define CONF_STRLEN					256
 
@@ -72,6 +73,9 @@
 #define ISAKMP_AH_MD5				2
 #define ISAKMP_AH_SHA				3
 #define ISAKMP_AH_DES				4
+#define ISAKMP_AH_SHA256			5
+#define ISAKMP_AH_SHA384			6
+#define ISAKMP_AH_SHA512			7
 
 // ipsec esp transforms
 
@@ -124,9 +128,12 @@
 // authentication algorithm values
 
 #define ISAKMP_AUTH_HMAC_MD5			1
-#define ISAKMP_AUTH_HMAC_SHA			2
+#define ISAKMP_AUTH_HMAC_SHA1			2
 #define ISAKMP_AUTH_DES_MAC				3
 #define ISAKMP_AUTH_KPDK				4
+#define ISAKMP_AUTH_HMAC_SHA2_256		5
+#define	ISAKMP_AUTH_HMAC_SHA2_384		6
+#define	ISAKMP_AUTH_HMAC_SHA2_512		7
 
 // identification types
 
@@ -295,6 +302,9 @@
 #define IKE_HASH_MD5				1
 #define IKE_HASH_SHA1				2
 #define IKE_HASH_TIGER				3
+#define	IKE_HASH_SHA2_256			4
+#define IKE_HASH_SHA2_384			5
+#define IKE_HASH_SHA2_512			6
 
 #define IKE_AUTH_PRESHARED_KEY		1
 #define IKE_AUTH_SIG_DSA			2
@@ -310,6 +320,8 @@
 #define IKE_GRP_GROUP14				14	// oakley modp 2048
 #define IKE_GRP_GROUP15				15	// oakley modp 3072
 #define IKE_GRP_GROUP16				16	// oakley modp 4096
+#define IKE_GRP_GROUP17				17	// oakley modp 6144
+#define IKE_GRP_GROUP18				18	// oakley modp 8192
 
 #define IKE_GRP_TYPE_MODP			1
 #define IKE_GRP_TYPE_ECP			2
@@ -458,15 +470,17 @@
 // ipsec config constants
 //
 
-#define	IPSEC_CONTACT_CLIENT	1
-#define	IPSEC_CONTACT_INIT		2
-#define	IPSEC_CONTACT_RESP		3
-#define	IPSEC_CONTACT_BOTH		4
+#define	IPSEC_CONTACT_SERVER	1
+#define	IPSEC_CONTACT_CLIENT	2
+#define	IPSEC_CONTACT_INIT		3
+#define	IPSEC_CONTACT_RESP		4
+#define	IPSEC_CONTACT_BOTH		5
 
 #define IPSEC_NATT_DISABLE		0
 #define IPSEC_NATT_ENABLE		1
 #define IPSEC_NATT_FORCE_DRAFT	2
 #define IPSEC_NATT_FORCE_RFC	3
+#define IPSEC_NATT_FORCE_CISCO	4
 
 #define IPSEC_FRAG_DISABLE		0
 #define IPSEC_FRAG_ENABLE		1
@@ -477,10 +491,11 @@
 #define IPSEC_DPD_FORCE			2
 
 #define IPSEC_NATT_NONE			0
-#define IPSEC_NATT_V00			1
-#define IPSEC_NATT_V01			2
-#define IPSEC_NATT_V02			3
-#define IPSEC_NATT_V03			4
+#define IPSEC_NATT_CISCO		1
+#define IPSEC_NATT_V00			2
+#define IPSEC_NATT_V01			3
+#define IPSEC_NATT_V02			4
+#define IPSEC_NATT_V03			5
 #define IPSEC_NATT_RFC			255
 
 #define IPSEC_OPTS_SPLITNET		0x0001
@@ -497,8 +512,8 @@
 #define IPSEC_OPTS_CISCO_GRP	0x0800
 #define IPSEC_OPTS_VEND_CHKPT	0x1000
 
-#define IPSEC_DNSS_MAX			4
-#define IPSEC_NBNS_MAX			2
+#define IPSEC_DNSS_MAX			8
+#define IPSEC_NBNS_MAX			4
 
 #define CONFIG_MODE_NONE		0
 #define CONFIG_MODE_PULL		1
@@ -754,45 +769,4 @@ typedef struct _PLD_DATA
 
 }PLD_DATA;
 
-typedef class _PACKET_IKE : public _PACKET
-{
-	protected:
-
-	PLD_DATA	pld_stack[ 8 ];
-	long		pld_depth;
-
-	IKE_HEADER	header;
-
-	public:
-
-	unsigned char	notify;
-
-	_PACKET_IKE();
-	~_PACKET_IKE();
-
-	void	reset();
-
-	void		set_msgid( uint32_t msgid );
-	uint32_t	get_msgid();
-
-	bool	add_payload( bool encap, uint8_t next );
-	bool	get_payload( bool encap, uint8_t & next );
-	void	end_payload( bool decap, bool write = true );
-
-	size_t	get_payload_left();
-
-	bool	write( IKE_COOKIES & cookies,
-					uint8_t payload,
-					uint8_t exchange,
-					uint8_t flags );
-
-	bool	read( IKE_COOKIES & cookies,
-					uint8_t & payload,
-					uint8_t & exchange,
-					uint8_t & flags );
-
-	bool	done();
-
-}PACKET_IKE;
-
 #endif
