@@ -81,6 +81,7 @@ _IDB_XCH::_IDB_XCH()
 	//
 
 	event_resend.xch = this;
+	event_resend.attempt = 0;
 }
 
 _IDB_XCH::~_IDB_XCH()
@@ -112,7 +113,7 @@ XCH_STATUS _IDB_XCH::status( XCH_STATUS status, XCH_ERRORCODE errorcode, uint16_
 		xch_notifycode = notifycode;
 
 		if( status == XCH_STATUS_DEAD )
-			setflags( IDB_FLAG_DEAD );
+			setflags( ENTRY_FLAG_DEAD );
 	}
 
 	lock.unlock();
@@ -189,9 +190,11 @@ bool _IDB_XCH::resend()
 	iked.text_addr( txtaddr_r, &tunnel->saddr_r, true );
 
 	iked.log.txt( LLOG_INFO,
-		"-> : resend %i %s packet(s) %s -> %s\n",
+		"-> : resend %i %s packet(s) [%i/%i] %s -> %s\n",
 		count,
 		name(),
+		event_resend.attempt,
+		iked.retry_count,
 		txtaddr_l,
 		txtaddr_r );
 
