@@ -371,7 +371,8 @@ _IDB_PH1::_IDB_PH1( IDB_TUNNEL * set_tunnel, bool set_initiator, IKE_COOKIES * s
 	// determine natt negotiation
 	//
 
-	if( tunnel->peer->natt_mode >= IPSEC_NATT_ENABLE )
+	if( ( tunnel->peer->natt_mode >= IPSEC_NATT_ENABLE ) &&
+		( tunnel->peer->natt_mode != IPSEC_NATT_FORCE_CISCO ) )
 		vendopts_l.flag.natt = true;
 
 	//
@@ -477,7 +478,7 @@ const char * _IDB_PH1::name()
 	return xname;
 }
 
-IDB_RC_LIST * _IDB_PH1::list()
+IKED_RC_LIST * _IDB_PH1::list()
 {
 	return &iked.idb_list_ph1;
 }
@@ -642,7 +643,8 @@ void _IDB_PH1::end()
 	//
 	// if this is a client tunnel and there
 	// was an error negotiating phase1, set
-	// a close error message
+	// a close error message and wakeup the
+	// client thread
 	//
 
 	if( tunnel->peer->contact == IPSEC_CONTACT_CLIENT )
@@ -650,7 +652,9 @@ void _IDB_PH1::end()
 		if( xch_errorcode != XCH_FAILED_EXPIRED )
 		{
 			tunnel->close = xch_errorcode;
-			tunnel->ikei->wakeup();
+
+			if( tunnel->ikei != NULL )
+				tunnel->ikei->wakeup();
 		}
 	}
 }
@@ -768,6 +772,24 @@ bool _IDB_PH1::setup_xform( IKE_PROPOSAL * proposal )
 
 			break;
 
+		case IKE_HASH_SHA2_256:
+
+			evp_hash = EVP_sha256();
+
+			break;
+
+		case IKE_HASH_SHA2_384:
+
+			evp_hash = EVP_sha384();
+
+			break;
+
+		case IKE_HASH_SHA2_512:
+
+			evp_hash = EVP_sha512();
+
+			break;
+
 		default:
 			return false;
 	}
@@ -971,4 +993,3 @@ bool _IDB_PH1::frag_get( PACKET_IKE & packet )
 
 	return true;
 }
-
