@@ -45,7 +45,7 @@
 // ike client io thread
 //
 
-long ITH_IKES::func( void * arg )
+long ITH_IKES::iked_func( void * arg )
 {
 	IKED * iked = ( IKED * ) arg;
 	return iked->loop_ipc_server();
@@ -82,7 +82,7 @@ long _IKED::loop_ipc_server()
 	return true;
 }
 
-long ITH_IKEC::func( void * arg )
+long ITH_IKEC::iked_func( void * arg )
 {
 	IKEI * ikei = ( IKEI * ) arg;
 	long result = iked.loop_ipc_client( ikei );
@@ -93,13 +93,39 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 {
 	loop_ref_inc( "ipc client" );
 
+	//
+	// temporary configuration data
+	//
+
+	IKE_XCONF ike_xconf;
+	memset( &ike_xconf, 0, sizeof( ike_xconf ) );
+
+	IKE_PEER ike_peer;
+	memset( &ike_peer, 0, sizeof( ike_peer ) );
+
+	BDATA xuser;
+	BDATA xpass;
+	BDATA psk;
+	BDATA cert_r;
+	BDATA cert_l;
+	BDATA cert_k;
+	BDATA iddata_r;
+	BDATA iddata_l;
+
+	IDB_LIST_PROPOSAL	proposals;
+	IDB_LIST_PH2ID		idlist_incl;
+	IDB_LIST_PH2ID		idlist_excl;
+	IDB_LIST_DOMAIN		domains;
+
+	IKE_SADDR saddr_l;
+	BDATA fpass;
+
+	//
+	// db configuration objects
+	//
+
 	IDB_PEER *		peer = NULL;
 	IDB_TUNNEL *	tunnel = NULL;
-	VNET_ADAPTER *	adapter = NULL;
-
-#ifdef OPT_DTP
-	DTPI dtpi;
-#endif
 
 	//
 	// enter client ctrl loop
@@ -107,9 +133,10 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 
 	IKEI_MSG msg;
 
-	bool failure = false;
+	bool detach = false;
+	bool suspended = false;
 
-	while( !failure )
+	while( !detach )
 	{
 		long result;
 
@@ -135,110 +162,51 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 
 		if( result == IPCERR_OK )
 		{
+			//
+			// set default result
+			//
+
+			result = IKEI_RESULT_FAILED;
+
+			//
+			// handle message by type
+			//
+
 			switch( msg.header.type )
 			{
 				//
-				// peer config add message
+				// client config message
+				//
+
+				case IKEI_MSGID_CLIENT:
+				{
+					log.txt( LLOG_INFO, "<A : client config message\n" );
+
+					if( msg.get_client( &ike_xconf ) != IPCERR_OK )
+					{
+						log.txt( LLOG_ERROR, "!! : failed to read client config message\n" );
+						break;
+					}
+
+					result = IKEI_RESULT_OK;
+					break;
+				}
+
+				//
+				// peer config message
 				//
 
 				case IKEI_MSGID_PEER:
 				{
 					log.txt( LLOG_INFO, "<A : peer config add message\n" );
 
-					IKE_PEER ike_peer;
 					if( msg.get_peer( &ike_peer ) != IPCERR_OK )
-						break;
-
-					//
-					// create new peer
-					//
-
-					peer = new IDB_PEER( &ike_peer );
-
-					//
-					// add new peer
-					//
-
-					if( !peer->add( true ) )
 					{
-						log.txt( LLOG_ERROR, "!! : unable to add peer object\n" );
-
-						delete peer;
-
-						msg.set_result( IKEI_RESULT_FAILED );
-						ikei->send_message( msg );
-
-						failure = true;
-
+						log.txt( LLOG_ERROR, "!! : failed to read peer config message\n" );
 						break;
 					}
 
-					//
-					// determine local tunnel addresses
-					//
-
-					IKE_SADDR saddr_l;
-
-					if( socket_lookup_addr(	peer->saddr, saddr_l ) != LIBIKE_OK )
-					{
-						log.txt( LLOG_ERROR, "!! : no route to host\n" );
-
-						delete tunnel;
-
-						msg.set_result( IKEI_RESULT_FAILED );
-						ikei->send_message( msg );
-
-						failure = true;
-
-						break;
-					}
-
-					//
-					// determine local socket port
-					//
-
-					if( socket_lookup_port( saddr_l, false ) != LIBIKE_OK )
-					{
-						log.txt( LLOG_ERROR, "!! : no socket for selected address\n" );
-
-						delete tunnel;
-
-						msg.set_result( IKEI_RESULT_FAILED );
-						ikei->send_message( msg );
-
-						failure = true;
-
-						break;
-					}
-
-					//
-					// create new tunnel
-					//
-
-					tunnel = new IDB_TUNNEL( peer, &saddr_l, &peer->saddr );
-					tunnel->ikei = ikei;
-
-					//
-					// add new tunnel
-					//
-
-					if( !tunnel->add( true ) )
-					{
-						log.txt( LLOG_ERROR, "!! : unable to add tunnel object\n" );
-
-						delete tunnel;
-
-						msg.set_result( IKEI_RESULT_FAILED );
-						ikei->send_message( msg );
-
-						tunnel->close = XCH_FAILED_CLIENT;
-
-						break;
-					}
-
-					msg.set_result( IKEI_RESULT_OK );
-					ikei->send_message( msg );
-
+					result = IKEI_RESULT_OK;
 					break;
 				}
 
@@ -252,51 +220,18 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 
 					IKE_PROPOSAL proposal;
 					if( msg.get_proposal( &proposal ) != IPCERR_OK )
-						break;
-
-					//
-					// fortigate hack
-					//
-
-//					if( ( proposal.auth_id == XAUTH_AUTH_INIT_PSK ) &&
-//						( peer->xconf_mode == CONFIG_MODE_DHCP ) )
-//						proposal.auth_id = IKE_AUTH_PRESHARED_KEY;
-
-					if( !peer->proposals.add( &proposal, true ) )
 					{
-						log.txt( LLOG_ERROR, "!! : unable to add peer proposal\n" );
-
-						msg.set_result( IKEI_RESULT_FAILED );
-						ikei->send_message( msg );
-
-						tunnel->close = XCH_FAILED_CLIENT;
-
+						log.txt( LLOG_ERROR, "!! : failed to read proposal config message\n" );
 						break;
 					}
 
-					msg.set_result( IKEI_RESULT_OK );
-					ikei->send_message( msg );
-
-					break;
-				}
-
-				//
-				// client config message
-				//
-
-				case IKEI_MSGID_CLIENT:
-				{
-					log.txt( LLOG_INFO, "<A : client config message\n" );
-
-					IKE_XCONF xconf;
-					if( msg.get_client( &xconf ) != IPCERR_OK )
+					if( !proposals.add( &proposal, true ) )
+					{
+						log.txt( LLOG_ERROR, "!! : unable to add proposal\n" );
 						break;
+					}
 
-					tunnel->xconf = xconf;
-
-					msg.set_result( IKEI_RESULT_OK );
-					ikei->send_message( msg );
-
+					result = IKEI_RESULT_OK;
 					break;
 				}
 
@@ -312,24 +247,27 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 					memset( &ph2id, 0, sizeof( ph2id ) );
 
 					long type;
-
 					if( msg.get_network( &type, &ph2id ) != IPCERR_OK )
-						break;
-
-					switch( type )
 					{
-						case UNITY_SPLIT_INCLUDE:
-							tunnel->idlist_incl.add( ph2id );
-							break;
-
-						case UNITY_SPLIT_EXCLUDE:
-							tunnel->idlist_excl.add( ph2id );
-							break;
+						log.txt( LLOG_ERROR, "!! : failed to read remote resource message\n" );
+						break;
 					}
 
-					msg.set_result( IKEI_RESULT_OK );
-					ikei->send_message( msg );
+					bool added = false;
 
+					if( type == UNITY_SPLIT_INCLUDE )
+						added = idlist_incl.add( ph2id );
+
+					if( type == UNITY_SPLIT_EXCLUDE )
+						added = idlist_excl.add( ph2id );
+
+					if( !added )
+					{
+						log.txt( LLOG_ERROR, "!! : unable to add network\n" );
+						break;
+					}
+
+					result = IKEI_RESULT_OK;
 					break;
 				}
 
@@ -339,11 +277,14 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 
 				case IKEI_MSGID_CFGSTR:
 				{
-					BDATA	text;
+					BDATA	data;
 					long	type;
 
-					if( msg.get_cfgstr( &type, &text ) != IPCERR_OK )
+					if( msg.get_cfgstr( &type, &data ) != IPCERR_OK )
+					{
+						log.txt( LLOG_ERROR, "!! : failed to read config string message\n" );
 						break;
+					}
 
 					switch( type )
 					{
@@ -355,11 +296,9 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 						{
 							log.txt( LLOG_INFO, "<A : xauth username message\n" );
 
-							tunnel->xauth.user = text;
-
-							msg.set_result( IKEI_RESULT_OK );
-							ikei->send_message( msg );
+							xuser = data;
 
+							result = IKEI_RESULT_OK;
 							break;
 						}
 
@@ -371,11 +310,9 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 						{
 							log.txt( LLOG_INFO, "<A : xauth password message\n" );
 
-							tunnel->xauth.pass = text;
-
-							msg.set_result( IKEI_RESULT_OK );
-							ikei->send_message( msg );
+							xpass = data;
 
+							result = IKEI_RESULT_OK;
 							break;
 						}
 
@@ -387,11 +324,9 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 						{
 							log.txt( LLOG_INFO, "<A : preshared key message\n" );
 
-							tunnel->peer->psk = text;
-
-							msg.set_result( IKEI_RESULT_OK );
-							ikei->send_message( msg );
+							psk = data;
 
+							result = IKEI_RESULT_OK;
 							break;
 						}
 
@@ -403,11 +338,10 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 						{
 							log.txt( LLOG_INFO, "<A : file password\n" );
 
-							tunnel->peer->fpass = text;
-
-							msg.set_result( IKEI_RESULT_OK );
-							ikei->send_message( msg );
+							fpass = data;
+							fpass.add( "", 0 );
 
+							result = IKEI_RESULT_OK;
 							break;
 						}
 
@@ -417,31 +351,18 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 
 						case CFGSTR_CRED_RSA_RCRT:
 						{
-							text.add( "", 1 );
+							log.txt( LLOG_INFO, "<A : remote certificate data message\n" );
 
-							log.txt( LLOG_INFO, "<A : remote cert \'%s\' message\n", text.text() );
-
-							long loaded = cert_load( tunnel->peer->cert_r, text.text(), true, tunnel->peer->fpass );
-
-							switch( loaded )
+							switch( cert_load( cert_r, data, true, fpass ) )
 							{
 								case FILE_OK:
-									msg.set_result( IKEI_RESULT_OK );
-									ikei->send_message( msg );
-									log.txt( LLOG_DEBUG, "ii : \'%s\' loaded\n", text.text() );
-									break;
-
-								case FILE_PATH:
-									log.txt( LLOG_ERROR, "!! : \'%s\' load failed, invalid path\n", text.text() );
-									msg.set_result( IKEI_RESULT_FAILED );
-									ikei->send_message( msg );
-									tunnel->close = XCH_FAILED_CLIENT;
+									log.txt( LLOG_DEBUG, "ii : remote certificate read complete ( %i bytes )\n", cert_r.size() );
+									result = IKEI_RESULT_OK;
 									break;
 
 								case FILE_FAIL:
-									log.txt( LLOG_ERROR, "!! : \'%s\' load failed, requesting password\n", text.text() );
-									msg.set_result( IKEI_RESULT_PASSWD );
-									ikei->send_message( msg );
+									log.txt( LLOG_ERROR, "!! : remote certificate read failed, requesting password\n" );
+									result = IKEI_RESULT_PASSWD;
 									break;
 							}
 
@@ -454,31 +375,18 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 
 						case CFGSTR_CRED_RSA_LCRT:
 						{
-							text.add( "", 1 );
+							log.txt( LLOG_INFO, "<A : local certificate data message\n" );
 
-							log.txt( LLOG_INFO, "<A : local cert \'%s\' message\n", text.text() );
-
-							long loaded = cert_load( tunnel->peer->cert_l, text.text(), false, tunnel->peer->fpass );
-
-							switch( loaded )
+							switch( cert_load( cert_l, data, false, fpass ) )
 							{
 								case FILE_OK:
-									msg.set_result( IKEI_RESULT_OK );
-									ikei->send_message( msg );
-									log.txt( LLOG_DEBUG, "ii : \'%s\' loaded\n", text.text() );
-									break;
-
-								case FILE_PATH:
-									log.txt( LLOG_ERROR, "!! : \'%s\' load failed, invalid path\n", text.text() );
-									msg.set_result( IKEI_RESULT_FAILED );
-									ikei->send_message( msg );
-									tunnel->close = XCH_FAILED_CLIENT;
+									log.txt( LLOG_DEBUG, "ii : local certificate read complete ( %i bytes )\n", cert_l.size() );
+									result = IKEI_RESULT_OK;
 									break;
 
 								case FILE_FAIL:
-									log.txt( LLOG_ERROR, "!! : \'%s\' load failed, requesting password\n", text.text() );
-									msg.set_result( IKEI_RESULT_PASSWD );
-									ikei->send_message( msg );
+									log.txt( LLOG_ERROR, "!! : local certificate read failed, requesting password\n" );
+									result = IKEI_RESULT_PASSWD;
 									break;
 							}
 
@@ -491,31 +399,18 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 
 						case CFGSTR_CRED_RSA_LKEY:
 						{
-							text.add( "", 1 );
+							log.txt( LLOG_INFO, "<A : local key data message\n" );
 
-							log.txt( LLOG_INFO, "<A : local key \'%s\' message\n", text.text() );
-
-							long loaded = prvkey_rsa_load( &tunnel->peer->key, text.text(), tunnel->peer->fpass );
-
-							switch( loaded )
+							switch( prvkey_rsa_load( cert_k, data, fpass ) )
 							{
 								case FILE_OK:
-									msg.set_result( IKEI_RESULT_OK );
-									ikei->send_message( msg );
-									log.txt( LLOG_DEBUG, "ii : \'%s\' loaded\n", text.text() );
-									break;
-
-								case FILE_PATH:
-									log.txt( LLOG_ERROR, "!! : \'%s\' load failed, invalid path\n", text.text() );
-									msg.set_result( IKEI_RESULT_FAILED );
-									ikei->send_message( msg );
-									tunnel->close = XCH_FAILED_CLIENT;
+									log.txt( LLOG_DEBUG, "ii : local key read complete ( %i bytes )\n", cert_k.size() );
+									result = IKEI_RESULT_OK;
 									break;
 
 								case FILE_FAIL:
-									log.txt( LLOG_ERROR, "!! : \'%s\' load failed, requesting password\n", text.text() );
-									msg.set_result( IKEI_RESULT_PASSWD );
-									ikei->send_message( msg );
+									log.txt( LLOG_ERROR, "!! : local key read failed, requesting password\n" );
+									result = IKEI_RESULT_PASSWD;
 									break;
 							}
 
@@ -529,16 +424,14 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 						case CFGSTR_CRED_LID:
 						{
 							BDATA idval;
-							idval = text;
-							idval.add( "", 1 );
+							idval = data;
+							idval.add( 0, 1 );
 
 							log.txt( LLOG_INFO, "<A : local id \'%s\' message\n", idval.text() );
 
-							tunnel->peer->iddata_l = text;
-
-							msg.set_result( IKEI_RESULT_OK );
-							ikei->send_message( msg );
+							iddata_l = data;
 
+							result = IKEI_RESULT_OK;
 							break;
 						}
 
@@ -549,16 +442,14 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 						case CFGSTR_CRED_RID:
 						{
 							BDATA idval;
-							idval = text;
-							idval.add( "", 1 );
+							idval = data;
+							idval.add( 0, 1 );
 
 							log.txt( LLOG_INFO, "<A : remote id \'%s\' message\n", idval.text() );
 
-							tunnel->peer->iddata_r = text;
-
-							msg.set_result( IKEI_RESULT_OK );
-							ikei->send_message( msg );
+							iddata_r = data;
 
+							result = IKEI_RESULT_OK;
 							break;
 						}
 
@@ -568,15 +459,13 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 
 						case CFGSTR_SPLIT_DOMAIN:
 						{
-							text.add( "", 1 );
+							data.add( 0, 1 );
 
-							log.txt( LLOG_INFO, "<A : split dns \'%s\' message\n", text.text() );
+							log.txt( LLOG_INFO, "<A : split dns \'%s\' message\n", data.text() );
 
-							tunnel->domains.add( text );
-
-							msg.set_result( IKEI_RESULT_OK );
-							ikei->send_message( msg );
+							domains.add( data );
 
+							result = IKEI_RESULT_OK;
 							break;
 						}
 					}
@@ -593,101 +482,224 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 					long enable;
 
 					if( msg.get_enable( &enable ) != IPCERR_OK )
+					{
+						log.txt( LLOG_ERROR, "!! : failed to read tunnel enable message\n" );
 						break;
+					}
 
 					if( enable )
 					{
 						log.txt( LLOG_INFO, "<A : peer tunnel enable message\n" );
 
 						//
-						// if the tunnel peer definition states
-						// that we are to act as a client, create
-						// a new phase1 sa and add it to our list
+						// dns proxy init check
+						//
+#ifdef WIN32
+						if( !dnsproxy_check( ikei ) )
+						{
+							detach = true;
+							break;
+						}
+#endif
+						//
+						// create peer object
 						//
 
-						if( tunnel->peer->contact == IPSEC_CONTACT_CLIENT )
+						peer = new IDB_PEER( &ike_peer );
+						if( peer == NULL )
 						{
-							IDB_PH1 * ph1 = new IDB_PH1( tunnel, true, NULL );
-							ph1->add( true );
-							process_phase1_send( ph1 );
-							ph1->dec( true );
+							log.txt( LLOG_ERROR, "!! : unable to create peer object\n" );
+							detach = true;
+							break;
 						}
 
-						if( tunnel->peer->contact == IPSEC_CONTACT_CLIENT )
+						peer->contact = IPSEC_CONTACT_CLIENT;
+						peer->psk = psk;
+						peer->cert_r = cert_r;
+						peer->cert_l = cert_l;
+						peer->cert_k = cert_k;
+						peer->iddata_r = iddata_r;
+						peer->iddata_l = iddata_l;
+
+						psk.del();
+						cert_l.del();
+						cert_k.del();
+						iddata_r.del();
+						iddata_l.del();
+
+						IKE_PROPOSAL * proposal;
+
+						long index = 0;
+						while( proposals.get( &proposal, index++ ) )
+							peer->proposals.add( proposal, true );
+
+						if( !peer->add( true ) )
 						{
-							//
-							// configure our private tunnel endpoint
-							//
-
-							if( tunnel->xconf.opts & IPSEC_OPTS_ADDR )
-							{
-								//
-								// if we require a virutal adapter,
-								// create one now as the startup
-								// time can be long if a new device
-								// instance needs to be created
-								//
-
-								if( !vnet_get( &adapter ) )
-								{
-									log.txt( LLOG_ERROR, "ii : unable to create vnet adapter ...\n" );
-									tunnel->close = XCH_FAILED_ADAPTER;
-
-									enable = false;
-								}
-							}
-							else
-							{
-								//
-								// if we are using a public adapter,
-								// set our client info to match the
-								// selected interface
-								//
-
-								tunnel->xconf.addr = tunnel->saddr_l.saddr4.sin_addr;
-								tunnel->xconf.mask.s_addr = 0xffffffff;
-							}
-
-							//
-							// add the statistics event
-							//
-
-							tunnel->inc( true );
-							tunnel->event_stats.delay = 1000;
-							ith_timer.add( &tunnel->event_stats );
+							log.txt( LLOG_ERROR, "!! : unable to add peer object\n" );
+							detach = true;
+							delete peer;
+							peer = NULL;
+							break;
 						}
-					}
-					else
-					{
-						log.txt( LLOG_INFO, "<A : peer tunnel disable message\n" );
-						tunnel->close = XCH_FAILED_USERREQ;
-					}
 
-					if( enable )
-					{
-						msg.set_enable( true );
+						//
+						// determine local tunnel addresses
+						//
+
+						if( socket_lookup_addr(	peer->saddr, saddr_l ) != LIBIKE_OK )
+						{
+							log.txt( LLOG_ERROR, "!! : no route to host\n" );
+							detach = true;
+							break;
+						}
+
+						//
+						// determine local socket port
+						//
+
+						if( socket_lookup_port( saddr_l, false ) != LIBIKE_OK )
+						{
+							log.txt( LLOG_ERROR, "!! : no socket for selected address\n" );
+							detach = true;
+							break;
+						}
+
+						//
+						// create tunnel object
+						//
+
+						tunnel = new IDB_TUNNEL( peer, &ike_xconf, &saddr_l, &peer->saddr );
+						if( tunnel == NULL )
+						{
+							log.txt( LLOG_ERROR, "!! : unable to create tunnel object\n" );
+							detach = true;
+							break;
+						}
+
+						tunnel->ikei = ikei;
+						tunnel->xauth.user = xuser;
+						tunnel->xauth.pass = xpass;
+
+						xuser.del( true );
+						xpass.del( true );
+
+						IKE_PH2ID ph2id;
+
+						index = 0;
+						while( idlist_incl.get( ph2id, index++ ) )
+							tunnel->idlist_incl.add( ph2id );
+
+						index = 0;
+						while( idlist_excl.get( ph2id, index++ ) )
+							tunnel->idlist_excl.add( ph2id );
+
+						BDATA domain;
+
+						index = 0;
+						while( domains.get( domain, index++ ) )
+							tunnel->domains.add( domain );
+
+						domain.del();
+
+						if( !tunnel->add( true ) )
+						{
+							log.txt( LLOG_ERROR, "!! : unable to add tunnel object\n" );
+							detach = true;
+							delete tunnel;
+							tunnel = NULL;
+							break;
+						}
+
+						//
+						// initiate communications with peer
+						//
+
+						IDB_PH1 * ph1 = new IDB_PH1( tunnel, true, NULL );
+						ph1->add( true );
+						process_phase1_send( ph1 );
+						ph1->dec( true );
+
+						msg.set_status( STATUS_CONNECTING, "tunnel connecting ...\n" );
 						ikei->send_message( msg );
 					}
 					else
 					{
-						msg.set_enable( false );
+						log.txt( LLOG_INFO, "<A : peer tunnel disable message\n" );
+
+						if( tunnel != NULL )
+							tunnel->close = XCH_FAILED_USERREQ;
+
+						msg.set_status( STATUS_DISCONNECTING, "tunnel disconnecting ...\n" );
 						ikei->send_message( msg );
 					}
 
 					break;
 				}
+#ifdef WIN32
+				//
+				// suspend tunnel message
+				//
 
+				case IKEI_MSGID_SUSPEND:
+				{
+					long suspend = 0;
+
+					if( msg.get_suspend( &suspend ) != IPCERR_OK )
+					{
+						log.txt( LLOG_ERROR, "!! : failed to read tunnel suspend message\n" );
+						break;
+					}
+
+					if( suspend )
+					{
+						iked.ith_timer.del( &tunnel->event_stats );
+
+						log.txt( LLOG_DEBUG, "ii : suspended client control of tunnel\n" );
+						tunnel->suspended = true;
+						tunnel->ikei = NULL;
+						tunnel->dec( true );
+						suspended = true;
+						detach = true;
+					}
+					else
+					{
+						if( !iked.idb_list_tunnel.find( true, &tunnel, NULL, NULL, false, true ) )
+						{
+							log.txt( LLOG_ERROR, "!! : failed to locate suspended tunnel\n" );
+							detach = true;
+							break;
+						}
+
+						log.txt( LLOG_DEBUG, "ii : resumed client control of tunnel\n" );
+						peer = tunnel->peer;
+						tunnel->suspended = false;
+						tunnel->ikei = ikei;
+
+						ith_timer.add( &tunnel->event_stats );
+					}
+
+					break;
+				}
+#endif
 				default:
 					log.txt( LLOG_ERROR, "!! : message type is invalid ( %u )\n", msg.header.type );
-					tunnel->close = XCH_FAILED_CLIENT;
+					if( tunnel != NULL )
+						tunnel->close = XCH_FAILED_CLIENT;
 					break;
 			}
 
+			//
+			// send result message
+			//
+
+			msg.set_result( result );
+			ikei->send_message( msg );
+
 			continue;
 		}
 
 		//
-		// tunnel configuration steps
+		// tunnel configuration steps ( IPCERR_WAKEUP )
 		//
 
 		if( tunnel != NULL )
@@ -724,15 +736,16 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 
 				if( !tunnel->xconf.mask.s_addr )
 				{
-					log.txt( LLOG_ERROR, "!! : invalid private netmask, defaulting to class c\n" );
+					log.txt( LLOG_ERROR, "!! : invalid private netmask, defaulting to 255.255.255.0\n" );
 					tunnel->xconf.mask.s_addr = inet_addr( "255.255.255.0" );
 				}
 
 				//
-				// setup client parameters
+				// setup client network parameters
 				//
 
-				client_setup( adapter, tunnel );
+				if( !client_net_config( tunnel ) )
+					break;
 
 				msg.set_status( STATUS_INFO, "network device configured\n" );
 				ikei->send_message( msg );
@@ -744,32 +757,31 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 				policy_list_create( tunnel, true );
 
 				//
-				// setup dns transparent proxy
+				// setup client dns parameters
 				//
 
-#ifdef OPT_DTP
-				dnsproxy_setup( dtpi, tunnel );
-#endif
+				client_dns_config( tunnel );
 
 				//
 				// tunnel is enabled
 				//
 
-				msg.set_status( STATUS_ENABLED, "tunnel enabled\n" );
+				msg.set_status( STATUS_CONNECTED, "tunnel connected\n" );
 				ikei->send_message( msg );
 
+				//
+				// add the statistics event
+				//
+
+				tunnel->inc( true );
+				tunnel->event_stats.delay = 1000;
+				ith_timer.add( &tunnel->event_stats );
+
 				tunnel->tstate |= TSTATE_VNET_ENABLE;
 			}
 		}
 	}
 
-	//
-	// flush our private pcap dump files
-	//
-
-	if( dump_decrypt )
-		pcap_decrypt.flush();
-
 	//
 	// perform tunnel cleanup
 	//
@@ -777,17 +789,27 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 	if( tunnel == NULL )
 	{
 		//
-		// peer or tunnel setup failed
+		// tunnel configuration failed
 		//
 
-		msg.set_status( STATUS_FAIL, "client configuration error\n" );
+		msg.set_status( STATUS_FAIL, "tunnel configuration failed\n" );
 		ikei->send_message( msg );
 	}
-	else
+
+	if( ( tunnel != NULL ) && !suspended )
 	{
 		//
-		// cleaup our security policy lists
-		// ( caller must hold the sdb lock )
+		// revert client network parameters
+		//
+
+		client_dns_revert( tunnel );
+
+		//
+		// cleanup client settings
+		//
+		// NOTE : policy cleanup must be done here to
+		// avoid route deletion failures from occuring
+		// after a virtual adapter has been removed
 		//
 
 		lock_idb.lock();
@@ -795,32 +817,22 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 		if( tunnel->peer->plcy_mode != POLICY_MODE_DISABLE )
 			iked.policy_list_remove( tunnel, true );
 
+		lock_idb.unlock();
+
 		if( tunnel->peer->xconf_mode == CONFIG_MODE_DHCP )
 			iked.socket_dhcp_remove( tunnel );
 
-		lock_idb.unlock();
-
 		//
-		// cleanup client settings
+		// revert client network parameters
 		//
 
-		if( tunnel->tstate & TSTATE_VNET_ENABLE )
-			client_cleanup( adapter, tunnel );
+		client_net_revert( tunnel );
 
 		//
-		// if we were using a virutal adapter,
-		// perform some addition cleanup
+		// flush our arp cache
 		//
 
-		if( tunnel->xconf.opts & IPSEC_OPTS_ADDR )
-		{
-			//
-			// disable the adapter
-			//
-
-			if( adapter != NULL )
-				vnet_rel( adapter );
-		}
+		iproute.flusharp( saddr_l.saddr4.sin_addr );
 
 		//
 		// report reason for closing the tunnel
@@ -925,43 +937,39 @@ long _IKED::loop_ipc_client( IKEI * ikei )
 		ikei->send_message( msg );
 
 		//
-		// flush our arp cache
-		//
-
-		iproute.flusharp( tunnel->saddr_l.saddr4.sin_addr );
-
-		//
-		// destroy the tunnel object
+		// release the tunnel object
 		//
 
 		tunnel->dec( true, true );
+		msg.set_status( STATUS_DISCONNECTED, "tunnel disconnected\n" );
 	}
 
 	//
 	// perform peer cleanup
 	//
 
-	if( peer != NULL )
+	if( ( peer != NULL ) && !suspended )
+	{
 		peer->dec( true, true );
-
-	//
-	// cleanup dns transparent proxy
-	//
-
-#ifdef OPT_DTP
-		dnsproxy_cleanup( dtpi );
-#endif
+		msg.set_status( STATUS_DISCONNECTED, "peer removed\n" );
+	}
 
 	//
 	// close the client interface
 	//
 
-	msg.set_status( STATUS_DISABLED, "tunnel disabled\n" );
 	ikei->send_message( msg );
 	ikei->detach();
 
 	delete ikei;
 
+	//
+	// flush our private pcap dump files
+	//
+
+	if( dump_decrypt )
+		pcap_decrypt.flush();
+
 	loop_ref_dec( "ipc client" );
 
 	return true;
