@@ -45,7 +45,7 @@
 // ike network io thread
 //
 
-long ITH_NWORK::func( void * arg )
+long ITH_NWORK::iked_func( void * arg )
 {
 	IKED * iked = ( IKED * ) arg;
 	return iked->loop_ike_nwork();
@@ -59,43 +59,22 @@ long _IKED::loop_ike_nwork()
 
 	loop_ref_inc( "network" );
 
-	while( state == DSTATE_ACTIVE )
+	ETH_HEADER	eth_header;
+	PACKET_IP	packet_ip;
+
+	while( true )
 	{
-		//
-		// wait for packet availablility
-		//
-
-		long result = socket_select( 500 );
-		if( result <= 0 )
-			continue;
-
-		//
-		// process inbound ike packets
-		//
-
-		IKE_SADDR saddr_src;
-		IKE_SADDR saddr_dst;
-
-		memset( &saddr_src, 0, sizeof( saddr_src ) );
-		memset( &saddr_dst, 0, sizeof( saddr_dst ) );
-
-		saddr_src.saddr4.sin_family = AF_INET;
-		saddr_dst.saddr4.sin_family = AF_INET;
-
-		unsigned char proto;
-
 		//
 		// attempt to recv packet
 		//
 
-		ETH_HEADER eth_header;
-
-		PACKET_IP packet_ip;
-
-		result = recv_ip(
+		long result = recv_ip(
 					packet_ip,
 					&eth_header );
 
+		if( result == LIBIKE_SOCKET )
+			break;
+
 		if( result == LIBIKE_NODATA )
 			continue;
 
@@ -112,6 +91,17 @@ long _IKED::loop_ike_nwork()
 		// read the ip header
 		//
 
+		IKE_SADDR saddr_src;
+		IKE_SADDR saddr_dst;
+
+		memset( &saddr_src, 0, sizeof( saddr_src ) );
+		memset( &saddr_dst, 0, sizeof( saddr_dst ) );
+
+		saddr_src.saddr4.sin_family = AF_INET;
+		saddr_dst.saddr4.sin_family = AF_INET;
+
+		unsigned char proto;
+
 		packet_ip.read(
 			saddr_src.saddr4.sin_addr,
 			saddr_dst.saddr4.sin_addr,
@@ -327,7 +317,8 @@ long _IKED::process_ike_recv( PACKET_IKE & packet, IKE_SADDR & saddr_src, IKE_SA
 				&tunnel,
 				NULL,
 				&saddr_src,
-				true ) )
+				true,
+				false ) )
 		{
 			//
 			// attempt to locate a peer
@@ -348,7 +339,7 @@ long _IKED::process_ike_recv( PACKET_IKE & packet, IKE_SADDR & saddr_src, IKE_SA
 				return LIBIKE_OK;
 			}
 
-			tunnel = new IDB_TUNNEL( peer, &saddr_dst, &saddr_src );
+			tunnel = new IDB_TUNNEL( peer, NULL, &saddr_dst, &saddr_src );
 
 			if( tunnel == NULL )
 			{
