@@ -45,7 +45,7 @@
 // ike pfkey io thread
 //
 
-long ITH_PFKEY::func( void * arg )
+long ITH_PFKEY::iked_func( void * arg )
 {
 	IKED * iked = ( IKED * ) arg;
 	return iked->loop_ike_pfkey();
@@ -61,7 +61,7 @@ long _IKED::loop_ike_pfkey()
 
 	PFKI_MSG msg;
 
-	while( state == DSTATE_ACTIVE )
+	while( true )
 	{
 		//
 		// read the next pfkey message
@@ -69,6 +69,9 @@ long _IKED::loop_ike_pfkey()
 
 		long result = pfki.recv_message( msg );
 
+		if( result == IPCERR_WAKEUP )
+			break;
+
 		if( result == IPCERR_CLOSED )
 		{
 			pfki.detach();
@@ -370,7 +373,7 @@ long _IKED::pfkey_init_phase2( bool nail, u_int16_t plcytype, u_int32_t plcyid,
 	// in the form of an aquire
 	//
 
-	if( policy_out->nailed && !nail )
+	if( ( policy_out->flags & PFLAG_NAILED ) && !nail )
 	{
 		log.txt( LLOG_INFO, "ii : ignoring init phase2 by acquire, tunnel is nailed\n" );
 
@@ -422,6 +425,7 @@ long _IKED::pfkey_init_phase2( bool nail, u_int16_t plcytype, u_int32_t plcyid,
 			&tunnel,
 			NULL,
 			&dst,
+			false, 
 			false ) )
 	{
 		//
@@ -469,7 +473,7 @@ long _IKED::pfkey_init_phase2( bool nail, u_int16_t plcytype, u_int32_t plcyid,
 		// tunnel for the peer
 		//
 
-		tunnel = new IDB_TUNNEL( peer, &src, &peer->saddr );
+		tunnel = new IDB_TUNNEL( peer, NULL, &src, &peer->saddr );
 
 		if( tunnel == NULL )
 		{
@@ -526,8 +530,8 @@ long _IKED::pfkey_init_phase2( bool nail, u_int16_t plcytype, u_int32_t plcyid,
 
 	//
 	// store the policy ids
- 	//
- 
+	//
+
 	ph2->plcyid_in = policy_in->sp.id;
 	ph2->plcyid_out = policy_out->sp.id;
 
@@ -537,7 +541,7 @@ long _IKED::pfkey_init_phase2( bool nail, u_int16_t plcytype, u_int32_t plcyid,
 
 	ph2->nailed = nail;
 
- 	//
+	//
 	// if the tunnel uses shared policy level, we
 	// use a remote ID of 0.0.0.0/0 regardless of
 	// the sa ID value
@@ -700,16 +704,15 @@ long _IKED::pfkey_recv_spadd( PFKI_MSG & msg )
 	// init phase2 now
 	//
 
-	if( policy->nailed )
+	if( policy->flags & PFLAG_NAILED )
 	{
 		log.txt( LLOG_DEBUG, "ii : calling init phase2 for nailed policy\n" );
 		pfkey_init_phase2( true, spinfo.sp.type, spinfo.sp.id, 0 );
 	}
 
-	if( policy->initial )
+	if( policy->flags & PFLAG_INITIAL )
 	{
-		policy->initial = false;
-
+		policy->flags &= ~PFLAG_INITIAL;
 		log.txt( LLOG_DEBUG, "ii : calling init phase2 for initial policy\n" );
 		pfkey_init_phase2( true, spinfo.sp.type, spinfo.sp.id, 0 );
 	}
@@ -1217,9 +1220,21 @@ long _IKED::pfkey_send_getspi( IDB_POLICY * policy, IDB_PH2 * ph2 )
 	// determine natt port usage
 	//
 
+	IKE_SADDR saddr_l = ph2->tunnel->saddr_l;
+	IKE_SADDR saddr_r = ph2->tunnel->saddr_r;
+
 	bool use_ports = false;
+
 	if( ph2->tunnel->natt_version != IPSEC_NATT_NONE )
+	{
+		if( ph2->tunnel->natt_version == IPSEC_NATT_CISCO )
+		{
+			socket_lookup_port( saddr_l, true );
+			set_sockport( saddr_r.saddr, ph2->tunnel->peer->natt_port );
+		}
+
 		use_ports = true;
+	}
 
 	//
 	// convert source and destination
@@ -1232,14 +1247,14 @@ long _IKED::pfkey_send_getspi( IDB_POLICY * policy, IDB_PH2 * ph2 )
 	switch( policy->sp.dir )
 	{
 		case IPSEC_DIR_INBOUND:
-			cpy_sockaddr( ph2->tunnel->saddr_r.saddr, sainfo.paddr_src.saddr, use_ports );
-			cpy_sockaddr( ph2->tunnel->saddr_l.saddr, sainfo.paddr_dst.saddr, use_ports );
+			cpy_sockaddr( saddr_r.saddr, sainfo.paddr_src.saddr, use_ports );
+			cpy_sockaddr( saddr_l.saddr, sainfo.paddr_dst.saddr, use_ports );
 			sainfo.seq = ph2->seqid_in;
 			break;
 
 		case IPSEC_DIR_OUTBOUND:
-			cpy_sockaddr( ph2->tunnel->saddr_l.saddr, sainfo.paddr_src.saddr, use_ports );
-			cpy_sockaddr( ph2->tunnel->saddr_r.saddr, sainfo.paddr_dst.saddr, use_ports );
+			cpy_sockaddr( saddr_l.saddr, sainfo.paddr_src.saddr, use_ports );
+			cpy_sockaddr( saddr_r.saddr, sainfo.paddr_dst.saddr, use_ports );
 			sainfo.seq = ph2->seqid_out;
 			break;
 	}
@@ -1294,8 +1309,6 @@ long _IKED::pfkey_send_getspi( IDB_POLICY * policy, IDB_PH2 * ph2 )
 		// step through all proposals and
 		// store the request id to be used
 		// when sending the update message.
-		// we also need to acquire the spi
-		// to be used for outbound sas
 		//
 
 		if( policy->sp.dir == IPSEC_DIR_INBOUND )
@@ -1309,15 +1322,23 @@ long _IKED::pfkey_send_getspi( IDB_POLICY * policy, IDB_PH2 * ph2 )
 				// match the protocol type
 				//
 
-				if( proposal->proto == proto )
-				{
-					//
-					// copy the request id
-					//
+				if( proposal->proto != proto )
+					continue;
 
-					proposal->reqid = ( uint16_t ) sainfo.sa2.reqid;
-				}
+				//
+				// copy the request id
+				//
+
+				proposal->reqid = ( uint16_t ) sainfo.sa2.reqid;
 			}
+
+			//
+			// inbound spis are dictated by
+			// our kernel. we will need to
+			// wait for an update response
+			//
+
+			ph2->spicount++;
 		}
 
 		if( policy->sp.dir == IPSEC_DIR_OUTBOUND )
@@ -1339,41 +1360,31 @@ long _IKED::pfkey_send_getspi( IDB_POLICY * policy, IDB_PH2 * ph2 )
 				// obtain the spi value
 				//
 
-				if( proposal->proto == proto )
+				if( proposal->proto != proto )
+					continue;
+
+				//
+				// copy the request id
+				//
+
+				proposal->reqid = ( uint16_t ) sainfo.sa2.reqid;
+
+				//
+				// copy the spi value
+				//
+
+				if( proto != ISAKMP_PROTO_IPCOMP )
 				{
-					//
-					// copy the request id
-					//
-
-					proposal->reqid = ( uint16_t ) sainfo.sa2.reqid;
-
-					//
-					// copy the spi value
-					//
-
-					if( proto != ISAKMP_PROTO_IPCOMP )
-					{
-						sainfo.range.min = ntohl( proposal->spi.spi );
-						sainfo.range.max = ntohl( proposal->spi.spi );
-					}
-					else
-					{
-						sainfo.range.min = ntohs( proposal->spi.cpi );
-						sainfo.range.max = ntohs( proposal->spi.cpi );
-					}
+					sainfo.range.min = ntohl( proposal->spi.spi );
+					sainfo.range.max = ntohl( proposal->spi.spi );
+				}
+				else
+				{
+					sainfo.range.min = ntohs( proposal->spi.cpi );
+					sainfo.range.max = ntohs( proposal->spi.cpi );
 				}
 			}
 		}
-		else
-		{
-			//
-			// inbound spis are dictated by
-			// our kernel. we will need to
-			// wait for an update response
-			//
-
-			ph2->spicount++;
-		}
 
 		//
 		// send our getspi request to pfkey
@@ -1413,6 +1424,26 @@ long _IKED::pfkey_send_update( IDB_PH2 * ph2, IKE_PROPOSAL * proposal, BDATA & e
 	PFKI_SAINFO sainfo;
 	memset( &sainfo, 0, sizeof( sainfo ) );
 
+	//
+	// determine natt port usage
+	//
+
+	IKE_SADDR saddr_l = ph2->tunnel->saddr_l;
+	IKE_SADDR saddr_r = ph2->tunnel->saddr_r;
+
+	bool use_ports = false;
+
+	if( ph2->tunnel->natt_version != IPSEC_NATT_NONE )
+	{
+		if( ph2->tunnel->natt_version == IPSEC_NATT_CISCO )
+		{
+			socket_lookup_port( saddr_l, true );
+			set_sockport( saddr_r.saddr, ph2->tunnel->peer->natt_port );
+		}
+
+		use_ports = true;
+	}
+
 	//
 	// convert mode
 	//
@@ -1505,10 +1536,22 @@ long _IKED::pfkey_send_update( IDB_PH2 * ph2, IKE_PROPOSAL * proposal, BDATA & e
 					sainfo.sa.auth = SADB_AALG_MD5HMAC;
 					break;
 
-				case ISAKMP_AUTH_HMAC_SHA:
+				case ISAKMP_AUTH_HMAC_SHA1:
 					sainfo.sa.auth = SADB_AALG_SHA1HMAC;
 					break;
 
+				case ISAKMP_AUTH_HMAC_SHA2_256:
+					sainfo.sa.auth = SADB_X_AALG_SHA2_256HMAC;
+					break;
+
+				case ISAKMP_AUTH_HMAC_SHA2_384:
+					sainfo.sa.auth = SADB_X_AALG_SHA2_384HMAC;
+					break;
+
+				case ISAKMP_AUTH_HMAC_SHA2_512:
+					sainfo.sa.auth = SADB_X_AALG_SHA2_512HMAC;
+					break;
+
 				default:
 				{
 					log.txt( LLOG_ERROR,
@@ -1521,37 +1564,68 @@ long _IKED::pfkey_send_update( IDB_PH2 * ph2, IKE_PROPOSAL * proposal, BDATA & e
 			}
 
 #ifdef OPT_NATT
+# ifndef __APPLE__
 
 			switch( proposal->encap )
 			{
+				case ISAKMP_ENCAP_TUNNEL:
+					if( ph2->tunnel->natt_version == IPSEC_NATT_CISCO )
+						sainfo.natt.type = UDP_ENCAP_ESPINUDP;
+					break;
+
 				case ISAKMP_ENCAP_VXX_UDP_TUNNEL:
 				case ISAKMP_ENCAP_RFC_UDP_TUNNEL:
 				case ISAKMP_ENCAP_VXX_UDP_TRANSPORT:
 				case ISAKMP_ENCAP_RFC_UDP_TRANSPORT:
-				{
 					if( ph2->tunnel->natt_version >= IPSEC_NATT_V02 )
 						sainfo.natt.type = UDP_ENCAP_ESPINUDP;
 					else
 						sainfo.natt.type = UDP_ENCAP_ESPINUDP_NON_IKE;
-
-					switch( dir )
-					{
-						case IPSEC_DIR_INBOUND:
-							get_sockport( ph2->tunnel->saddr_r.saddr, sainfo.natt.port_src );
-							get_sockport( ph2->tunnel->saddr_l.saddr, sainfo.natt.port_dst );
-							break;
-
-						case IPSEC_DIR_OUTBOUND:
-							get_sockport( ph2->tunnel->saddr_l.saddr, sainfo.natt.port_src );
-							get_sockport( ph2->tunnel->saddr_r.saddr, sainfo.natt.port_dst );
-							break;
-					}
-
 					break;
+			}
+
+			if( sainfo.natt.type )
+			{
+				switch( dir )
+				{
+					case IPSEC_DIR_INBOUND:
+						get_sockport( saddr_r.saddr, sainfo.natt.port_src );
+						get_sockport( saddr_l.saddr, sainfo.natt.port_dst );
+						break;
+
+					case IPSEC_DIR_OUTBOUND:
+						get_sockport( saddr_l.saddr, sainfo.natt.port_src );
+						get_sockport( saddr_r.saddr, sainfo.natt.port_dst );
+						break;
 				}
 			}
 
-#endif
+# else // __APPLE__
+
+			switch( proposal->encap )
+			{
+				case ISAKMP_ENCAP_TUNNEL:
+					if( ph2->tunnel->natt_version == IPSEC_NATT_CISCO )
+						sainfo.sa.flags |= SADB_X_EXT_NATT;
+					break;
+
+				case ISAKMP_ENCAP_VXX_UDP_TUNNEL:
+				case ISAKMP_ENCAP_RFC_UDP_TUNNEL:
+				case ISAKMP_ENCAP_VXX_UDP_TRANSPORT:
+				case ISAKMP_ENCAP_RFC_UDP_TRANSPORT:
+					sainfo.sa.flags |= SADB_X_EXT_NATT;
+					break;
+			}
+
+			if( sainfo.sa.flags & SADB_X_EXT_NATT )
+			{
+				sainfo.sa.flags |= SADB_X_EXT_NATT_KEEPALIVE;
+				get_sockport( saddr_l.saddr, sainfo.sa.natt_port );
+				sainfo.sa.natt_port = ntohs( sainfo.sa.natt_port );
+			}
+
+# endif // __APPLE__
+#endif // OPT_NATT
 
 			break;
 		}
@@ -1571,6 +1645,18 @@ long _IKED::pfkey_send_update( IDB_PH2 * ph2, IKE_PROPOSAL * proposal, BDATA & e
 					sainfo.sa.auth = SADB_AALG_SHA1HMAC;
 					break;
 
+				case ISAKMP_AH_SHA256:
+					sainfo.sa.auth = SADB_X_AALG_SHA2_256HMAC;
+					break;
+
+				case ISAKMP_AH_SHA384:
+					sainfo.sa.auth = SADB_X_AALG_SHA2_384HMAC;
+					break;
+
+				case ISAKMP_AH_SHA512:
+					sainfo.sa.auth = SADB_X_AALG_SHA2_512HMAC;
+					break;
+
 				default:
 				{
 					log.txt( LLOG_ERROR,
@@ -1642,14 +1728,6 @@ long _IKED::pfkey_send_update( IDB_PH2 * ph2, IKE_PROPOSAL * proposal, BDATA & e
 	sainfo.ltime_soft.bytes		*= PFKEY_SOFT_LIFETIME_RATE;
 	sainfo.ltime_soft.bytes		/= 100;
 
-	//
-	// determine natt port usage
-	//
-
-	bool use_ports = false;
-	if( ph2->tunnel->natt_version != IPSEC_NATT_NONE )
-		use_ports = true;
-
 	//
 	// convert source and destination
 	// and store sequence ids
@@ -1661,14 +1739,14 @@ long _IKED::pfkey_send_update( IDB_PH2 * ph2, IKE_PROPOSAL * proposal, BDATA & e
 	switch( dir )
 	{
 		case IPSEC_DIR_INBOUND:
-			cpy_sockaddr( ph2->tunnel->saddr_r.saddr, sainfo.paddr_src.saddr, use_ports );
-			cpy_sockaddr( ph2->tunnel->saddr_l.saddr, sainfo.paddr_dst.saddr, use_ports );
+			cpy_sockaddr( saddr_r.saddr, sainfo.paddr_src.saddr, use_ports );
+			cpy_sockaddr( saddr_l.saddr, sainfo.paddr_dst.saddr, use_ports );
 			sainfo.seq = ph2->seqid_in;
 			break;
 
 		case IPSEC_DIR_OUTBOUND:
-			cpy_sockaddr( ph2->tunnel->saddr_l.saddr, sainfo.paddr_src.saddr, use_ports );
-			cpy_sockaddr( ph2->tunnel->saddr_r.saddr, sainfo.paddr_dst.saddr, use_ports );
+			cpy_sockaddr( saddr_l.saddr, sainfo.paddr_src.saddr, use_ports );
+			cpy_sockaddr( saddr_r.saddr, sainfo.paddr_dst.saddr, use_ports );
 			sainfo.seq = ph2->seqid_out;
 			break;
 	}
@@ -1717,10 +1795,19 @@ long _IKED::pfkey_send_update( IDB_PH2 * ph2, IKE_PROPOSAL * proposal, BDATA & e
 		"ii : - auth = %s\n"
 		"ii : - akey = %i bits\n"
 		"ii : - hard = %i\n"
+#ifndef OPT_NATT
+		"ii : - soft = %i\n",
+#else
 		"ii : - soft = %i\n"
+# ifndef __APPLE__
 		"ii : - natt = %s\n"
 		"ii : - nsrc = %i\n"
 		"ii : - ndst = %i\n",
+# else
+		"ii : - natt = %s\n"
+		"ii : - port = %i\n",
+# endif // __APPLE__
+#endif // OPT_NATT
 		ntohl( sainfo.sa.spi ),
 		txtid_src,
 		txtid_dst,
@@ -1729,10 +1816,19 @@ long _IKED::pfkey_send_update( IDB_PH2 * ph2, IKE_PROPOSAL * proposal, BDATA & e
 		pfki.name( NAME_SAAUTH, sainfo.sa.auth ),
 		sainfo.akey.length * 8,
 		long( sainfo.ltime_hard.addtime ),
+#ifndef OPT_NATT
+		long( sainfo.ltime_soft.addtime ) );
+#else
 		long( sainfo.ltime_soft.addtime ),
+# ifndef __APPLE__
 		pfki.name( NAME_NTTYPE, sainfo.natt.type ),
 		ntohs( sainfo.natt.port_src ),
 		ntohs( sainfo.natt.port_dst ) );
+# else
+		pfki.name( NAME_NTTYPE, UDP_ENCAP_ESPINUDP ),
+		ntohs( sainfo.sa.natt_port ) );
+# endif // __APPLE__
+#endif // OPT_NATT
 
 	pfki.send_update( sainfo );
 
@@ -1754,16 +1850,32 @@ long _IKED::pfkey_send_delete( IDB_PH2 * ph2 )
 		PFKI_SAINFO sainfo;
 		memset( &sainfo, 0, sizeof( sainfo ) );
 
+		//
+		// determine natt port usage
+		//
+
+		IKE_SADDR saddr_l = ph2->tunnel->saddr_l;
+		IKE_SADDR saddr_r = ph2->tunnel->saddr_r;
+
+		bool use_ports = false;
+
+		if( ph2->tunnel->natt_version != IPSEC_NATT_NONE )
+		{
+			if( ph2->tunnel->natt_version == IPSEC_NATT_CISCO )
+			{
+				socket_lookup_port( saddr_l, true );
+				set_sockport( saddr_r.saddr, ph2->tunnel->peer->natt_port );
+			}
+
+			use_ports = true;
+		}
+
 		//
 		// determine the sa endpoint addresses
 		//
 
-		bool use_ports = false;
-		if( ph2->tunnel->natt_version != IPSEC_NATT_NONE )
-			use_ports = true;
-
-		cpy_sockaddr( ph2->tunnel->saddr_l.saddr, sainfo.paddr_src.saddr, use_ports );
-		cpy_sockaddr( ph2->tunnel->saddr_r.saddr, sainfo.paddr_dst.saddr, use_ports );
+		cpy_sockaddr( saddr_l.saddr, sainfo.paddr_src.saddr, use_ports );
+		cpy_sockaddr( saddr_r.saddr, sainfo.paddr_dst.saddr, use_ports );
 
 		char txtid_src[ LIBIKE_MAX_TEXTP2ID ];
 		char txtid_dst[ LIBIKE_MAX_TEXTP2ID ];
@@ -1816,16 +1928,32 @@ long _IKED::pfkey_send_delete( IDB_PH2 * ph2 )
 		PFKI_SAINFO sainfo;
 		memset( &sainfo, 0, sizeof( sainfo ) );
 
+		//
+		// determine natt port usage
+		//
+
+		IKE_SADDR saddr_l = ph2->tunnel->saddr_l;
+		IKE_SADDR saddr_r = ph2->tunnel->saddr_r;
+
+		bool use_ports = false;
+
+		if( ph2->tunnel->natt_version != IPSEC_NATT_NONE )
+		{
+			if( ph2->tunnel->natt_version == IPSEC_NATT_CISCO )
+			{
+				socket_lookup_port( saddr_l, true );
+				set_sockport( saddr_r.saddr, ph2->tunnel->peer->natt_port );
+			}
+
+			use_ports = true;
+		}
+
 		//
 		// determine the sa endpoint addresses
 		//
 
-		bool use_ports = false;
-		if( ph2->tunnel->natt_version != IPSEC_NATT_NONE )
-			use_ports = true;
-
-		cpy_sockaddr( ph2->tunnel->saddr_r.saddr, sainfo.paddr_src.saddr, use_ports );
-		cpy_sockaddr( ph2->tunnel->saddr_l.saddr, sainfo.paddr_dst.saddr, use_ports );
+		cpy_sockaddr( saddr_r.saddr, sainfo.paddr_src.saddr, use_ports );
+		cpy_sockaddr( saddr_l.saddr, sainfo.paddr_dst.saddr, use_ports );
 
 		char txtid_src[ LIBIKE_MAX_TEXTP2ID ];
 		char txtid_dst[ LIBIKE_MAX_TEXTP2ID ];
