@@ -57,23 +57,27 @@
 # define D2I_RSA_CONST 
 #endif
 
-// openssl password callback
-
-int keyfile_cb( char * buf, int size, int rwflag, void * userdata )
+void log_openssl_errors()
 {
-	BDATA * fpass = ( BDATA * ) userdata;
+	const char * file;
+	int line;
 
-	memset( buf, 0, size );
+	while( true )
+	{
+		unsigned long code = ERR_get_error_line( &file, &line );
+		if( code == 0 )
+			break;
 
-	if( size > int( fpass->size() ) )
-		size = int( fpass->size() );
-
-	memcpy( buf, fpass->buff(), size );
-
-	return size;
+		char buf[ 1024 ];
+		ERR_error_string_n( code, buf, sizeof( buf ) );
+		iked.log.txt( LLOG_LOUD,
+				"!! : libeay : %s:%i\n"
+				"!! : %s\n",
+				file, line, buf );
+	}
 }
 
-bool _IKED::cert_2_bdata( BDATA & cert, X509 * x509 )
+bool cert_2_bdata( BDATA & cert, X509 * x509 )
 {
 	int size = i2d_X509( x509, NULL );
 	cert.size( size );
@@ -96,17 +100,127 @@ bool bdata_2_cert( X509 ** x509, BDATA & cert )
 	return true;
 }
 
-bool _IKED::bdata_2_cert( X509 ** x509, BDATA & cert )
+bool prvkey_rsa_2_bdata( BDATA & prvkey, RSA * rsa )
 {
-	D2I_X509_CONST unsigned char * cert_buff = cert.buff();
+	int size = i2d_RSAPrivateKey( rsa, NULL );
+	prvkey.size( size );
 
-	*x509 = d2i_X509( NULL, &cert_buff, ( long ) cert.size() );
-	if( *x509 == NULL )
+	unsigned char * prvkey_buff = prvkey.buff();
+	if( i2d_RSAPrivateKey( rsa, &prvkey_buff ) < size )
 		return false;
 
 	return true;
 }
 
+bool bdata_2_prvkey_rsa( RSA ** rsa, BDATA & prvkey )
+{
+	D2I_RSA_CONST unsigned char * prvkey_buff = prvkey.buff();
+
+	*rsa = d2i_RSAPrivateKey( NULL, &prvkey_buff, ( long ) prvkey.size() );
+	if( *rsa == NULL )
+		return false;
+
+	return true;
+}
+
+bool pubkey_rsa_2_bdata( BDATA & pubkey, RSA * rsa )
+{
+	int size = i2d_RSAPublicKey( rsa, NULL );
+	pubkey.size( size );
+
+	unsigned char * pubkey_buff = pubkey.buff();
+	if( i2d_RSAPublicKey( rsa, &pubkey_buff ) < size )
+		return false;
+
+	return true;
+}
+
+bool bdata_2_pubkey_rsa( RSA ** rsa, BDATA & pubkey )
+{
+	D2I_RSA_CONST unsigned char * pubkey_buff = pubkey.buff();
+
+	*rsa = d2i_RSAPublicKey( NULL, &pubkey_buff, ( long ) pubkey.size() );
+	if( *rsa == NULL )
+		return false;
+
+	return true;
+}
+
+// openssl pem password callback
+
+int password_cb( char * buf, int size, int rwflag, void * userdata )
+{
+	BDATA * fpass = ( BDATA * ) userdata;
+	if( !fpass->size() )
+		return 0;
+
+	memset( buf, 0, size );
+	if( size > fpass->size() )
+		size = fpass->size();
+
+	memcpy( buf, fpass->buff(), size );
+
+	return size;
+}
+
+bool cert_load_pem( BDATA & cert, FILE * fp, bool ca, BDATA & pass )
+{
+	fseek( fp, 0, SEEK_SET );
+
+	X509 * x509 = PEM_read_X509( fp, NULL, password_cb, &pass );
+	if( x509 == NULL )
+		return false;
+
+	bool converted = cert_2_bdata( cert, x509 );
+	X509_free( x509 );
+
+	return converted;
+}
+
+bool cert_load_p12( BDATA & cert, FILE * fp, bool ca, BDATA & pass )
+{
+	// PKCS12 required a null terminated password
+	BDATA nullpass;
+	nullpass.set( pass );
+	nullpass.add( "", 1 );
+
+	fseek( fp, 0, SEEK_SET );
+
+	PKCS12 * p12 = d2i_PKCS12_fp( fp, NULL );
+	if( p12 == NULL )
+		return false;
+
+	X509 * x509 = NULL;
+
+	if( ca )
+	{
+		STACK_OF( X509 ) * stack = NULL;
+
+		if( PKCS12_parse( p12, nullpass.text(), NULL, NULL, &stack ) )
+		{
+			if( stack != NULL )
+			{
+				if( sk_X509_value( stack, 0 ) != NULL )
+					x509 = sk_X509_value( stack, 0 );
+
+				sk_X509_free( stack );
+			}
+		}
+	}
+	else
+		PKCS12_parse( p12, nullpass.text(), NULL, &x509, NULL );
+
+	PKCS12_free( p12 );
+
+	if( x509 == NULL )
+		return false;
+
+	bool converted = cert_2_bdata( cert, x509 );
+	X509_free( x509 );
+
+	return converted;
+}
+
 long _IKED::cert_load( BDATA & cert, char * fpath, bool ca, BDATA & pass )
 {
 #ifdef WIN32
@@ -130,45 +244,71 @@ long _IKED::cert_load( BDATA & cert, char * fpath, bool ca, BDATA & pass )
 	fclose( fp );
 
 	if( !loaded )
+	{
+		log_openssl_errors();
 		return FILE_FAIL;
+	}
 
 	return FILE_OK;
 }
 
-bool _IKED::cert_load_pem( BDATA & cert, FILE * fp, bool ca, BDATA & pass )
+bool cert_load_pem( BDATA & cert, BDATA & input, bool ca, BDATA & pass )
 {
-	fseek( fp, 0, SEEK_SET );
+	BIO * bp = BIO_new( BIO_s_mem() );
+	if( bp == NULL )
+		return false;
+
+	if( BIO_write( bp, input.buff(), ( int ) input.size() ) < 0 )
+	{
+		BIO_free_all( bp );
+		return false;
+	}
+
+	X509 * x509 = PEM_read_bio_X509( bp, NULL, password_cb, &pass );
+
+	BIO_free_all( bp );
 
-	X509 * x509 = PEM_read_X509( fp, NULL, keyfile_cb, &pass );
 	if( x509 == NULL )
 		return false;
 
-	cert_2_bdata( cert, x509 );
-
+	bool converted = cert_2_bdata( cert, x509 );
 	X509_free( x509 );
 
-	return true;
+	return converted;
 }
 
-bool _IKED::cert_load_p12( BDATA & cert, FILE * fp, bool ca, BDATA & pass )
+bool cert_load_p12( BDATA & cert, BDATA & input, bool ca, BDATA & pass )
 {
-	fseek( fp, 0, SEEK_SET );
+	// PKCS12 required a null terminated password
+	BDATA nullpass;
+	nullpass.set( pass );
+	nullpass.add( "", 1 );
+
+	BIO * bp = BIO_new( BIO_s_mem() );
+	if( bp == NULL )
+		return false;
+
+	if( BIO_write( bp, input.buff(), ( int ) input.size() ) < 0 )
+	{
+		BIO_free_all( bp );
+		return false;
+	}
+
+	PKCS12 * p12 = d2i_PKCS12_bio( bp, NULL );
+
+	BIO_free_all( bp );
 
-	PKCS12 * p12 = d2i_PKCS12_fp( fp, NULL );
 	if( p12 == NULL )
 		return false;
 
 	X509 * x509 = NULL;
-
-	BDATA passnull;
-	passnull.set( pass );
-	passnull.add( 0, 1 );
+	EVP_PKEY * evp_pkey = NULL;
 
 	if( ca )
 	{
 		STACK_OF( X509 ) * stack = NULL;
 
-		if( PKCS12_parse( p12, ( const char * ) passnull.buff(), NULL, NULL, &stack ) )
+		if( PKCS12_parse( p12, ( const char * ) nullpass.buff(), NULL, NULL, &stack ) )
 		{
 			if( stack != NULL )
 			{
@@ -180,44 +320,31 @@ bool _IKED::cert_load_p12( BDATA & cert, FILE * fp, bool ca, BDATA & pass )
 		}
 	}
 	else
-		PKCS12_parse( p12, ( const char * ) passnull.buff(), NULL, &x509, NULL );
+		PKCS12_parse( p12, ( const char * ) nullpass.buff(), &evp_pkey, &x509, NULL );
 
+	EVP_PKEY_free( evp_pkey );
 	PKCS12_free( p12 );
 
 	if( x509 == NULL )
 		return false;
 
-	cert_2_bdata( cert, x509 );
+	bool converted = cert_2_bdata( cert, x509 );
 	X509_free( x509 );
 
-	return true;
+	return converted;
 }
 
-long _IKED::cert_save( BDATA & cert, char * fpath )
+long _IKED::cert_load( BDATA & cert, BDATA & input, bool ca, BDATA & pass )
 {
-	X509 * x509;
-	if( !bdata_2_cert( &x509, cert ) )
+	bool loaded = cert_load_pem( cert, input, ca, pass );
+	if( !loaded )
+		loaded = cert_load_p12( cert, input, ca, pass );
+
+	if( !loaded )
+	{
+		log_openssl_errors();
 		return FILE_FAIL;
-
-#ifdef WIN32
-
-	FILE * fp;
-	if( fopen_s( &fp, fpath, "wb" ) )
-		return FILE_FAIL;
-
-#else
-
-	FILE * fp = fopen( fpath, "wb" );
-	if( !fp )
-		return FILE_FAIL;
-
-#endif
-	
-	PEM_write_X509( fp, x509 );
-
-	fclose( fp );
-
-	X509_free( x509 );
+	}
 
 	return FILE_OK;
 }
@@ -269,7 +396,6 @@ bool _IKED::cert_subj( BDATA & cert, BDATA & subj )
 	}
 
 	short size = i2d_X509_NAME( x509_name, NULL );
-
 	if( size > LIBIKE_MAX_VARID )
 	{
 		X509_free( x509 );
@@ -529,6 +655,8 @@ bool _IKED::text_asn1( BDATA & text, BDATA & asn1 )
 	return false;
 }
 
+// openssl certificate verify callback
+
 static int verify_cb( int ok, X509_STORE_CTX * store_ctx )
 {
 	if( !ok )
@@ -594,6 +722,7 @@ STACK_OF( X509 ) * build_cert_stack( IDB_LIST_CERT & certs, BDATA & leaf )
 				continue;
 
 			unsigned long hash2 = X509_issuer_name_hash( x509_cert2 );
+			X509_free( x509_cert2 );
 
 			if( hash1 == hash2 )
 				break;
@@ -602,7 +731,10 @@ STACK_OF( X509 ) * build_cert_stack( IDB_LIST_CERT & certs, BDATA & leaf )
 		if( index2 < certs.count() )
 			sk_X509_push( chain, x509_cert1 );	
 		else
+		{
 			leaf = cert1;
+			X509_free( x509_cert1 );
+		}
 	}
 
 	return chain;
@@ -700,36 +832,64 @@ bool _IKED::cert_verify( IDB_LIST_CERT & certs, BDATA & ca, BDATA & cert )
 			//
 
 			result = X509_verify_cert( store_ctx );
-			X509_STORE_CTX_cleanup( store_ctx );
+			X509_STORE_CTX_free( store_ctx );
 		}
 
 		X509_free( x509_cert );
 	}
 
-	//
-	// destroy certificate chain
-	//
-
-	while( sk_X509_num( chain ) > 0 )
-	{
-		x509_cert = sk_X509_pop( chain );
-		if( x509_cert == NULL )
-			break;
-
-		X509_free( x509_cert );
-	}
-
 	//
 	// cleanup
 	//
 
+	sk_X509_pop_free( chain, X509_free );
 	X509_free( x509_ca );
 	X509_STORE_free( store );
 
 	return ( result > 0 );
 }
 
-long _IKED::prvkey_rsa_load( EVP_PKEY ** evp_pkey, char * fpath, BDATA & pass )
+bool prvkey_rsa_load_pem( BDATA & prvkey, FILE * fp, BDATA & pass )
+{
+	fseek( fp, 0, SEEK_SET );
+
+	EVP_PKEY * evp_pkey = PEM_read_PrivateKey( fp, NULL, password_cb, &pass );
+	if( evp_pkey == NULL )
+		return false;
+
+	bool converted = prvkey_rsa_2_bdata( prvkey, evp_pkey->pkey.rsa );
+	EVP_PKEY_free( evp_pkey );
+
+	return converted;
+}
+
+bool prvkey_rsa_load_p12( BDATA & prvkey, FILE * fp, BDATA & pass )
+{
+	// PKCS12 required a null terminated password
+	BDATA nullpass;
+	nullpass.set( pass );
+	nullpass.add( "", 1 );
+
+	fseek( fp, 0, SEEK_SET );
+
+	PKCS12 * p12 = d2i_PKCS12_fp( fp, NULL );
+	if( p12 == NULL )
+		return false;
+
+	EVP_PKEY * evp_pkey;
+	PKCS12_parse( p12, nullpass.text(), &evp_pkey, NULL, NULL );
+	PKCS12_free( p12 );
+
+	if( evp_pkey == NULL )
+		return false;
+
+	bool converted = prvkey_rsa_2_bdata( prvkey, evp_pkey->pkey.rsa );
+	EVP_PKEY_free( evp_pkey );
+
+	return converted;
+}
+
+long _IKED::prvkey_rsa_load( BDATA & prvkey, char * fpath, BDATA & pass )
 {
 #ifdef WIN32
 
@@ -745,103 +905,165 @@ long _IKED::prvkey_rsa_load( EVP_PKEY ** evp_pkey, char * fpath, BDATA & pass )
 
 #endif
 
-	bool loaded = prvkey_rsa_load_pem( evp_pkey, fp, pass );
+	bool loaded = prvkey_rsa_load_pem( prvkey, fp, pass );
 	if( !loaded )
-		loaded = prvkey_rsa_load_p12( evp_pkey, fp, pass );
+		loaded = prvkey_rsa_load_p12( prvkey, fp, pass );
 
 	fclose( fp );
 
 	if( !loaded )
+	{
+		log_openssl_errors();
 		return FILE_FAIL;
+	}
 
 	return FILE_OK;
 }
 
-bool _IKED::prvkey_rsa_load_pem( EVP_PKEY ** evp_pkey, FILE * fp, BDATA & pass )
+bool prvkey_rsa_load_pem( BDATA & prvkey, BDATA & input, BDATA & pass )
 {
-	fseek( fp, 0, SEEK_SET );
-
-	*evp_pkey = PEM_read_PrivateKey( fp, NULL, keyfile_cb, &pass );
-	if( *evp_pkey == NULL )
+	BIO * bp = BIO_new( BIO_s_mem() );
+	if( bp == NULL )
 		return false;
 
-	return true;
+	if( BIO_write( bp, input.buff(), ( int ) input.size() ) < 0 )
+	{
+		BIO_free_all( bp );
+		return false;
+	}
+
+	EVP_PKEY * evp_pkey = PEM_read_bio_PrivateKey( bp, NULL, password_cb, &pass );
+
+	BIO_free_all( bp );
+
+	if( evp_pkey == NULL )
+		return false;
+
+	bool converted = prvkey_rsa_2_bdata( prvkey, evp_pkey->pkey.rsa );
+	EVP_PKEY_free( evp_pkey );
+
+	return converted;
 }
 
-bool _IKED::prvkey_rsa_load_p12( EVP_PKEY ** evp_pkey, FILE * fp, BDATA & pass )
+bool prvkey_rsa_load_p12( BDATA & prvkey, BDATA & input, BDATA & pass )
 {
-	fseek( fp, 0, SEEK_SET );
+	// PKCS12 required a null terminated password
+	BDATA nullpass;
+	nullpass.set( pass );
+	nullpass.add( "", 1 );
+
+	BIO * bp = BIO_new( BIO_s_mem() );
+	if( bp == NULL )
+		return false;
+
+	if( BIO_write( bp, input.buff(), ( int ) input.size() ) < 0 )
+	{
+		BIO_free_all( bp );
+		return false;
+	}
+
+	PKCS12 * p12 = d2i_PKCS12_bio( bp, NULL );
+
+	BIO_free_all( bp );
 
-	PKCS12 * p12 = d2i_PKCS12_fp( fp, NULL );
 	if( p12 == NULL )
 		return false;
 
-	BDATA passnull;
-	passnull.set( pass );
-	passnull.add( 0, 1 );
-
-	PKCS12_parse( p12, ( const char * ) passnull.buff(), evp_pkey, NULL, NULL );
+	EVP_PKEY * evp_pkey;
+	PKCS12_parse( p12, nullpass.text(), &evp_pkey, NULL, NULL );
 	PKCS12_free( p12 );
 
-	if( *evp_pkey == NULL )
+	if( evp_pkey == NULL )
 		return false;
 
-	return true;
+	bool converted = prvkey_rsa_2_bdata( prvkey, evp_pkey->pkey.rsa );
+	EVP_PKEY_free( evp_pkey );
+
+	return converted;
 }
 
-bool _IKED::pubkey_rsa_read( BDATA & cert, EVP_PKEY ** evp_pkey )
+long _IKED::prvkey_rsa_load( BDATA & prvkey, BDATA & input, BDATA & pass )
+{
+	bool loaded = prvkey_rsa_load_pem( prvkey, input, pass );
+	if( !loaded )
+		loaded = prvkey_rsa_load_p12( prvkey, input, pass );
+
+	if( !loaded )
+	{
+		log_openssl_errors();
+		return FILE_FAIL;
+	}
+
+	return FILE_OK;
+}
+
+bool _IKED::pubkey_rsa_read( BDATA & cert, BDATA & pubkey )
 {
 	X509 * x509;
 	if( !bdata_2_cert( &x509, cert ) )
 		return false;
 
-	*evp_pkey = X509_get_pubkey( x509 );
+	EVP_PKEY * evp_pkey = X509_get_pubkey( x509 );
 
 	X509_free( x509 );
 
-	if( !( *evp_pkey ) )
+	if( evp_pkey == NULL )
 		return false;
 
-	return true;
+	bool result = pubkey_rsa_2_bdata( pubkey, evp_pkey->pkey.rsa );
+
+	EVP_PKEY_free( evp_pkey );
+
+	return result;
 }
 
-bool _IKED::prvkey_rsa_encrypt( EVP_PKEY * evp_pkey, BDATA & hash, BDATA & sign )
+bool _IKED::prvkey_rsa_encrypt( BDATA & prvkey, BDATA & hash, BDATA & sign )
 {
-	int size = RSA_size( evp_pkey->pkey.rsa );
+	RSA * rsa;
+	if( !bdata_2_prvkey_rsa( &rsa, prvkey ) )
+		return false;
 
+	int size = RSA_size( rsa );
 	sign.size( size );
 
 	size = RSA_private_encrypt(
 				( int ) hash.size(),
 				hash.buff(),
 				sign.buff(),
-				evp_pkey->pkey.rsa,
+				rsa,
 				RSA_PKCS1_PADDING );
 
 	if( size == -1 )
 		return false;
 
+	RSA_free( rsa );
+
 	sign.size( size );
 
 	return true;
 }
 
-bool _IKED::pubkey_rsa_decrypt( EVP_PKEY * evp_pkey, BDATA & sign, BDATA & hash )
+bool _IKED::pubkey_rsa_decrypt( BDATA & pubkey, BDATA & sign, BDATA & hash )
 {
-	int size = RSA_size( evp_pkey->pkey.rsa );
+	RSA * rsa;
+	if( !bdata_2_pubkey_rsa( &rsa, pubkey ) )
+		return false;
 
+	int size = RSA_size( rsa );
 	hash.size( size );
 
 	size = RSA_public_decrypt(
 				( int ) sign.size(),
 				sign.buff(),
 				hash.buff(),
-				evp_pkey->pkey.rsa,
+				rsa,
 				RSA_PKCS1_PADDING );
 
 	if( size == -1 )
 		return false;
 
+	RSA_free( rsa );
+
 	hash.size( size );
 
 	return true;
