@@ -146,6 +146,9 @@ const char * _IKED::find_name( long type, long id )
 			static const char * xform1 = "ah-md5";
 			static const char * xform2 = "ah-sha";
 			static const char * xform3 = "ah-des";
+			static const char * xform4 = "ah-sha256";
+			static const char * xform5 = "ah-sha384";
+			static const char * xform6 = "ah-sha512";
 
 			switch( id )
 			{
@@ -158,6 +161,15 @@ const char * _IKED::find_name( long type, long id )
 				case ISAKMP_AH_DES:
 					return xform3;
 
+				case ISAKMP_AH_SHA256:
+					return xform4;
+
+				case ISAKMP_AH_SHA384:
+					return xform5;
+
+				case ISAKMP_AH_SHA512:
+					return xform6;
+
 				default:
 					return unknown2;
 			}
@@ -464,16 +476,19 @@ const char * _IKED::find_name( long type, long id )
 		case NAME_MAUTH:
 		{
 			static const char * hash1 = "hmac-md5";
-			static const char * hash2 = "hmac-sha";
+			static const char * hash2 = "hmac-sha1";
 			static const char * hash3 = "des-mac";
 			static const char * hash4 = "kpdk";
+			static const char * hash5 = "hmac-sha2-256";
+			static const char * hash6 = "hmac-sha2-384";
+			static const char * hash7 = "hmac-sha2-512";
 
 			switch( id )
 			{
 				case ISAKMP_AUTH_HMAC_MD5:
 					return hash1;
 
-				case ISAKMP_AUTH_HMAC_SHA:
+				case ISAKMP_AUTH_HMAC_SHA1:
 					return hash2;
 
 				case ISAKMP_AUTH_DES_MAC:
@@ -482,6 +497,15 @@ const char * _IKED::find_name( long type, long id )
 				case ISAKMP_AUTH_KPDK:
 					return hash4;
 
+				case ISAKMP_AUTH_HMAC_SHA2_256:
+					return hash5;
+
+				case ISAKMP_AUTH_HMAC_SHA2_384:
+					return hash6;
+
+				case ISAKMP_AUTH_HMAC_SHA2_512:
+					return hash7;
+
 				default:
 					return unknown2;
 			}
@@ -492,6 +516,9 @@ const char * _IKED::find_name( long type, long id )
 			static const char * hash1 = "md5";
 			static const char * hash2 = "sha1";
 			static const char * hash3 = "tiger";
+			static const char * hash4 = "sha2-256";
+			static const char * hash5 = "sha2-384";
+			static const char * hash6 = "sha2-512";
 
 			switch( id )
 			{
@@ -504,6 +531,15 @@ const char * _IKED::find_name( long type, long id )
 				case IKE_HASH_TIGER:
 					return hash3;
 
+				case IKE_HASH_SHA2_256:
+					return hash4;
+
+				case IKE_HASH_SHA2_384:
+					return hash5;
+
+				case IKE_HASH_SHA2_512:
+					return hash6;
+
 				default:
 					return unknown2;
 			}
@@ -570,14 +606,16 @@ const char * _IKED::find_name( long type, long id )
 		case NAME_GROUP:
 		{
 			static const char * group0 = "none";
-			static const char * group1 = "modp-768";
-			static const char * group2 = "modp-1024";
-			static const char * group3 = "ecn-155";
-			static const char * group4 = "ecn-185";
-			static const char * group5 = "modp-1536";
-			static const char * group6 = "modp-2048";
-			static const char * group7 = "modp-3072";
-			static const char * group8 = "modp-4096";
+			static const char * group1 = "group1 ( modp-768 )";
+			static const char * group2 = "group2 ( modp-1024 )";
+			static const char * group3 = "group3 ( ecn-155 )";
+			static const char * group4 = "group4 ( ecn-185 )";
+			static const char * group5 = "group5 ( modp-1536 )";
+			static const char * group14 = "group14 ( modp-2048 )";
+			static const char * group15 = "group15 ( modp-3072 )";
+			static const char * group16 = "group16 ( modp-4096 )";
+			static const char * group17 = "group17 ( modp-6144 )";
+			static const char * group18 = "group18 ( modp-8192 )";
 
 			switch( id )
 			{
@@ -600,13 +638,19 @@ const char * _IKED::find_name( long type, long id )
 					return group5;
 
 				case IKE_GRP_GROUP14:
-					return group6;
+					return group14;
 
 				case IKE_GRP_GROUP15:
-					return group7;
+					return group15;
 
 				case IKE_GRP_GROUP16:
-					return group8;
+					return group16;
+
+				case IKE_GRP_GROUP17:
+					return group17;
+
+				case IKE_GRP_GROUP18:
+					return group18;
 
 				default:
 					return unknown2;
