@@ -379,102 +379,6 @@ void _IKED::text_ph2id( char * text, IKE_PH2ID * ph2id )
 	}
 }
 
-bool _IKED::find_addr_r( sockaddr_in & raddr, unsigned short rport, char * rname )
-{
-	//
-	// trim whitespaces from the
-	// hostname or address string
-	//
-
-	if( !rname  )
-		return false;
-
-	if( !( *rname ) )
-		return false;
-
-	while( rname && *rname == ' ' )
-		rname++;
-
-	//
-	// determine if this is an ip
-	// address or hostname. convert
-	// this into a sockaddr struct
-	//
-
-	long rsize = sizeof( sockaddr_in );
-	memset( &raddr, 0, rsize );
-
-	if( isdigit( rname[ 0 ] ) )
-	{
-		//
-		// looks like an address 
-		//
-
-		raddr.sin_family		= AF_INET;
-		raddr.sin_addr.s_addr	= inet_addr( rname );
-		raddr.sin_port			= htons( rport );
-	}
-	else
-	{
-		//
-		// looks like a hostname
-		//
-
-		struct hostent * hp = gethostbyname( rname );
-		if( !hp )
-			return false;
-
-		memcpy( &raddr.sin_addr, hp->h_addr, hp->h_length );
-		raddr.sin_family = hp->h_addrtype;
-		raddr.sin_port = htons( rport );
-	}
-
-	return true;
-}
-
-bool _IKED::find_addr_l( IKE_SADDR & saddr_r, IKE_SADDR & saddr_l, unsigned short lport )
-{
-	//
-	// determine the best interface to
-	// reach the remote host address
-	//
-
-	IPROUTE_ENTRY entry;
-	memset( &entry, 0, sizeof( entry ) );
-	entry.addr = saddr_r.saddr4.sin_addr;
-
-	bool found = iproute.best( entry );
-
-	//
-	// log the result
-	//
-
-	if( found )
-	{
-		saddr_l.saddr4.sin_family = AF_INET;
-		saddr_l.saddr4.sin_port	= htons( lport );
-		saddr_l.saddr4.sin_addr = entry.iface;
-
-		char txtaddr[ LIBIKE_MAX_TEXTADDR ];
-		text_addr( txtaddr, &saddr_l, true );
-
-		log.txt( LLOG_DEBUG,
-				"ii : local address %s selected for peer\n",
-				txtaddr );
-	}
-	else
-	{
-		char txtaddr[ LIBIKE_MAX_TEXTADDR ];
-		text_addr( txtaddr, &saddr_r, true );
-
-		log.txt( LLOG_DEBUG,
-				"ii : unable to select local address for peer %s\n",
-				txtaddr );
-	}
-
-	return found;
-}
-
 bool has_sockaddr( sockaddr * saddr )
 {
 	switch( saddr->sa_family )
