@@ -137,6 +137,7 @@ long _IKED::payload_get_frag( PACKET_IKE & packet, IDB_PH1 * ph1, bool & complet
 		"ii : - frag size = %i\n",
 		index,
 		size );
+
 	//
 	// add to our ph1 fragment list
 	//
@@ -1314,7 +1315,7 @@ long _IKED::payload_get_cert( PACKET_IKE & packet, uint8_t & type, BDATA & cert
 	if( type == ISAKMP_CERT_X509_SIG )
 		packet.get( cert, size );			// certificate data
 	else
-		packet.get_null( size );
+		packet.get_null( size );			// certificate data
 
 	return LIBIKE_OK;
 }
@@ -1517,24 +1518,16 @@ long _IKED::payload_add_cfglist( PACKET_IKE & packet, IDB_CFG * cfg, uint8_t nex
 	{
 		IKE_ATTR * attr = cfg->attr_get( index );
 
-		switch( attr->basic )
+		if( attr->basic )
 		{
-			case true:
-			{
-				packet.add_word( BASIC | attr->atype );
-				packet.add_word( attr->bdata );
-
-				break;
-			}
-
-			case false:
-			{
-				packet.add_word( attr->atype );
-				packet.add_word( short( attr->vdata.size() ) );
-				packet.add( attr->vdata );
-
-				break;
-			}
+			packet.add_word( BASIC | attr->atype );
+			packet.add_word( attr->bdata );
+		}
+		else
+		{
+			packet.add_word( attr->atype );
+			packet.add_word( short( attr->vdata.size() ) );
+			packet.add( attr->vdata );
 		}
 	}
 
