@@ -203,7 +203,7 @@ bool _IKED::policy_list_create( IDB_TUNNEL * tunnel, bool initiator )
 		}
 	}
 
- 	//
+	//
 	// determine ipsec policy level value
 	//
 
@@ -266,6 +266,8 @@ bool _IKED::policy_list_create( IDB_TUNNEL * tunnel, bool initiator )
 
 	policy_create( tunnel, IPSEC_POLICY_NONE, IPSEC_LEVEL_DEFAULT, id1, id2, true );
 
+#ifdef WIN32
+
 	IPROUTE_ENTRY entry;
 	memset( &entry, 0, sizeof( entry ) );
 	entry.addr = tunnel->saddr_r.saddr4.sin_addr;
@@ -280,9 +282,11 @@ bool _IKED::policy_list_create( IDB_TUNNEL * tunnel, bool initiator )
 		id2.type = ISAKMP_ID_IPV4_ADDR;
 		id2.addr1 = entry.next;
 
-		policy_create( tunnel, IPSEC_POLICY_NONE, IPSEC_LEVEL_DEFAULT, id1, id2, true );
+		policy_create( tunnel, IPSEC_POLICY_NONE, IPSEC_LEVEL_DEFAULT, id1, id2, false );
 	}
 
+#endif
+
 	//
 	// build the client id
 	//
@@ -319,7 +323,7 @@ bool _IKED::policy_list_create( IDB_TUNNEL * tunnel, bool initiator )
 		if( initiator )
 			policy_create( tunnel, IPSEC_POLICY_IPSEC, ipsec_level, id1, id2, true );
 		else
-			policy_create( tunnel, IPSEC_POLICY_IPSEC, ipsec_level, id1, id2, true );
+			policy_create( tunnel, IPSEC_POLICY_IPSEC, ipsec_level, id2, id1, true );
 	}
 
 	return true;
@@ -327,7 +331,7 @@ bool _IKED::policy_list_create( IDB_TUNNEL * tunnel, bool initiator )
 
 bool _IKED::policy_list_remove( IDB_TUNNEL * tunnel, bool initiator )
 {
- 	//
+	//
 	// determine ipsec policy level value
 	//
 
@@ -396,15 +400,7 @@ bool _IKED::policy_list_remove( IDB_TUNNEL * tunnel, bool initiator )
 	// remove our gateway NONE policies
 	//
 
-	memset( &id1, 0, sizeof( id1 ) );
-	id1.type = ISAKMP_ID_IPV4_ADDR;
-	id1.addr1 = tunnel->saddr_l.saddr4.sin_addr;
-
-	memset( &id2, 0, sizeof( id2 ) );
-	id2.type = ISAKMP_ID_IPV4_ADDR;
-	id2.addr1 = tunnel->saddr_r.saddr4.sin_addr;
-
-	policy_remove( tunnel, IPSEC_POLICY_NONE, IPSEC_LEVEL_DEFAULT, id1, id2, true );
+#ifdef WIN32
 
 	IPROUTE_ENTRY entry;
 	memset( &entry, 0, sizeof( entry ) );
@@ -420,9 +416,21 @@ bool _IKED::policy_list_remove( IDB_TUNNEL * tunnel, bool initiator )
 		id2.type = ISAKMP_ID_IPV4_ADDR;
 		id2.addr1 = entry.next;
 
-		policy_remove( tunnel, IPSEC_POLICY_NONE, IPSEC_LEVEL_DEFAULT, id1, id2, true );
+		policy_remove( tunnel, IPSEC_POLICY_NONE, IPSEC_LEVEL_DEFAULT, id1, id2, false );
 	}
 
+#endif
+
+	memset( &id1, 0, sizeof( id1 ) );
+	id1.type = ISAKMP_ID_IPV4_ADDR;
+	id1.addr1 = tunnel->saddr_l.saddr4.sin_addr;
+
+	memset( &id2, 0, sizeof( id2 ) );
+	id2.type = ISAKMP_ID_IPV4_ADDR;
+	id2.addr1 = tunnel->saddr_r.saddr4.sin_addr;
+
+	policy_remove( tunnel, IPSEC_POLICY_NONE, IPSEC_LEVEL_DEFAULT, id1, id2, true );
+
 	if( tunnel->force_all )
 		tunnel->force_all = false;
 
@@ -431,7 +439,7 @@ bool _IKED::policy_list_remove( IDB_TUNNEL * tunnel, bool initiator )
 
 bool _IKED::policy_dhcp_create( IDB_TUNNEL * tunnel )
 {
- 	//
+	//
 	// determine ipsec policy level value
 	//
 
@@ -478,7 +486,7 @@ bool _IKED::policy_dhcp_create( IDB_TUNNEL * tunnel )
 
 bool _IKED::policy_dhcp_remove( IDB_TUNNEL * tunnel )
 {
- 	//
+	//
 	// determine ipsec policy level value
 	//
 
@@ -636,12 +644,17 @@ bool _IKED::policy_create( IDB_TUNNEL * tunnel, u_int16_t type, u_int8_t level,
 		return false;
 	}
 
-	policy->nailed = tunnel->peer->nailed;
+	//
+	// set special flags for outbound policies
+	//
+
+	if( tunnel->peer->nailed )
+		policy->flags |= PFLAG_NAILED;
 
 	if( ( type == IPSEC_POLICY_IPSEC ) && ( tunnel->tstate & TSTATE_POLICY_INIT ) )
 	{
+		policy->flags |= PFLAG_INITIAL;
 		tunnel->tstate &= ~TSTATE_POLICY_INIT;
-		policy->initial = true;
 	}
 
 	//
@@ -669,7 +682,8 @@ bool _IKED::policy_create( IDB_TUNNEL * tunnel, u_int16_t type, u_int8_t level,
 					route_entry.addr,
 					route_entry.mask );
 
-				policy->route_added = iproute.add( route_entry );
+				if( iproute.add( route_entry ) )
+					policy->flags |= PFLAG_ROUTED;
 
 				break;
 			}
@@ -686,14 +700,15 @@ bool _IKED::policy_create( IDB_TUNNEL * tunnel, u_int16_t type, u_int8_t level,
 					if( id2.type == ISAKMP_ID_IPV4_ADDR )
 						route_entry.mask.s_addr = 0xffffffff;
 
-					policy->route_added = iproute.add( route_entry );
+					if( iproute.add( route_entry ) )
+						policy->flags |= PFLAG_ROUTED;
 				}
 
 				break;
 			}
 		}
 
-		if( policy->route_added )
+		if( policy->flags & PFLAG_ROUTED )
 		{
 			log.txt( LLOG_INFO,
 				"ii : created %s policy route for %s\n",
@@ -728,8 +743,8 @@ bool _IKED::policy_remove( IDB_TUNNEL * tunnel, u_int16_t type, u_int8_t level,
 
 	IDB_POLICY * policy;
 
-	bool route_added = false;
 	bool route_deleted = false;
+	long flags = 0;
 	IPROUTE_ENTRY route_entry;
 
 	IKE_SADDR * src;
@@ -800,8 +815,8 @@ bool _IKED::policy_remove( IDB_TUNNEL * tunnel, u_int16_t type, u_int8_t level,
 			&id1,
 			&id2 ) )
 	{
-		route_added = policy->route_added;
 		route_entry = policy->route_entry;
+		flags = policy->flags;
 
 		text_ph2id( txtid_src, &id1 );
 		text_ph2id( txtid_dst, &id2 );
@@ -824,7 +839,7 @@ bool _IKED::policy_remove( IDB_TUNNEL * tunnel, u_int16_t type, u_int8_t level,
 	// remove client policy route
 	//
 
-	if( route && route_added && ( tunnel->peer->contact == IPSEC_CONTACT_CLIENT ) )
+	if( route && ( flags & PFLAG_ROUTED ) && ( tunnel->peer->contact == IPSEC_CONTACT_CLIENT ) )
 	{
 		switch( type )
 		{
