@@ -75,7 +75,6 @@ long _IKED::phase1_gen_prop( IDB_PH1 * ph1 )
 
 	unsigned short glist[] =
 	{
-		IKE_GRP_GROUP15,
 		IKE_GRP_GROUP14,
 		IKE_GRP_GROUP5,
 		IKE_GRP_GROUP2,
@@ -650,7 +649,10 @@ long _IKED::phase2_gen_prop( IDB_PH2 * ph2, IDB_POLICY * policy )
 				unsigned char tlist[] =
 				{
 					ISAKMP_AH_MD5,
-					ISAKMP_AH_SHA
+					ISAKMP_AH_SHA,
+					ISAKMP_AH_SHA256,
+					ISAKMP_AH_SHA384,
+					ISAKMP_AH_SHA512,
 				};
 
 				//
@@ -705,7 +707,19 @@ long _IKED::phase2_gen_prop( IDB_PH2 * ph2, IDB_POLICY * policy )
 							break;
 
 						case ISAKMP_AH_SHA:
-							proposal.hash_id = ISAKMP_AUTH_HMAC_SHA;
+							proposal.hash_id = ISAKMP_AUTH_HMAC_SHA1;
+							break;
+
+						case ISAKMP_AH_SHA256:
+							proposal.hash_id = ISAKMP_AUTH_HMAC_SHA2_256;
+							break;
+
+						case ISAKMP_AH_SHA384:
+							proposal.hash_id = ISAKMP_AUTH_HMAC_SHA2_384;
+							break;
+
+						case ISAKMP_AH_SHA512:
+							proposal.hash_id = ISAKMP_AUTH_HMAC_SHA2_512;
 							break;
 					}
 
@@ -744,7 +758,10 @@ long _IKED::phase2_gen_prop( IDB_PH2 * ph2, IDB_POLICY * policy )
 				unsigned short alist[] =
 				{
 					ISAKMP_AUTH_HMAC_MD5,
-					ISAKMP_AUTH_HMAC_SHA
+					ISAKMP_AUTH_HMAC_SHA1,
+					ISAKMP_AUTH_HMAC_SHA2_256,
+					ISAKMP_AUTH_HMAC_SHA2_384,
+					ISAKMP_AUTH_HMAC_SHA2_512
 				};
 
 				//
@@ -852,6 +869,7 @@ long _IKED::phase2_gen_prop( IDB_PH2 * ph2, IDB_POLICY * policy )
 								switch( ph2->tunnel->natt_version )
 								{
 									case IPSEC_NATT_NONE:
+									case IPSEC_NATT_CISCO:
 										proposal.encap = ISAKMP_ENCAP_TUNNEL;
 										break;
 
@@ -872,6 +890,7 @@ long _IKED::phase2_gen_prop( IDB_PH2 * ph2, IDB_POLICY * policy )
 								switch( ph2->tunnel->natt_version )
 								{
 									case IPSEC_NATT_NONE:
+									case IPSEC_NATT_CISCO:
 										proposal.encap = ISAKMP_ENCAP_TRANSPORT;
 										break;
 
