@@ -45,12 +45,10 @@
 // UNIX socket code
 //
 
-#ifndef SOL_UDP
-#define SOL_UDP 17
-#endif
-
 long _IKED::socket_init()
 {
+	socketpair( AF_UNIX, SOCK_STREAM, 0, wake_socket );
+	fcntl( wake_socket[ 0 ], F_SETFL, O_NONBLOCK );
 	return LIBIKE_OK;
 }
 
@@ -64,41 +62,6 @@ void _IKED::socket_done()
 	}
 }
 
-long _IKED::socket_select( unsigned long timeout )
-{
-	struct timeval tv;
-	tv.tv_sec = 0;
-	tv.tv_usec = timeout * 1000;
-
-	fd_set fdset;
-	FD_ZERO( &fdset ); 
-
-	lock_net.lock();
-
-	long count = list_socket.count();
-	long index = 0;
-	int  hival = 0;
-
-	for( ; index < count; index++ )
-	{
-		SOCK_INFO * sock_info = static_cast<SOCK_INFO*>( list_socket.get_entry( index ) );
-
-		FD_SET( sock_info->sock, &fdset );
-
-		if( hival < sock_info->sock )
-			hival = sock_info->sock;
-	}
-
-	long result = select( hival + 1, &fdset, NULL, NULL, &tv );
-
-	lock_net.unlock();
-
-	if( result < 0 )
-		return LIBIKE_SOCKET;
-
-	return result;
-}
-
 long _IKED::socket_create( IKE_SADDR & saddr, bool natt )
 {
 	SOCK_INFO * sock_info = new SOCK_INFO;
@@ -149,19 +112,13 @@ long _IKED::socket_create( IKE_SADDR & saddr, bool natt )
 
 #endif
 
-	if( fcntl( sock_info->sock, F_SETFL, O_NONBLOCK ) == -1 )
-	{
-		log.txt( LLOG_ERROR, "!! : socket set non-blocking mode failed\n" );
-		return LIBIKE_SOCKET;
-	}
-
 	if( bind( sock_info->sock, &sock_info->saddr.saddr, sizeof( sock_info->saddr.saddr4 ) ) < 0 )
 	{
 		log.txt( LLOG_ERROR, "!! : socket bind failed\n" );
 		return LIBIKE_SOCKET;
 	}
 
-#ifdef OPT_NATT
+#if defined( OPT_NATT ) && !defined( __APPLE__ )
 
 	if( natt )
 	{
@@ -207,6 +164,12 @@ long _IKED::socket_create( IKE_SADDR & saddr, bool natt )
 	return LIBIKE_OK;
 }
 
+void _IKED::socket_wakeup()
+{
+	char c = 0;
+	send( wake_socket[ 1 ], &c, 1, 0 );
+}
+
 long _IKED::socket_lookup_addr( IKE_SADDR & saddr_r, IKE_SADDR & saddr_l )
 {
 	//
@@ -215,7 +178,6 @@ long _IKED::socket_lookup_addr( IKE_SADDR & saddr_r, IKE_SADDR & saddr_l )
 	//
 
 	IPROUTE_ENTRY entry;
-	memset( &entry, 0, sizeof( entry ) );
 	entry.addr = saddr_r.saddr4.sin_addr;
 
 	if( !iproute.best( entry ) )
@@ -287,6 +249,37 @@ long _IKED::header( PACKET_IP & packet, ETH_HEADER & header )
 
 long _IKED::recv_ip( PACKET_IP & packet, ETH_HEADER * ethhdr )
 {
+	fd_set fdset;
+	FD_ZERO( &fdset );
+
+	lock_net.lock();
+
+	long count = list_socket.count();
+	long index = 0;
+	int  hival = wake_socket[ 0 ];
+
+	FD_SET( wake_socket[ 0 ], &fdset );
+
+	for( ; index < count; index++ )
+	{
+		SOCK_INFO * sock_info = static_cast<SOCK_INFO*>( list_socket.get_entry( index ) );
+
+		FD_SET( sock_info->sock, &fdset );
+
+		if( hival < sock_info->sock )
+			hival = sock_info->sock;
+	}
+
+	long result = select( hival + 1, &fdset, NULL, NULL, NULL );
+
+	lock_net.unlock();
+
+	if( result < 0 )
+		return LIBIKE_SOCKET;
+
+	if( FD_ISSET( wake_socket[ 0 ], &fdset ) )
+		return LIBIKE_SOCKET;
+
 	//
 	// recv packet data
 	//
@@ -296,13 +289,13 @@ long _IKED::recv_ip( PACKET_IP & packet, ETH_HEADER * ethhdr )
 
 	socklen_t	flen = sizeof( from.saddr4 );
 
-	long count = list_socket.count();
-	long index = 0;
-
-	for( ; index < count; index++ )
+	for( index = 0; index < count; index++ )
 	{
 		SOCK_INFO * sock_info = static_cast<SOCK_INFO*>( list_socket.get_entry( index ) );
 
+		if( FD_ISSET( sock_info->sock, &fdset ) == 0 )
+			continue;
+
 		unsigned char buff[ RAWNET_BUFF_SIZE ];
 		unsigned char ctrl[ 256 ];
 
@@ -496,6 +489,7 @@ bool _IKED::vnet_init()
 
 #endif
 
+	return false;
 }
 
 bool _IKED::vnet_get( VNET_ADAPTER ** adapter )
@@ -506,7 +500,7 @@ bool _IKED::vnet_get( VNET_ADAPTER ** adapter )
 	if( !*adapter )
 		return false;
 
-#ifdef __FreeBSD__
+#if defined( __FreeBSD__ ) || defined( __APPLE__ )
 
 	// find existing device
 
@@ -659,10 +653,21 @@ bool _IKED::vnet_rel( VNET_ADAPTER * adapter )
 	return true;
 }
 
-bool _IKED::client_setup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel )
+bool _IKED::client_net_config( IDB_TUNNEL * tunnel )
 {
 	if( tunnel->xconf.opts & IPSEC_OPTS_ADDR )
 	{
+		//
+		// acquire virtual adapter
+		//
+
+		if( !vnet_get( &tunnel->adapter ) )
+		{
+			log.txt( LLOG_ERROR, "ii : unable to create tap adapter ...\n" );
+
+			return false;
+		}
+
 		//
 		// open socket for configuration
 		//
@@ -682,12 +687,12 @@ bool _IKED::client_setup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel )
 
 		struct ifreq ifr;
 		memset( &ifr, 0, sizeof( struct ifreq ) );
-		strncpy( ifr.ifr_name, adapter->name, IFNAMSIZ );
+		strncpy( ifr.ifr_name, tunnel->adapter->name, IFNAMSIZ );
 
 		if( ioctl( sock, SIOCGIFFLAGS, &ifr ) < 0 )
 		{
 			log.txt( LLOG_ERROR, "!! : failed to get interface flags %s for ( %s )\n",
-				adapter->name, strerror( errno ) );
+				tunnel->adapter->name, strerror( errno ) );
 
 			close( sock );
 			return false;
@@ -698,7 +703,7 @@ bool _IKED::client_setup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel )
 		if( ioctl( sock, SIOCSIFFLAGS, &ifr ) < 0 )
 		{
 			log.txt( LLOG_ERROR, "!! : failed to set interface flags %s for ( %s )\n",
-				adapter->name, strerror( errno ) );
+				tunnel->adapter->name, strerror( errno ) );
 
 			close( sock );
 			return false;
@@ -717,7 +722,7 @@ bool _IKED::client_setup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel )
 		if( ioctl( sock, SIOCSIFADDR, &ifr ) != 0 )
 		{
 			log.txt( LLOG_ERROR, "!! : failed to configure address for %s ( %s )\n",
-				adapter->name, strerror( errno ) );
+				tunnel->adapter->name, strerror( errno ) );
 
 			close( sock );
 			return false;
@@ -732,7 +737,7 @@ bool _IKED::client_setup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel )
 		if( ioctl( sock, SIOCSIFNETMASK, &ifr ) != 0 )
 		{
 			log.txt( LLOG_ERROR, "!! : failed to configure netmask for %s ( %s )\n",
-				adapter->name, strerror( errno ) );
+				tunnel->adapter->name, strerror( errno ) );
 
 			close( sock );
 			return false;
@@ -749,7 +754,7 @@ bool _IKED::client_setup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel )
 		if( ioctl( sock, SIOCSIFBRDADDR, &ifr ) != 0 )
 		{
 			log.txt( LLOG_ERROR, "!! : failed to configure broadcast address for %s ( %s )\n",
-				adapter->name, strerror( errno ) );
+				tunnel->adapter->name, strerror( errno ) );
 
 			close( sock );
 			return false;
@@ -763,7 +768,7 @@ bool _IKED::client_setup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel )
 
 		struct ifaliasreq ifra;
 		memset( &ifra, 0, sizeof( struct ifaliasreq ) );
-		strncpy( ifra.ifra_name, adapter->name, IFNAMSIZ );
+		strncpy( ifra.ifra_name, tunnel->adapter->name, IFNAMSIZ );
 
 		struct sockaddr_in * addr = ( struct sockaddr_in * ) &( ifra.ifra_addr );
 		addr->sin_family = AF_INET;
@@ -785,7 +790,7 @@ bool _IKED::client_setup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel )
 		if( ioctl( sock, SIOCAIFADDR, &ifra ) < 0 )
 		{
 			log.txt( LLOG_ERROR, "!! : failed to configure address for %s ( %s )\n",
-				adapter->name, strerror( errno ) );
+				tunnel->adapter->name, strerror( errno ) );
 
 			close( sock );
 			return false;
@@ -802,7 +807,7 @@ bool _IKED::client_setup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel )
 		if( ioctl( sock, SIOCSIFMTU, &ifr ) != 0 )
 		{
 			log.txt( LLOG_ERROR, "!! : failed to configure MTU for %s ( %s )\n",
-				adapter->name, strerror( errno ) );
+				tunnel->adapter->name, strerror( errno ) );
 
 			close( sock );
 			return false;
@@ -815,7 +820,7 @@ bool _IKED::client_setup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel )
 		if( ioctl( sock, SIOCGIFFLAGS, &ifr ) < 0 )
 		{
 			log.txt( LLOG_ERROR, "!! : failed to get interface flags for %s ( %s )\n",
-				adapter->name, strerror( errno ) );
+				tunnel->adapter->name, strerror( errno ) );
 
 			close( sock );
 			return false;
@@ -826,7 +831,7 @@ bool _IKED::client_setup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel )
 	        if( ioctl( sock, SIOCSIFFLAGS, &ifr ) < 0 )
 		{
 			log.txt( LLOG_ERROR, "!! : failed to set interface flags for %s ( %s )\n",
-				adapter->name, strerror( errno ) );
+				tunnel->adapter->name, strerror( errno ) );
 
 			close( sock );
 			return false;
@@ -834,22 +839,42 @@ bool _IKED::client_setup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel )
 
 		close( sock );
 
-		log.txt( LLOG_INFO, "ii : configured adapter %s\n", adapter->name );
+		log.txt( LLOG_INFO, "ii : configured adapter %s\n",
+			tunnel->adapter->name );
 	}
 
+	return true;
+}
+
+bool _IKED::client_net_revert( IDB_TUNNEL * tunnel )
+{
+	if( tunnel->xconf.opts & IPSEC_OPTS_ADDR )
+	{
+		if( tunnel->adapter != NULL )
+		{
+			vnet_rel( tunnel->adapter );
+			tunnel->adapter = NULL;
+		}
+	}
+
+	return true;
+}
+
+bool _IKED::client_dns_config( IDB_TUNNEL * tunnel )
+{
 	if( tunnel->xconf.opts & ( IPSEC_OPTS_DNSS | IPSEC_OPTS_DOMAIN ) )
 	{
 		// backup the current resolv.conf file
 
 		rename( "/etc/resolv.conf", "/etc/resolv.iked" );
 
-		// write the new resolv.conf file
-
 		FILE * fp1 = fopen( "/etc/resolv.iked", "r" );
 		FILE * fp2 = fopen( "/etc/resolv.conf", "w+" );
 
 		if( fp2 != NULL )
 		{
+			// write configuration
+
 			if( tunnel->xconf.opts & IPSEC_OPTS_DOMAIN )
 				fprintf( fp2, "domain\t%s\n", tunnel->xconf.nscfg.dnss_suffix );
 
@@ -895,7 +920,7 @@ bool _IKED::client_setup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel )
 	return true;
 }
 
-bool _IKED::client_cleanup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel )
+bool _IKED::client_dns_revert( IDB_TUNNEL * tunnel )
 {
 	if( tunnel->xconf.opts & ( IPSEC_OPTS_DNSS | IPSEC_OPTS_DOMAIN ) )
 	{
@@ -906,5 +931,3 @@ bool _IKED::client_cleanup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel )
 
 	return true;
 }
-
-
