@@ -41,39 +41,57 @@
 
 #include "iked.h"
 
+long _IKED_EXEC::func( void * arg )
+{
+	long result = iked_func( arg );
+
+	// openssl thread cleanup
+	ERR_remove_state( 0 );
+
+	return result;
+}
+
 bool _IKED::rand_bytes( void * buff, long size )
 {
 	RAND_pseudo_bytes( ( unsigned char * ) buff, size );
 	return true;
 }
 
+void _IKED::set_files( char * set_path_conf, const char * set_path_log )
+{
+	strcpy_s( path_conf, MAX_PATH, set_path_conf );
+	strcpy_s( path_log, MAX_PATH, set_path_log );
+}
+
 void _IKED::loop_ref_inc( const char * name )
 {
 	log.txt( LLOG_INFO, "ii : %s process thread begin ...\n", name );
 
 	lock_run.lock();
-
-	if( loopcount++ == 0 )
-		cond_run.reset();
-
+	long tempcount = loopcount++;
 	lock_run.unlock();
+
+	if( tempcount == 0 )
+		cond_run.reset();
 }
 
 void _IKED::loop_ref_dec( const char * name )
 {
+	log.txt( LLOG_INFO, "ii : %s process thread exit ...\n", name );
+
 	lock_run.lock();
-
-	if( --loopcount == 0 )
-		cond_run.alert();
-
+	long tempcount = --loopcount;
 	lock_run.unlock();
 
-	log.txt( LLOG_INFO, "ii : %s process thread exit ...\n", name );
+	if( tempcount == 0 )
+		cond_run.alert();
 }
 
 _IKED::_IKED()
 {
-	state = DSTATE_ACTIVE;
+	path_conf[ 0 ] = 0;
+	path_log[ 0 ] = 0;
+
 	peercount = 0;
 	loopcount = 0;
 	tunnelid = 2;
@@ -105,6 +123,9 @@ _IKED::_IKED()
 	unsigned char dpd1[] = VEND_DPD1;
 	vend_dpd1.set( dpd1, sizeof( dpd1 ) );
 
+	unsigned char dpd1_ng[] = VEND_DPD1_NG;
+	vend_dpd1_ng.set( dpd1_ng, sizeof( dpd1_ng ) );
+
 	unsigned char hbeat[] = VEND_HBEAT;
 	vend_hbeat.set( hbeat, sizeof( hbeat ) );
 
@@ -174,10 +195,10 @@ long _IKED::init( long setlevel )
 	}
 
 	//
-	// ititialize openssl
+	// ititialize openssl libcrypto
 	//
 
-	OpenSSL_add_all_algorithms();
+	crypto_init();
 
 	//
 	// open our log ( debug and echo )
@@ -322,6 +343,21 @@ long _IKED::init( long setlevel )
 
 	socket_init();
 
+	//
+	// setup natt port on OSX systems
+	//
+
+#ifdef __APPLE__
+
+	int natt_port = LIBIKE_NATT_PORT;
+
+	sysctlbyname(
+		"net.inet.ipsec.esp_port",
+		NULL, NULL,
+		&natt_port, sizeof( natt_port ) );
+
+#endif
+
 	//
 	// default socket initialization
 	//
@@ -400,45 +436,68 @@ void _IKED::loop()
 	ith_timer.run();
 
 	//
-	// cleanup
+	// wait for all threads to exit
 	//
 
-	socket_done();
-
-	ikes.done();
-
-	log.close();
-}
-
-long _IKED::halt()
-{
-	log.txt( LLOG_INFO,
-		"ii : halt signal received, shutting down\n" );
-
-	//
-	// remove all top level db objects
-	//
-
-	idb_list_peer.clean();
-
-	cond_idb.wait( -1 );
-
-	//
-	// terminate all thread loops
-	//
-
-	state = DSTATE_TERMINATE;
-
-	ikes.wakeup();
-	pfki.wakeup();
-
 	cond_run.wait( -1 );
 
 	//
-	// exit event timer loop
+	// cleanup
 	//
 
-	ith_timer.end();
+	socket_done();
+	ikes.done();
+	log.close();
+
+	//
+	// cleanup openssl libcrypto
+	//
+
+	crypto_done();
+}
+
+long _IKED::halt( bool terminate )
+{
+	if( terminate )
+	{
+		log.txt( LLOG_INFO,
+			"ii : hard halt signal received, shutting down\n" );
+
+		//
+		// exit event timer loop
+		//
+
+		ith_timer.end();
+
+		//
+		// remove all top level db objects
+		//
+
+		idb_list_peer.clean();
+
+		cond_idb.wait( -1 );
+
+		//
+		// terminate all thread loops
+		//
+
+		ikes.wakeup();
+		pfki.wakeup();
+		socket_wakeup();
+	}
+	else
+	{
+		log.txt( LLOG_INFO,
+			"ii : soft halt signal received, closing tunnels\n" );
+
+		//
+		// remove all top level db objects
+		//
+
+		idb_list_peer.clean();
+
+		cond_idb.wait( -1 );
+	}
 
 	return LIBIKE_OK;
 }
