@@ -73,7 +73,12 @@
 #  include <sys/param.h>
 #  include <sys/socket.h>
 #  include <net/if.h>
-#  include <net/if_tap.h>
+#  ifndef __APPLE__
+#   include <net/if_tap.h>
+#  else
+#   include <sys/sysctl.h>
+#   include "compat/tun_ioctls.h"
+#  endif
 # endif
 # ifdef __FreeBSD__
 #  include <sys/linker.h>
@@ -88,12 +93,12 @@
 #endif
 
 #ifdef OPT_LDAP
-#include <ldap.h>
+# include <ldap.h>
 #endif
 
 #include "version.h"
+#include "libip.h"
 #include "liblog.h"
-#include "libike.h"
 #include "libith.h"
 #include "libpfk.h"
 #include "libike.h"
@@ -104,10 +109,6 @@
 #include "xauth.h"
 #include "xconf.h"
 
-#ifdef OPT_DTP
-#include "libdtp.h"
-#endif
-
 //
 // Win32 specific
 //
@@ -143,7 +144,6 @@ namespace yy{ class conf_parser; };
 
 #endif
 
-
 //
 // IKED constants
 //
@@ -168,6 +168,7 @@ namespace yy{ class conf_parser; };
 #define	VEND_XAUTH		{ 0x09, 0x00, 0x26, 0x89, 0xDF, 0xD6, 0xB7, 0x12 }
 #define VEND_FRAG		{ 0x40, 0x48, 0xb7, 0xd5, 0x6e, 0xbc, 0xe8, 0x85, 0x25, 0xe7, 0xde, 0x7f, 0x00, 0xd6, 0xc2, 0xd3, 0x80, 0x00, 0x00, 0x00 }
 #define VEND_DPD1		{ 0xaf, 0xca, 0xd7, 0x13, 0x68, 0xa1, 0xf1, 0xc9, 0x6b, 0x86, 0x96, 0xfc, 0x77, 0x57, 0x01, 0x00 }
+#define	VEND_DPD1_NG	{ 0x3b, 0x90, 0x31, 0xdc, 0xe4, 0xfc, 0xf8, 0x8b, 0x48, 0x9a, 0x92, 0x39, 0x63, 0xdd, 0x0c, 0x49 }
 #define VEND_HBEAT		{ 0x48, 0x65, 0x61, 0x72, 0x74, 0x42, 0x65, 0x61, 0x74, 0x5f, 0x4e, 0x6f, 0x74, 0x69, 0x66, 0x79, 0x38, 0x6b, 0x01, 0x00 }
 #define	VEND_NATT_V00	{ 0x44, 0x85, 0x15, 0x2d, 0x18, 0xb6, 0xbb, 0xcd, 0x0b, 0xe8, 0xa8, 0x46, 0x95, 0x79, 0xdd, 0xcc }
 #define	VEND_NATT_V01	{ 0x16, 0xf6, 0xca, 0x16, 0xe4, 0xa4, 0x06, 0x6d, 0x83, 0x82, 0x1a, 0x0f, 0x0a, 0xea, 0xa8, 0x62 }
@@ -196,7 +197,7 @@ namespace yy{ class conf_parser; };
 #define LIBIKE_MAX_TEXTP2ID		64		// max text phase2 id length
 #define LIBIKE_MAX_TEXTSPI		64		// max text phase2 id length
 #define LIBIKE_MAX_VARID		512		// max variable id length
-#define LIBIKE_MAX_DHGRP		512		// max dh group size
+#define LIBIKE_MAX_DHGRP		1024	// max dh group size
 
 #define LIBIKE_OK				0
 #define LIBIKE_FAILED			-1
@@ -278,9 +279,9 @@ namespace yy{ class conf_parser; };
 #define TSTATE_VNET_ENABLE		0x00000008
 #define TSTATE_POLICY_INIT		0x00000010
 
-#define DSTATE_ACTIVE			0
-#define DSTATE_TERMINATE		1
-#define DSTATE_INACTIVE			2
+#define PFLAG_ROUTED			0x00000001
+#define PFLAG_NAILED			0x00000002		// negotiate persistent SAs
+#define PFLAG_INITIAL			0x00000004		// negotiate an initial SA
 
 #define RLEVEL_DAEMON			2
 
@@ -292,46 +293,36 @@ namespace yy{ class conf_parser; };
 // IKED main classes and structures
 //
 
-#ifdef UNIX
-
-typedef struct _SOCK_INFO : public IDB_ENTRY
+typedef class _IKED_EXEC : public _ITH_EXEC
 {
-	int			sock;
-	IKE_SADDR	saddr;
-	bool		natt;
+	public:
 
-}SOCK_INFO;
-
-typedef struct _VNET_ADAPTER
-{
-	int	fn;
-	char	name[ IFNAMSIZ ];
-
-}VNET_ADAPTER;
-
-#endif
-
-typedef class _ITH_IKES : public _ITH_EXEC
-{
 	virtual long func( void * arg );
+	virtual long iked_func( void * arg ) = 0;
+
+}IKED_EXEC;
+
+typedef class _ITH_IKES : public _IKED_EXEC
+{
+	virtual long iked_func( void * arg );
 
 }ITH_IKES;
 
-typedef class _ITH_IKEC : public _ITH_EXEC
+typedef class _ITH_IKEC : public _IKED_EXEC
 {
-	virtual long func( void * arg );
+	virtual long iked_func( void * arg );
 
 }ITH_IKEC;
 
-typedef class _ITH_NWORK : public _ITH_EXEC
+typedef class _ITH_NWORK : public _IKED_EXEC
 {
-	virtual long func( void * arg );
+	virtual long iked_func( void * arg );
 
 }ITH_NWORK;
 
-typedef class _ITH_PFKEY : public _ITH_EXEC
+typedef class _ITH_PFKEY : public _IKED_EXEC
 {
-	virtual long func( void * arg );
+	virtual long iked_func( void * arg );
 
 }ITH_PFKEY;
 
@@ -342,7 +333,6 @@ typedef class _IKED
 	friend class _ITH_NWORK;
 	friend class _ITH_PFKEY;
 
-	friend class _IDB_RC_ENTRY;
 	friend class _IDB_PEER;
 	friend class _IDB_TUNNEL;
 	friend class _IDB_POLICY;
@@ -352,6 +342,7 @@ typedef class _IKED
 	friend class _IDB_CFG;
 	friend class _IDB_INF;
 
+	friend class _IKED_RC_LIST;
 	friend class _IDB_LIST_IKED;
 	friend class _IDB_LIST_PEER;
 	friend class _IDB_LIST_TUNNEL;
@@ -389,6 +380,7 @@ typedef class _IKED
 	private:
 
 	char	path_ins[ MAX_PATH ];		// install path
+	char	path_conf[ MAX_PATH ];		// configuration file
 	char	path_log[ MAX_PATH ];		// logfile path
 	char	path_decrypt[ MAX_PATH ];	// decrypted pcap path
 	char	path_encrypt[ MAX_PATH ];	// encrypted pcap path
@@ -399,7 +391,6 @@ typedef class _IKED
 
 	long	peercount;			// peer reference count
 	long	loopcount;			// loop reference count
-	long	state;				// daemon run state
 
 	long	tunnelid;			// next tunnel id
 	short	policyid;			// next request id
@@ -429,7 +420,12 @@ typedef class _IKED
 	ITH_LOCK	lock_net;
 	ITH_LOCK	lock_idb;
 
+#ifdef UNIX
+
 	IDB_LIST	list_socket;		// socket list
+	int			wake_socket[2];		// wakeup socket
+
+#endif
 
 	IDB_LIST			idb_list_netgrp;
 	IDB_LIST_PEER		idb_list_peer;
@@ -449,6 +445,7 @@ typedef class _IKED
 	BDATA	vend_xauth;
 	BDATA	vend_frag;
 	BDATA	vend_dpd1;
+	BDATA	vend_dpd1_ng;
 	BDATA	vend_hbeat;
 	BDATA	vend_natt_v00;
 	BDATA	vend_natt_v01;
@@ -498,7 +495,7 @@ typedef class _IKED
 	long	socket_init();
 	void	socket_done();
 	long	socket_create( IKE_SADDR & saddr, bool natt );
-	long	socket_select( unsigned long timeout );
+	void	socket_wakeup();
 	long	socket_lookup_addr( IKE_SADDR & saddr_l, IKE_SADDR & saddr_r );
 	long	socket_lookup_port( IKE_SADDR & saddr_l, bool natt );
 
@@ -517,13 +514,17 @@ typedef class _IKED
 	bool	vnet_get( VNET_ADAPTER ** adapter );
 	bool	vnet_rel( VNET_ADAPTER * adapter );
 
-	bool	client_setup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel );
-	bool	client_cleanup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel );
+	bool	client_net_config( IDB_TUNNEL * tunnel );
+	bool	client_net_revert( IDB_TUNNEL * tunnel );
+
+	bool	client_dns_config( IDB_TUNNEL * tunnel );
+	bool	client_dns_revert( IDB_TUNNEL * tunnel );
 
 #ifdef OPT_DTP
 
-	bool	dnsproxy_setup( DTPI & dtpi, IDB_TUNNEL * tunnel );
-	void	dnsproxy_cleanup( DTPI & dtpi );
+	bool	dnsproxy_check( IKEI * ikei );
+	bool	dnsproxy_setup( IDB_TUNNEL * tunnel );
+	void	dnsproxy_cleanup( IDB_TUNNEL * tunnel );
 
 #endif
 
@@ -544,25 +545,19 @@ typedef class _IKED
 
 	// x.509 certificate helper functions
 
-	bool	cert_2_bdata( BDATA & cert, X509 * x509 );
-	bool	bdata_2_cert( X509 ** x509, BDATA & cert );
-
-	long	cert_save( BDATA & cert, char * fpath );
 	long	cert_load( BDATA & cert, char * fpath, bool ca, BDATA & pass );
-	bool	cert_load_pem( BDATA & cert, FILE * fp, bool ca, BDATA & pass );
-	bool	cert_load_p12( BDATA & cert, FILE * fp, bool ca, BDATA & pass );
+	long	cert_load( BDATA & cert, BDATA & input, bool ca, BDATA & pass );
 	bool	cert_desc( BDATA & cert, BDATA & text );
 	bool	cert_subj( BDATA & cert, BDATA & subj );
 	bool	asn1_text( BDATA & asn1, BDATA & text );
 	bool	text_asn1( BDATA & text, BDATA & asn1 );
 	bool	cert_verify( IDB_LIST_CERT & certs, BDATA & ca, BDATA & cert );
 
-	long	prvkey_rsa_load( EVP_PKEY ** evp_pkey, char * fpath, BDATA & pass );
-	bool	prvkey_rsa_load_pem( EVP_PKEY ** evp_pkey, FILE * fp, BDATA & pass );
-	bool	prvkey_rsa_load_p12( EVP_PKEY ** evp_pkey, FILE * fp, BDATA & pass );
-	bool	pubkey_rsa_read( BDATA & cert, EVP_PKEY ** evp_pkey );
-	bool	prvkey_rsa_encrypt( EVP_PKEY * evp_pkey, BDATA & hash, BDATA & sign );
-	bool	pubkey_rsa_decrypt( EVP_PKEY * evp_pkey, BDATA & sign, BDATA & hash );
+	long	prvkey_rsa_load( BDATA & prvkey, char * fpath, BDATA & pass );
+	long	prvkey_rsa_load( BDATA & prvkey, BDATA & input, BDATA & pass );
+	bool	pubkey_rsa_read( BDATA & cert, BDATA & pubkey );
+	bool	prvkey_rsa_encrypt( BDATA & prvkey, BDATA & hash, BDATA & sign );
+	bool	pubkey_rsa_decrypt( BDATA & pubkey, BDATA & sign, BDATA & hash );
 
 	// id helper functions
 
@@ -672,8 +667,8 @@ typedef class _IKED
 	bool	config_server_xconf_push_recv( IDB_CFG * cfg, IDB_PH1 * ph1 );
 	bool	config_server_xconf_push_send( IDB_CFG * cfg, IDB_PH1 * ph1 );
 
-	long	config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask, VENDOPTS vendopts );
-	long	config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask, VENDOPTS vendopts );
+	long	config_xconf_set( IDB_CFG * cfg, long setbits, long setmask, VENDOPTS vendopts );
+	long	config_xconf_get( IDB_CFG * cfg, long & getbits, long getmask, VENDOPTS vendopts );
 
 	long	config_chk_hash( IDB_PH1 * ph1, IDB_CFG * cfg, unsigned long msgid );
 	long	config_message_send( IDB_PH1 * ph1, IDB_CFG * cfg );
@@ -794,8 +789,10 @@ typedef class _IKED
 
 	LOG	log;	// generic log object
 
+	void	set_files( char * set_path_conf, const char * set_path_log );
+
 	long	init( long setlevel );
-	long	halt();
+	long	halt( bool terminate );
 	void	loop();
 
 }IKED;
