@@ -276,13 +276,13 @@ bool _ITH_COND::wait( long msecs )
 
 void _ITH_COND::alert()
 {
-	char c;
+	char c = 0;
 	long result = send( conn_wake[ 1 ], &c, 1, 0 );
 }
 
 void _ITH_COND::reset()
 {
-	char c;
+	char c = 0;
 	long result = recv( conn_wake[ 0 ], &c, 1, 0 );
 }
 
@@ -374,6 +374,9 @@ bool _ITH_EXEC::exec( void * arg )
 		&help,
 		param );
 
+	pthread_detach(
+		thread );
+
 	return true;
 }
 
@@ -393,6 +396,12 @@ _ITH_TIMER::_ITH_TIMER()
 
 _ITH_TIMER::~_ITH_TIMER()
 {
+	while( head != NULL )
+	{
+		ITH_ENTRY * next = head->next;
+		delete head;
+		head = next;
+	}
 }
 
 #ifdef WIN32
@@ -544,9 +553,8 @@ void _ITH_TIMER::run()
 
 			if( entry->event->func() )
 				add( entry->event );
-			else
-				delete entry;
 
+			delete entry;
 			lock.lock();
 		}
 	}
@@ -646,33 +654,6 @@ bool _ITH_TIMER::del( ITH_EVENT * event )
 // shared platform functions
 //
 
-long _ITH_IPCC::io_recv( void *data, size_t size )
-{
-	char * buff = ( char * ) data;
-	size_t rcvd = 0;
-	size_t temp = 0;
-
-	while( size > rcvd )
-	{
-		temp = size - rcvd;
-		long result = io_recv( buff + rcvd, temp, temp );
-
-		switch( result )
-		{
-			case IPCERR_OK:
-			case IPCERR_BUFFER:
-				break;
-
-			default:
-				return result;
-		}
-
-		rcvd += temp;
-	}
-
-	return IPCERR_OK;
-}
-
 long _ITH_IPCC::io_send( void *data, size_t size )
 {
 	char * buff = ( char * ) data;
@@ -701,6 +682,33 @@ long _ITH_IPCC::io_send( void *data, size_t size )
 	return IPCERR_OK;
 }
 
+long _ITH_IPCC::io_recv( void *data, size_t size )
+{
+	char * buff = ( char * ) data;
+	size_t rcvd = 0;
+	size_t temp = 0;
+
+	while( size > rcvd )
+	{
+		temp = size - rcvd;
+		long result = io_recv( buff + rcvd, temp, temp );
+
+		switch( result )
+		{
+			case IPCERR_OK:
+			case IPCERR_BUFFER:
+				break;
+
+			default:
+				return result;
+		}
+
+		rcvd += temp;
+	}
+
+	return IPCERR_OK;
+}
+
 #ifdef WIN32
 
 //
@@ -713,7 +721,6 @@ _ITH_IPCC::_ITH_IPCC()
 	hmutex_send = CreateMutex( NULL, false, NULL );
 
 	hevent_wake = CreateEvent( NULL, true, false, NULL );
-	hevent_send = CreateEvent( NULL, true, false, NULL );
 
 	conn = INVALID_HANDLE_VALUE;
 }
@@ -722,12 +729,6 @@ _ITH_IPCC::~_ITH_IPCC()
 {
 	detach();
 
-	if( hevent_send != NULL )
-	{
-		CloseHandle( hevent_send );
-		hevent_send = NULL;
-	}
-
 	if( hevent_wake != NULL )
 	{
 		CloseHandle( hevent_wake );
@@ -753,24 +754,137 @@ void _ITH_IPCC::io_conf( IPCCONN sconn )
 	conn = sconn;
 }
 
+VOID WINAPI io_send_complete( DWORD result, DWORD size, LPOVERLAPPED olapp )
+{
+	// nothing to do
+}
+
+long _ITH_IPCC::io_send( void * data, size_t size, size_t & sent )
+{
+//	if( conn == INVALID_HANDLE_VALUE )
+//		return IPCERR_CLOSED;
+
+	DWORD dwsize = ( DWORD ) sent;
+
+	WaitForSingleObject( hmutex_send, INFINITE );
+
+	// windows does not always set
+	// the GetLastError value to
+	// success after ReadFileEx but
+	// the documentation says you
+	// should check it for errors
+
+	SetLastError( ERROR_SUCCESS );
+
+	memset( &olapp_send, 0, sizeof( olapp_send ) );
+
+	long result = WriteFileEx(
+					conn,
+					data,
+					dwsize,
+					&olapp_send,
+					io_send_complete );
+
+	if( !result )
+	{
+		result = WaitForSingleObject( hevent_wake, 0 );
+//		if( result == WAIT_OBJECT_0 )
+//			ResetEvent( hevent_wake );
+
+		ReleaseMutex( hmutex_send );
+
+		if( result == WAIT_OBJECT_0 )
+			return IPCERR_WAKEUP;
+
+		return IPCERR_CLOSED;
+	}
+
+	result = GetLastError();
+
+	switch( result )
+	{
+		case ERROR_SUCCESS:
+
+			//
+			// wait in an alertable state until
+			// the operation completes or until
+			// the wake event is signaled
+			//
+
+			result = WaitForSingleObjectEx(
+				hevent_wake,
+				INFINITE,
+				true );
+
+			if( result == WAIT_OBJECT_0 )
+			{
+//				ResetEvent( hevent_wake );
+
+				// cancel the current overlaped
+				// request and give it a chance
+				// to complete in a wait state
+
+				CancelIo( conn );
+
+				ReleaseMutex( hmutex_recv );
+
+				return IPCERR_WAKEUP;
+			}
+
+			GetOverlappedResult(
+				conn,
+				&olapp_send,
+				&dwsize,
+				true );
+
+			result = GetLastError();
+			break;
+	}
+
+	switch( result )
+	{
+		case ERROR_SUCCESS:
+			result = IPCERR_OK;
+			break;
+
+		case ERROR_MORE_DATA:
+			result = IPCERR_BUFFER;
+			break;
+
+		case ERROR_OPERATION_ABORTED:
+//			ResetEvent( hevent_wake );
+//			result = IPCERR_WAKEUP;
+//			break;
+
+		case ERROR_BROKEN_PIPE:
+		case ERROR_INVALID_HANDLE:
+			result = IPCERR_CLOSED;
+			break;
+
+		default:
+			result = IPCERR_NODATA;
+			break;
+	}
+
+	sent = dwsize;
+
+	ReleaseMutex( hmutex_send );
+
+	return result;
+}
+
 VOID WINAPI io_recv_complete( DWORD result, DWORD size, LPOVERLAPPED olapp )
 {
-	// we do nothing here as the
-	// WaitForSingleObjectEx call
-	// will wake on io completion
+	// nothing to do
 }
 
 long _ITH_IPCC::io_recv( void * data, size_t size, size_t & rcvd )
 {
-	if( conn == INVALID_HANDLE_VALUE )
-		return IPCERR_CLOSED;
+//	if( conn == INVALID_HANDLE_VALUE )
+//		return IPCERR_CLOSED;
 
 	DWORD dwsize = ( DWORD ) size;
 
-	OVERLAPPED olapp;
-	memset( &olapp, 0, sizeof( olapp ) );
-	olapp.hEvent = hevent_send;
-
 	WaitForSingleObject( hmutex_recv, INFINITE );
 
 	// windows does not always set
@@ -781,16 +895,26 @@ long _ITH_IPCC::io_recv( void * data, size_t size, size_t & rcvd )
 
 	SetLastError( ERROR_SUCCESS );
 
+	memset( &olapp_recv, 0, sizeof( olapp_recv ) );
+
 	long result = ReadFileEx(
 					conn,
 					data,
 					dwsize,
-					&olapp,
+					&olapp_recv,
 					io_recv_complete );
 
 	if( !result )
 	{
+		result = WaitForSingleObject( hevent_wake, 0 );
+		if( result == WAIT_OBJECT_0 )
+			ResetEvent( hevent_wake );
+
 		ReleaseMutex( hmutex_recv );
+
+		if( result == WAIT_OBJECT_0 )
+			return IPCERR_WAKEUP;
+
 		return IPCERR_CLOSED;
 	}
 
@@ -800,36 +924,45 @@ long _ITH_IPCC::io_recv( void * data, size_t size, size_t & rcvd )
 	{
 		case ERROR_SUCCESS:
 
+			//
+			// wait in an alertable state until
+			// the operation completes or until
+			// the wake event is signaled
+			//
+
 			result = WaitForSingleObjectEx(
-						hevent_wake,
-						INFINITE,
-						true );
+				hevent_wake,
+				INFINITE,
+				true );
 
 			if( result == WAIT_OBJECT_0 )
 			{
+				ResetEvent( hevent_wake );
+
 				// cancel the current overlaped
 				// request and give it a chance
 				// to complete in a wait state
 
 				CancelIo( conn );
-				SleepEx( 0, true );
+
+				ReleaseMutex( hmutex_recv );
+
+				return IPCERR_WAKEUP;
 			}
 
 			GetOverlappedResult(
 				conn,
-				&olapp,
+				&olapp_recv,
 				&dwsize,
-				false );
+				true );
 
 			result = GetLastError();
-
 			break;
 	}
 
 	switch( result )
 	{
 		case ERROR_SUCCESS:
-//			assert( size == dwsize );
 			result = IPCERR_OK;
 			break;
 
@@ -838,11 +971,10 @@ long _ITH_IPCC::io_recv( void * data, size_t size, size_t & rcvd )
 			break;
 
 		case ERROR_OPERATION_ABORTED:
-			ResetEvent( hevent_wake );
-			result = IPCERR_WAKEUP;
-			break;
+//			ResetEvent( hevent_wake );
+//			result = IPCERR_WAKEUP;
+//			break;
 
-		case ERROR_GEN_FAILURE:
 		case ERROR_BROKEN_PIPE:
 		case ERROR_INVALID_HANDLE:
 			result = IPCERR_CLOSED;
@@ -860,86 +992,6 @@ long _ITH_IPCC::io_recv( void * data, size_t size, size_t & rcvd )
 	return result;
 }
 
-long _ITH_IPCC::io_send( void * data, size_t size, size_t & sent )
-{
-	if( conn == INVALID_HANDLE_VALUE )
-		return IPCERR_CLOSED;
-
-	DWORD dwsize = ( DWORD ) size;
-
-	OVERLAPPED olapp;
-	memset( &olapp, 0, sizeof( olapp ) );
-	olapp.hEvent = hevent_send;
-
-	WaitForSingleObject( hmutex_send, INFINITE );
-
-	// windows does not always set
-	// the GetLastError value to
-	// success after WriteFileEx but
-	// the documentation says you
-	// should check it for errors
-
-	SetLastError( ERROR_SUCCESS );
-
-	WriteFile(
-		conn,
-		data,
-		dwsize,
-		&dwsize,
-		&olapp );
-
-	long result = GetLastError();
-
-	switch( result )
-	{
-		case ERROR_IO_PENDING:
-
-			WaitForSingleObjectEx(
-				hevent_send,
-				INFINITE,
-				true );
-
-			GetOverlappedResult(
-				conn,
-				&olapp,
-				&dwsize,
-				true );
-
-			SetLastError( ERROR_SUCCESS );
-
-			result = GetLastError();
-
-			break;
-	}
-
-	switch( result )
-	{
-		case ERROR_SUCCESS:
-			assert( size == dwsize );
-			result = IPCERR_OK;
-			break;
-
-		case ERROR_MORE_DATA:
-			result = IPCERR_BUFFER;
-			break;
-
-		case ERROR_GEN_FAILURE:
-		case ERROR_BROKEN_PIPE:
-		case ERROR_INVALID_HANDLE:
-			result = IPCERR_CLOSED;
-			break;
-
-		default:
-			result = IPCERR_FAILED;
-			break;
-	}
-
-	sent = dwsize;
-
-	ReleaseMutex( hmutex_send );
-	return result;
-}
-
 long _ITH_IPCC::attach( const char * path, long timeout )
 {
 	if( !WaitNamedPipe( path, timeout ) )
@@ -1127,18 +1179,28 @@ long _ITH_IPCS::init( const char * path, bool admin )
 void _ITH_IPCS::done()
 {
 	if( acl != NULL )
+	{
 		LocalFree( acl );
+		acl = NULL;
+	}
 
 	if( sid_client != NULL )
+	{
 		FreeSid( sid_client );
+		sid_client = NULL;
+	}
 
 	if( sid_server != NULL )
+	{
 		FreeSid( sid_server );
+		sid_server = NULL;
+	}
 
 	if( conn != INVALID_HANDLE_VALUE )
+	{
 		CloseHandle( conn );
-
-	conn = INVALID_HANDLE_VALUE;
+		conn = INVALID_HANDLE_VALUE;
+	}
 }
 
 long _ITH_IPCS::inbound( const char * path, IPCCONN & ipcconn )
@@ -1184,9 +1246,7 @@ long _ITH_IPCS::inbound( const char * path, IPCCONN & ipcconn )
 	{
 		case ERROR_IO_PENDING:
 		{
-			HANDLE events[ 2 ];
-			events[ 0 ] = hevent_conn;
-			events[ 1 ] = hevent_wake;
+			HANDLE events[ 2 ] = { hevent_conn, hevent_wake };
 
 			result = WaitForMultipleObjects(
 						2,
@@ -1287,6 +1347,17 @@ void _ITH_IPCC::io_conf( IPCCONN sconn )
 	conn = sconn;
 }
 
+long _ITH_IPCC::io_send( void * data, size_t size, size_t & sent )
+{
+	long result = send( conn, data, size, 0 );
+	if( result < 0 )
+		return IPCERR_FAILED;
+
+	sent = result;
+
+	return IPCERR_OK;
+}
+
 long _ITH_IPCC::io_recv( void * data, size_t size, size_t & rcvd )
 {
 	fd_set fds;
@@ -1326,17 +1397,6 @@ long _ITH_IPCC::io_recv( void * data, size_t size, size_t & rcvd )
 	return IPCERR_NODATA;
 }
 
-long _ITH_IPCC::io_send( void * data, size_t size, size_t & sent )
-{
-	long result = send( conn, data, size, 0 );
-	if( result < 0 )
-		return IPCERR_FAILED;
-
-	sent = result;
-
-	return IPCERR_OK;
-}
-
 long _ITH_IPCC::attach( const char * path, long timeout )
 {
 	conn = socket( AF_UNIX, SOCK_STREAM, 0 );
@@ -1366,7 +1426,7 @@ long _ITH_IPCC::attach( const char * path, long timeout )
 
 void _ITH_IPCC::wakeup()
 {
-	char c;
+	char c = 0;
 	send( conn_wake[ 1 ], &c, 1, 0 );
 }
 
@@ -1479,7 +1539,7 @@ long _ITH_IPCS::inbound( const char * path, IPCCONN & ipcconn )
 
 void _ITH_IPCS::wakeup()
 {
-	char c;
+	char c = 0;
 	send( conn_wake[ 1 ], &c, 1, 0 );
 }
 
