@@ -391,6 +391,9 @@ const char * _PFKI::name( long type, long value )
 		{
 			static const char * authtype_02 = "HMAC-MD5";
 			static const char * authtype_03 = "HMAC-SHA1";
+			static const char * authtype_04 = "HMAC-SHA2-256";
+			static const char * authtype_05 = "HMAC-SHA2-384";
+			static const char * authtype_06 = "HMAC-SHA2-512";
 
 			switch( value )
 			{
@@ -400,6 +403,15 @@ const char * _PFKI::name( long type, long value )
 				case SADB_AALG_SHA1HMAC:
 					return authtype_03;
 
+				case SADB_X_AALG_SHA2_256HMAC:
+					return authtype_04;
+
+				case SADB_X_AALG_SHA2_384HMAC:
+					return authtype_05;
+
+				case SADB_X_AALG_SHA2_512HMAC:
+					return authtype_06;
+
 				default:
 					return unknown;
 			}
@@ -756,6 +768,7 @@ long _PFKI::buff_set_key( sadb_key * ext, PFKI_KEY & key )
 		return IPCERR_FAILED;
 	}
 
+	assert( PFKI_MAX_KEYLEN >= key.length );
 	memcpy( buff, key.keydata, key.length );
 
 	return IPCERR_OK;
@@ -943,7 +956,16 @@ long _PFKI::send_sainfo( u_int8_t sadb_msg_type, PFKI_SAINFO & sainfo, bool serv
 {
 	PFKI_MSG msg;
 
+#if defined( OPT_NATT ) && defined( __APPLE__ )
+
+	sadb_sa_natt * xsa;
+
+#else
+
 	sadb_sa * xsa;
+
+#endif
+
 	sadb_x_sa2 * xsa2;
 	sadb_address *xas, *xad;
 	sadb_lifetime *xlh, *xls, *xlc;
@@ -971,6 +993,23 @@ long _PFKI::send_sainfo( u_int8_t sadb_msg_type, PFKI_SAINFO & sainfo, bool serv
 			if( ( sadb_msg_type == SADB_GETSPI ) && !serv )
 				break;
 
+#if defined( OPT_NATT ) && defined( __APPLE__ )
+
+			result = buff_add_ext( msg, ( sadb_ext ** ) &xsa, sizeof( sadb_sa_natt ) );
+			if( result != IPCERR_OK )
+				return result;
+
+			xsa->sa.sadb_sa_exttype = SADB_EXT_SA;
+			xsa->sa.sadb_sa_spi = sainfo.sa.spi;
+			xsa->sa.sadb_sa_replay = sainfo.sa.replay;
+			xsa->sa.sadb_sa_state = sainfo.sa.state;
+			xsa->sa.sadb_sa_auth = sainfo.sa.auth;
+			xsa->sa.sadb_sa_encrypt = sainfo.sa.encrypt;
+			xsa->sa.sadb_sa_flags = sainfo.sa.flags;
+			xsa->sadb_sa_natt_port = sainfo.sa.natt_port;
+
+#else
+
 			result = buff_add_ext( msg, ( sadb_ext ** ) &xsa, sizeof( sadb_sa ) );
 			if( result != IPCERR_OK )
 				return result;
@@ -983,6 +1022,8 @@ long _PFKI::send_sainfo( u_int8_t sadb_msg_type, PFKI_SAINFO & sainfo, bool serv
 			xsa->sadb_sa_encrypt = sainfo.sa.encrypt;
 			xsa->sadb_sa_flags = sainfo.sa.flags;
 
+#endif
+
 			break;
 	}
 
@@ -1184,7 +1225,7 @@ long _PFKI::send_sainfo( u_int8_t sadb_msg_type, PFKI_SAINFO & sainfo, bool serv
 			break;
 	}
 
-#ifdef OPT_NATT
+#if defined( OPT_NATT ) && !defined( __APPLE__ )
 
 	//
 	// natt extension
@@ -1591,11 +1632,10 @@ long _PFKI::read_address_dst( PFKI_MSG & msg, PFKI_ADDR & addr )
 	return IPCERR_OK;
 }
 
+#if defined( OPT_NATT ) && !defined( __APPLE__ )
+
 long _PFKI::read_natt( PFKI_MSG & msg, PFKI_NATT & natt )
 {
-
-#ifdef OPT_NATT
-
 	sadb_x_nat_t_type * xnt;
 	sadb_x_nat_t_port * xnps, * xnpd;
 
@@ -1620,15 +1660,10 @@ long _PFKI::read_natt( PFKI_MSG & msg, PFKI_NATT & natt )
 	natt.port_dst = xnpd->sadb_x_nat_t_port_port;
 
 	return IPCERR_OK;
-
-#else
-
-	return IPCERR_FAILED;
-
-#endif
-
 }
 
+# endif // OPT_NATT && !__APPLE__
+
 long _PFKI::read_policy( PFKI_MSG & msg, PFKI_SPINFO & spinfo )
 {
 	//
