@@ -82,6 +82,7 @@
 #endif
 
 #include "libidb.h"
+#include "libith.h"
 
 //
 // Win32 specific
@@ -111,7 +112,19 @@
 # define SADB_X_EALG_CAST128CBC	6
 #endif
 
-#ifndef __FreeBSD__
+#ifndef SADB_X_AALG_SHA2_256HMAC
+# define SADB_X_AALG_SHA2_256HMAC SADB_X_AALG_SHA2_256
+#endif
+
+#ifndef SADB_X_AALG_SHA2_384HMAC
+# define SADB_X_AALG_SHA2_384HMAC SADB_X_AALG_SHA2_384
+#endif
+
+#ifndef SADB_X_AALG_SHA2_512HMAC
+# define SADB_X_AALG_SHA2_512HMAC SADB_X_AALG_SHA2_512
+#endif
+
+# ifndef __FreeBSD__
 
 // Linux and NetBSD compat
 
@@ -130,15 +143,63 @@
 #define PFKEY_ADDR_SADDR(ext) \
         ((struct sockaddr *)((caddr_t)(ext) + sizeof(struct sadb_address)))
 
+# endif	// __FreeBSD__
+
+# ifdef __APPLE__
+
+# endif // __APPLE__
+
+#endif	// UNIX
+
+//
+// these socket options are required for NAT-T
+// but not always defined in userland headers
+//
+
+#ifdef OPT_NATT
+
+#ifndef SOL_UDP
+#define SOL_UDP 17
 #endif
 
+#ifndef UDP_ENCAP
+#define UDP_ENCAP 100
 #endif
 
+#ifndef UDP_ENCAP_ESPINUDP_NON_IKE
+#define UDP_ENCAP_ESPINUDP_NON_IKE 1
+#endif
+
+#ifndef UDP_ENCAP_ESPINUDP
+#define UDP_ENCAP_ESPINUDP 2
+#endif
+
+#ifdef __APPLE__
+
+#define SADB_X_EXT_NATT				2
+#define SADB_X_EXT_NATT_KEEPALIVE	4
+
+struct sadb_sa_natt
+{
+	struct sadb_sa	sa;
+	u_int16_t		sadb_sa_natt_port;
+	u_int16_t		sadb_reserved0;
+	u_int32_t		sadb_reserved1;
+};
+
+#endif
+
+#endif // OPT_NATT
+
+//
+// common to all platforms
+//
+
 #include <stdio.h>
 #include "export.h"
 
 #define PFKI_MAX_XFORMS		4
-#define PFKI_MAX_KEYLEN		32
+#define PFKI_MAX_KEYLEN		128
 
 #define NAME_MSGTYPE		1
 #define NAME_SATYPE			2
@@ -162,6 +223,12 @@ typedef struct _PFKI_SA
 	u_int8_t	encrypt;
 	u_int32_t	flags;
 
+#if defined( OPT_NATT ) && defined( __APPLE__ )
+
+	u_int16_t	natt_port;
+
+#endif
+
 }PFKI_SA;
 
 typedef struct _PFKI_SA2
@@ -218,6 +285,8 @@ typedef struct _PFKI_SP
 
 }PFKI_SP;
 
+#if defined( OPT_NATT ) && !defined( __APPLE__ )
+
 typedef struct _PFKI_NATT
 {
 	u_int8_t	type;
@@ -227,6 +296,8 @@ typedef struct _PFKI_NATT
 
 }PFKI_NATT;
 
+#endif
+
 typedef struct _PFKI_SAINFO
 {
 	u_int8_t	satype;
@@ -243,9 +314,14 @@ typedef struct _PFKI_SAINFO
 	PFKI_LTIME	ltime_soft;
 	PFKI_KEY	ekey;
 	PFKI_KEY	akey;
-	PFKI_NATT	natt;
 	PFKI_RANGE	range;
 
+#if defined( OPT_NATT ) && !defined( __APPLE__ )
+
+	PFKI_NATT	natt;
+
+#endif
+
 }PFKI_SAINFO;
 
 typedef struct _PFKI_XFORM
@@ -323,9 +399,14 @@ typedef class DLX _PFKI  : private _ITH_IPCC, public IDB_ENTRY
 	long	read_key_e( PFKI_MSG & msg, PFKI_KEY & ekey );
 	long	read_address_src( PFKI_MSG & msg, PFKI_ADDR & addr );
 	long	read_address_dst( PFKI_MSG & msg, PFKI_ADDR & addr );
-	long	read_natt( PFKI_MSG & msg, PFKI_NATT & natt );
 	long	read_policy( PFKI_MSG & msg, PFKI_SPINFO & spinfo );
 
+#if defined( OPT_NATT ) && !defined( __APPLE__ )
+
+	long	read_natt( PFKI_MSG & msg, PFKI_NATT & natt );
+
+#endif
+
 	const char *	name( long type, long value );
 
 	long	attach( long timeout );
