@@ -40,7 +40,7 @@
  */
 
 #define CLIENT_VER_MAJ		2
-#define CLIENT_VER_MIN		1
-#define CLIENT_VER_BLD		7
-#define CLIENT_YEAR			2010
-#define CLIENT_VER_CFG		2
+#define CLIENT_VER_MIN		2
+#define CLIENT_VER_BLD		1
+#define CLIENT_RELEASE		1
+#define CLIENT_YEAR			2013
